sketch <- function(data, file, epsilon = NULL, dim = NULL, warn = TRUE, affine=TRUE,
                   method= c("R", "S", "C"), ...)
{
	if (missing(data) & missing(file)) stop ("One of data or file has to be specified")
	if (!missing(file) & !missing(data)) warning("Both data and file specified, file ignored")
	if (!missing(data))
		if(!is.data.frame(data) && !is.matrix(data)) stop ("data must be a data.frame or matrix!")
	if (!missing(file) & missing(data))
	  {
	    if(!file.exists(file)) stop("File not found")
	    data = read.table(file, ...)
	  }
	if(!is.numeric(epsilon) && !is.numeric(dim)) stop ("One of epsilon and dim must be numeric!")
#	if(!is.logical(print)) stop ("print must be logical!")
	if(!is.logical(affine)) stop ("affine must be logical!")
	method = match.arg(method)
	if (is.numeric(epsilon) && (epsilon > 0.5 | epsilon <= 0)) stop("Possible values for epsilon lie in the interval (0, 0.5]")

	if (warn)
  {
    n <- dim(data)[1]
    d <- dim(data)[2] + affine
    if (is.numeric(epsilon))
  	{
      s = if (method == "R" | method == "S")
        as.integer(ceiling(d * log(d) / (epsilon*epsilon)))
      else if (method == "C")
        as.integer(2^ceiling(log(ceiling(d*d/(20*epsilon*epsilon)), base = 2)))
      if (s > n) warning("Sketch has more rows than original data set, consider using larger epsilon")
    }
    else
    {
      if (dim > n) warning("Sketch has more rows than original data set, consider using smaller dim")
    }
  }
    
	if (is.numeric(epsilon) && is.numeric(dim))
	  warning('Both epsilon and dim specified\nUsed epsilon only, dim was ignored')
	erg =
	  if(is.numeric(epsilon))
	    .Call("sketch", data + 0.0, epsilon + 0.0, affine, method, PACKAGE = "RaProR")
	  else
	    .Call("sketchDim", data + 0.0, dim, affine, method, PACKAGE = "RaProR")
	erg = as.data.frame(erg) #sapply(erg, matrix)
	if(!is.null(names(data)))
	  if (!affine) names(erg) <- names(data)
	  else names(erg) <- c('x0', names(data))

	return(erg)
}

setSeed <- function(seed)
{
	if(!is.numeric(seed)) stop ("seed must be numeric!")
	erg = .Call("setSeed", seed)
}

readinandsketch = function(file, nrows = 50000L, epsilon = NULL, dim = NULL,
                           affine = TRUE, method= c("C", "S", "R"), header = FALSE,
                           col.names, skip = 0, warn = FALSE, ...)
  {
	if(!file.exists(file)) stop("File not found")
	if(!is.numeric(epsilon) && !is.numeric(dim)) stop ("One of epsilon and dim must be numeric!")
#	if(!is.logical(print)) stop ("print must be logical!")
	if(!is.logical(affine)) stop ("affine must be logical!")
	method = match.arg(method)
	if (is.numeric(epsilon) && is.numeric(dim))
	  warning('Both epsilon and dim specified\nUsed epsilon only, dim was ignored')
  if (nrows <= 0) stop("nrows has to be a positive integer")

	epsgiven = is.numeric(epsilon)

    r1 = read.table(file = file, header = header, nrows = nrows, col.names = col.names,
      skip = skip, ...)
    skizze = if (epsgiven)
    	sketch(data = r1, epsilon = epsilon, warn = warn, affine = affine, method = method)
    else
    	sketch(data = r1, dim = dim, warn = warn, affine = affine, method = method)
    iter = 1
    if (dim(r1)[1] < nrows) return(skizze)
    repeat({
      rnew = read.table(file = file, header = FALSE, nrows = nrows, col.names = names(r1),
        skip = skip + header + iter * nrows, ...)
      iter = iter + 1
      skizze = skizze + 
      if (epsgiven)
    	sketch(data = r1, epsilon = epsilon, warn = warn, affine = affine, method = method)
      else
    	sketch(data = r1, dim = dim, warn = warn, affine = affine, method = method)
      if (dim(rnew)[1] < nrows) break
    })
    skizze
  }
