/** @file RInterface.cpp
 *  @brief Implementation of RInterface.h
 *  @author Jens Quedenfeld
 */

#ifndef NO_R

#include "sketch/BCH_Sketch.h"
#include "sketch/CW.h"
#include "RInterface.h"
#include "sketch/SRHT_direct.h"
#include "exp/Statistik.h"
#include <ctime>
#include <stdexcept>

using namespace RInterface;

// ===================================
// ===== UMLEITUNG VON STD::COUT =====
// ===================================
Rprintf_buf sb_Rprintf_cout;
Rprintf_buf sb_Rprintf_clog;
REprintf_buf sb_Rprintf_cerr;
SetRprintf_buf::SetRprintf_buf() {
    //std::cout.rdbuf( &sb_Rprintf_cout );
    //std::clog.rdbuf( &sb_Rprintf_clog );
    //std::cerr.rdbuf( &sb_Rprintf_cerr );
	useful::srand_(time(0)); // Zufallsstartwert setzen
}
#ifdef R_COUT_CHANGE
SetRprintf_buf sb_Rprintf_set;
#endif


// ========================================
// ===== UMWANDLUNGEN VON R ZU MATRIX =====
// ========================================

SEXP RInterface::mat2r_mat(Matrix<double> mat) {
	int n = mat.getDim1(); // Zeilen-Anzahl
	int m = mat.getDim2(); // Spalten-Anzahl

	SEXP rmat;
	PROTECT(rmat = allocMatrix(REALSXP, n, m));
	double* rdata = REAL(rmat);
	for (int i = 0; i < n; i++)
		for (int j = 0; j < m; j++)
			rdata[j*n+i] = mat(i,j);

	UNPROTECT(1);
	return rmat;
}

SEXP RInterface::mat2r_vec(Matrix<double> mat) {
	int n = mat.getDim1(); // Zeilen-Anzahl
	int m = mat.getDim2(); // Spalten-Anzahl

	SEXP rmat;
	PROTECT(rmat = allocVector(VECSXP, m));
	for (int j = 0; j < m; j++) {
		SEXP spalte;
		PROTECT(spalte = allocVector(REALSXP, n));
		double* sdata = REAL(spalte); // Daten der Spalte
		for (int i = 0; i < n; i++)
			sdata[i] = mat(i,j);
		SET_VECTOR_ELT(rmat, j, spalte);
	}

	UNPROTECT(m+1);
	return rmat;
}

Matrix<double> RInterface::r2mat(SEXP rmat, bool addAffineColumn) {
	switch (TYPEOF(rmat)) {
	case INTSXP: {
		SEXP dim = getAttrib(rmat, R_DimSymbol);
		int n, m;
		if (!isNull(dim)) {
			n = INTEGER(dim)[0]; // Anzahl der Zeilen
			m = INTEGER(dim)[1]; // Anzahl der Spalten
		} else {
			n = 1;
			m = LENGTH(rmat);
		}

		Matrix <double> mat(n, m + (addAffineColumn ? 1 : 0));
		if (addAffineColumn) {
			for (int i = 0; i < n; i++) {
				mat(i, 0) = 1;
				for (int j = 0; j < m; j++)
					mat(i, j+1) = INTEGER(rmat)[j*n + i];
			}
		} else {
			for (int i = 0; i < n; i++) {
				for (int j = 0; j < m; j++)
					mat(i, j) = INTEGER(rmat)[j*n + i];
			}
		}

		return mat;
	}
	case REALSXP: {
		SEXP dim = getAttrib(rmat, R_DimSymbol);
		int n, m;
		if (!isNull(dim)) {
			n = INTEGER(dim)[0]; // Anzahl der Zeilen
			m = INTEGER(dim)[1]; // Anzahl der Spalten
		} else {
			n = 1;
			m = LENGTH(rmat);
		}

		Matrix <double> mat(n, m + (addAffineColumn ? 1 : 0));
		if (addAffineColumn) {
			for (int i = 0; i < n; i++) {
				mat(i, 0) = 1;
				for (int j = 0; j < m; j++)
					mat(i, j+1) = REAL(rmat)[j*n + i];
			}
		} else {
			for (int i = 0; i < n; i++) {
				for (int j = 0; j < m; j++)
					mat(i, j) = REAL(rmat)[j*n + i];
			}
		}

		return mat;
	}
	case VECSXP: {
		int m = length(rmat); // Anzahl der Spalten
		int n = length(VECTOR_ELT(rmat, 0)); // Anzahl der Zeilen (der 1. Spalte)

		Matrix <double> mat(n, m + (addAffineColumn ? 1 : 0));
		if (addAffineColumn) {
			for (int i = 0; i < n; i++) {
				mat(i, 0) = 1;
				for (int j = 0; j < m; j++)
					mat(i, j+1) = REAL(VECTOR_ELT(rmat, j))[i];
			}
		} else {
			for (int i = 0; i < n; i++)
				for (int j = 0; j < m; j++)
					mat(i, j) = REAL(VECTOR_ELT(rmat, j))[i];
		}

		return mat;
	}
	default:
		return Matrix<double>();
	}

}


// ============================================
// ===== VON R AUS AUSF�HRBARE FUNKTIONEN =====
// ============================================

#ifdef __cplusplus
extern "C" {
#endif
SEXP setSeed(SEXP r_seed) {
    try {
        unsigned seed;
        if (isReal(r_seed))
            seed = (unsigned) *REAL(r_seed);
        else if (isInteger(r_seed))
            seed = *INTEGER(r_seed);
        else error_return("RaProR: RInterface: setSeed: seed has to be numeric");
        useful::srand_(seed);
        return R_NilValue;
    } catch (std::exception& e) {
        error_return(e.what());
    } catch (...) {
        error_return("RaProR: unknwon error");
    }
}

SEXP sketch(SEXP r_data, SEXP r_eps, SEXP r_affine, SEXP r_method) {
    try {
        // Parameter umwandeln
        double eps;
        if (isReal(r_eps))
            eps = *REAL(r_eps);
        else if (isInteger(r_eps))
            eps = *INTEGER(r_eps);
        else error_return("RaProR: RInterface: skizze: epsilon has to be numeric");

    //	bool print = false;
    //	if (isLogical(r_print)) print = *LOGICAL(r_print);

        bool affine = false;
        if (isLogical(r_affine)) affine = *LOGICAL(r_affine);

        char method = 'R';
        if (isString(r_method)) method = *CHAR(Rf_asChar(r_method));

        Matrix<double> data = r2mat(r_data, affine);
        if (data.isEmpty()) error_return("RaProR: RInterface: r2mat: unknown matrix format");

        int n = data.getDim1();
        int d = data.getDim2();
        Matrix<double> sketch;
        switch (method) {
        case 'R': {
            long long t0 = clock();
            sketch = statistik::embed<BCH_Sketch>(data, eps); //(data, eps, print);
    //		sketch.swap(sa.A);
            Rprintf("Time: %d ms\n", clock() - t0);
            break;
        }
        case 'S': {
            long long t0 = clock();
            SRHT_direct<double> srht(n, BCH_Sketch<double>::calcDim_Reduced(eps, d));
            sketch = srht.calc(data);
            Rprintf("Time: %d ms\n", clock() - t0);
            break;
        }
        case 'C': {
            long long t0 = clock();
            int k = (int) ceil(d * d / (20 * eps * eps));
            CW<double> cw(k, d);
            cw.updateMatrix(data);
            sketch = cw.getSketch_normalized();
            Rprintf("Time: %d ms\n", clock() - t0);
            break;
        }
        default:
            error_return("RaProR: RInterface: method: unknown sketching method");
        }

        return mat2r_mat(sketch);
    } catch (std::exception& e) {
        error_return(e.what());
    } catch (...) {
        error_return("RaProR: Unknwon error");
    }
}

SEXP sketchDim(SEXP r_data, SEXP r_dim, SEXP r_affine, SEXP r_method) {
    try {
        // Parameter umwandeln
        int k;
        if (isReal(r_dim))
            k = *REAL(r_dim);
        else if (isInteger(r_dim))
            k = *INTEGER(r_dim);
        else error_return("RaProR: RInterface: skizze: epsilon has to be numeric");

    //    bool print = false;
    //    if (isLogical(r_print)) print = *LOGICAL(r_print);

        bool affine = false;
        if (isLogical(r_affine)) affine = *LOGICAL(r_affine);

        char method = 'R';
        if (isString(r_method)) method = *CHAR(Rf_asChar(r_method));

        Matrix<double> data = r2mat(r_data, affine);
        if (data.isEmpty()) error_return("RaProR: RInterface: r2mat: unknown matrix format");

        int n = data.getDim1();
        int d = data.getDim2();
        Matrix<double> sketch;
        switch (method) {
        case 'R': {
            long long t0 = clock();
            sketch = statistik::embed<BCH_Sketch>(data, k); //(data, eps, print);
    //		sketch.swap(sa.A);
            Rprintf("Time: %d ms\n", clock() - t0);
            break;
        }
        case 'S': {
            long long t0 = clock();
            SRHT_direct<double> srht(n, k);
            sketch = srht.calc(data);
            Rprintf("Time: %d ms\n", clock() - t0);
            break;
        }
        case 'C': {
            long long t0 = clock();
            CW<double> cw(k, d);
            cw.updateMatrix(data);
            sketch = cw.getSketch_normalized();
            Rprintf("Time: %d ms\n", clock() - t0);
            break;
        }
        default:
            error_return("RaProR: RInterface: method: unknwon sketching method");
        }

        return mat2r_mat(sketch);
    } catch (std::exception& e) {
        error_return(e.what());
    } catch (...) {
        error_return("RaProR: unknwon error");
    }
}

#ifdef __cplusplus
}
#endif

#endif // NO_R
