/** @file RInterface.h
 * 	@brief Interface between R and this library.
 * 	@details If the macro \a NO_R is defined, this compilation unit is not compiled. \n
 * 		Only if the macro \a R_COUT_CHANGE is defined, the standard output stream is redirected to the R output.
 *  @author Jens Quedenfeld
 */

#ifndef RINTERFACE_H_
#define RINTERFACE_H_

#ifndef NO_R

#include "mat/matrix.h"
#include "R.h"
#include "Rinternals.h"
#include <vector>


// ===================================
// ===== UMLEITUNG VON STD::COUT =====
// ===================================

/** @name Classes for redirect std::cout to the standard output of R */
/** @{ */

	/** @brief Stream buffer that writes to Rprintf()
	 *  @details This stream buffer writes its input to the standard output stream of R. */
	class Rprintf_buf : public std::streambuf {
	protected:
		/** @brief Overwritten Method for writing to the R output
		 *  @details
		 *  	This method writes exactly one character to Rprintf(). */
		virtual int_type overflow( traits_type::int_type m = traits_type::eof()) {
			if( !traits_type::eq_int_type( m, traits_type::eof() ) )
				Rprintf( "%c", traits_type::to_char_type( m ) ); // genau ein Zeichen auf Rprintf ausgeben
			return traits_type::not_eof( m ); // Ok
		}
	};

    /** @brief Stream buffer that writes to REprintf()
     *  @details This stream buffer writes its input to the standard error stream of R. */
    class REprintf_buf : public std::streambuf {
    protected:
        /** @brief Overwritten Method for writing to the R output
         *  @details
         *  	This method writes exactly one character to REprintf(). */
        virtual int_type overflow( traits_type::int_type m = traits_type::eof()) {
            if( !traits_type::eq_int_type( m, traits_type::eof() ) )
                REprintf( "%c", traits_type::to_char_type( m ) ); // genau ein Zeichen auf REprintf ausgeben
            return traits_type::not_eof( m ); // Ok
        }
    };


	/** @brief Initialization of the redirection.
	 *  @details The constructor of this class is used to redirect std::cout to the standard output of R.
	 *  	If this library is compiled for R, then an object of this class is created. */
	class SetRprintf_buf {
	public:
		/** @brief Initialization of the redirection.
		 *  @details This method redirects std::cout to the standard output of R. */
		SetRprintf_buf();
	};

/** @} */



// ========================================
// ===== UMWANDLUNGEN VON R ZU MATRIX =====
// ========================================

/** @brief Matrix conversion between R and this library.
 *
 *  @details This namespace contains functions for converting matrices between the intern format of R and the format used by this library. */
namespace RInterface {
	/** @brief Converts an matrix to an R matrix.
	 *  @param mat
	 *  	The matrix that will be converted.
	 *  @returns
	 *  	an R matrix with the same content as the passed matrix.
	 *  @see mat2r_vec, r2mat */
	SEXP mat2r_mat(Matrix<double> mat);

	/** @brief Converts an matrix to an R vector matrix.
	 *  @param mat
	 *  	The matrix that will be converted.
	 *  @returns
	 *  	an R vector matrix with the same content as the passed matrix.
	 *  @see mat2r_mat, r2mat */
	SEXP mat2r_vec(Matrix<double> mat);

	/** @brief Converts an R matrix to a matrix of this library.
	 *  @param rmat
	 *  	The matrix that will be converted.
	 *  @param addAffineColumn
	 *  	If true, a column with ones is added to the left of the matrix.
	 *  @returns
	 *  	a matrix with the same content as the passed matrix.
	 *  @see mat2r_mat, mat2r_vec */
	Matrix<double> r2mat(SEXP rmat, bool addAffineColumn = false);
}

#ifdef __cplusplus
extern "C" {
#endif
// ============================================
// ===== VON R AUS AUSF�HRBARE FUNKTIONEN =====
// ============================================

/** @name Functions that can be called from R */
/** @{ */

	/** @brief Calculates an embedding.
	 * 	@details the number of rows of the input matrix is reduced.
	 *  @param data
	 *  	The matrix that will be embedded. \n
	 *  	The parameter must be an matrix or vector matrix
	 *  @param eps
	 *  	The quality of the embedding. The value must be between zero and one. The result is an (1 + \a eps) approximation. \n
	 *  	The parameter must be numeric.
	 *  @param affine
	 *  	If true, an column with ones is added to the data matrix. \n
	 *  	The parameter must be a logical value.
	 *  @param method
	 *  	The used method. The following methods are supported: \n
     *  	'R': Slow embedding using Rademacher matrices and the BCH generator. \n
	 *  	'C': Very fast embedding (Clarkson and Woodruff) \n
	 *  	'S': Fast embedding using an randomized Hadamard transformation (Subsampled Randomized Hadamard Transform (SRHT)) \n
	 *  	The parameter must be a character.
	 *  @returns
	 *  	the embedding matrix or an R error containing an message.
	 *  @see setSeed */
    SEXP sketch(SEXP data, SEXP eps, SEXP affine, SEXP method);

    /** @brief Calculates an embedding.
     * 	@details the number of rows of the input matrix is reduced.
     *  @param data
     *  	The matrix that will be embedded. \n
     *  	The parameter must be an matrix or vector matrix
     *  @param dim
     *  	The dimension of the resulting sketch. This value should be smaller than the primary data dimension. \n
     *  	The parameter must be numeric.
     *  @param affine
     *  	If true, an column with ones is added to the data matrix. \n
     *  	The parameter must be a logical value.
     *  @param method
     *  	The used method. The following methods are supported: \n
     *  	'R': Slow embedding using Rademacher matrices and the BCH generator. \n
     *  	'C': Very fast embedding (Clarkson and Woodruff) \n
     *  	'S': Fast embedding using an randomized Hadamard transformation (Subsampled Randomized Hadamard Transform (SRHT)) \n
     *  	The parameter must be a character.
     *  @returns
     *  	the embedding matrix or an R error containing an message.
     *  @see setSeed */
    SEXP sketchDim(SEXP data, SEXP dim, SEXP affine, SEXP method);

	/** @brief Sets the seed.
	 *  @param seed
	 *  	The new seed. The parameter must be numeric.
	 *  @returns
	 *  	\a R_NilValue or an R error if \a seed was not numeric.
	 *  @details You can use this method to create reproducible results.
	 *  	If a matrix is embedded serveral times, the results will be different each time.
	 *  	If you want to create the same results, call this function before the call of sketch.
	 *  @see sketch */
	SEXP setSeed(SEXP seed);

/** @} */



#ifdef __cplusplus
}
#endif

#endif /* NO_R */

#endif /* RINTERFACE_H_ */
