/** @file Statistik.h
 *  @brief Miscellaneous functions for data analysing.
 *  @author Jens Quedenfeld
 */
#ifndef STATISTIK_H_
#define STATISTIK_H_

#include "sketch/BCH_Sketch.h"
#include "mat/matrix.h"
#include <vector>
#include <string>

/** @brief Several function for data analysing.
 *  @details This namespace contains miscellaneous functions
 *  	to embed matrices, to solve a system of linear equations,
 *  	to compare solutions for the linear systems and other
 *  	functions connected to this subject area. Furthermore
 *  	this namespace contains a Weighted Sampler. */
namespace statistik {
	// == STRUCTS ==

		/** @brief System of linear equations
		 *  @details This structure contains a matrix \a A and a vector \a b.
		 *  	They represent the equation <em> Ax - b = 0 </em>, where \a x is the solution.
		 *  @see Readed, Analysed */
		struct Base {
			/** @brief Time for creating or reading the data
			 *  @details This variable contains the time duration it takes to create or read the matrices {@link A A} and {@link b b}.
			 *  	If the time duration is unknwon, then this variable is zero or uninitialized. The use of this attribute is optional. */
			long long time;

			/** @brief The matrix A of the linear system
			 *  @details The first dimension of this matrix is equal to the first dimension of {@link b b}. */
			Matrix<double> A;

			/** @brief The vector b of the linear system
			 *  @details The first dimension of this vector is equal to the first dimension of {@link A A}.
			 *  	The second dimension is usually one. */
			Matrix<double> b;
		};

		/** @brief Data readed from a file
		 *  @details This structure extends the structure {@link Base Base}.
		 *  	In addition to the attributes of the superclass,
		 *  	it contains a possible solution and the name of the file or data set.
		 *  @see Base, read, read_slow, Analysed, run */
		struct Readed : Base {
			/** @brief The name of the file or data set
			 *  @details If the file name is unknown, this variable contains an empty string. */
			std::string name;

			/** @brief A possible solution.
			 *  @details The real solution of the measured equation system. This solution need not
			 *  	to be the best solution! This variable is sometimes called "beta data".
			 *  	If the readed data contains no such vector, then this variable is a default constructed (0 x 0)-matrix. */
			Matrix<double> x; //betadaten
		};

		/** @brief Analysed linear system
		 *  @details This structure contains the original system of linear equations (matrix \a A and vector \a b), the
		 *  	best solution (vector \a mean), the covariant matrix (matrix \a var) and its eigenvalues and vectors.
		 *  @see Base, analyse(const Base&), analyse(const Matrix<double>&, const Matrix<double>&, long long), analyse(const Matrix<double>&), Epsilon */
		struct Analysed {
			/** @brief Time duration for creating the input data (A, b). */
			long long time_base;

			/** @brief Time duration for analysing the data. */
			long long time_analysis;

			/** @brief The matrix A of the linear system
			 *  @details The first dimension of this matrix is equal to the first dimension of {@link b b}. */
			Matrix<double> A;

			/** @brief The vector b of the linear system
			 *  @details The first dimension of this vector is equal to the first dimension of {@link A A}.
			 *  	The second dimension is usually one. */
			Matrix<double> b;

			/** @brief Covariant matrix
			 *  @details This matrix is the pseudo inverse of <em> (A^T * A) </em>, where <em> A^T </em> is the transpose matrix of \a A.
			 *  @see A */
			Matrix<double> var;

			/** @brief Optimal solution
			 * 	@details This vector is also called \a x_opt. The L2-Norm of <em>(A*x - b) </em> is minimal for <em> x = mean </em>.
			 * 		 \a mean is equal to <em> var * A^T * b </em>.
			 * 	@see A, b, var */
			Matrix<double> mean;

			/** @brief Square roots of the eigen values of the covariant matrix
			 * 	@details This vector contains the square roots of the eigen values of {@link var var}.
			 * 	@see var, eigenVectors */
			Matrix<double> eigenValues;

			/** @brief Eigen vectors of the covariant matrix
			 *  @details This matrix contains the eigen vectors of {@link var var}. The order fits to the order of {@link eigenValues eigenValues}
			 *  @see var, eigenValues */
			Matrix<double> eigenVectors;
		};

		/** @brief Comparison between {@link Analysed Analysed} structures.
		 *  @details This structure contains different evaluation criteria for an approximate soultion.
		 *  	The input data is usually compared to an embedding of this data. Both the original and the embedded data
		 *  	are analysed via the function \a analyse. Afterwards, you can compare both \a Analysed objects with the
		 *  	function \a calaEps. The result is an object of this structure.
		 *  @see Analaysed, calcEps, compare, analyse */
		struct Epsilon {
			/** @brief Row vector with the angles between the eigen vectors of the original an embedded data (order: not changed)
			 *  @details The entries of this vector contains the cosinus of the anlges.
			 *  	See {@link Analysed::eigenVectors Analysed::eigenVectors} for more information about the eigen vectors.
			 *  	The order of the eigen vectors is not changed, when you use this method.
			 *  @see Analysed::eigenVectors, angles_greedy, angles_sorted  */
			Matrix<double> angles_original;

			/** @brief Row vector with the angles between the eigen vectors of the original an embedded data (order: greedy algorithm)
			 *  @details The entries of this vector contains the cosinus of the anlges.
			 *  	See {@link Analysed::eigenVectors Analysed::eigenVectors} for more information about the eigen vectors.
			 *  	The angle between all eigen vectors of the original data and all eigen vectors of the embedded data is calculated.
			 *  	Then a greedy algorithm takes the smallest angle and marks the corresponding vectors. The algorithms repeats this step with all unmarked vectors,
			 *  	until all vector are marked. This row vector contains the taken angles.
			 *  @see Analysed::eigenVectors, angles_original, angles_sorted  */
			Matrix<double> angles_greedy;

			/** @brief Row vector with the angles between the eigen vectors of the original an embedded data (order: sorted eigen values
			 *  @details The entries of this vector contains the cosinus of the anlges.
			 *  	See {@link Analysed::eigenVectors Analysed::eigenVectors} for more information about the eigen vectors.
			 *  	First, the eigen values (and the corresponding eigen vectors) of the original and embedded data
			 *  	are sorted. Then the angles are calculated.
			 *  @see Analysed::eigenVectors, angles_original, angles_greedy  */
			Matrix<double> angles_sorted;

			/** @brief Trace of the covariant matrix of the original linear system
			 *  @details The trace is the sum of the elements on the main diagonal.
			 *  	The covariant matrix is the pseudo inverse of <em> A^T * A </em>,
			 *  	where <em> A^T </em> denotes the transpose matrix of \a A.
			 *  @see trace_var_approx, trace_var_mix, trace_sum */
			double trace_var_exact;

			/** @brief Trace of the covariant matrix of the embedded linear system
			 *  @details The trace is the sum of the elements on the main diagonal.
			 *  	The covariant matrix is the pseudo inverse of <em> (SA)^T * SA </em>,
			 *  	where <em> M^T </em> denotes the transpose matrix of \a M.
			 *  	\a SA is the embedding of the matrix \a A.
			 *  @see trace_var_exact, trace_var_mix, trace_sum */
			double trace_var_approx;

			/** @brief Trace of the "mixed" covarint matrix
			 *  @details The trace is the sum of the elements on the main diagonal.
			 *  	Let \a A be the matrix of the original linear system <em> A*x - b = 0 </em>
			 *  	and let \a SA be the corresponding matrix of the embedded linear system.
			 *  	<em> M^T </em> denotes the transpose matrix of \a M.
			 *  	The pseudoinverse of <em> A^T * A </em> is called <em> M1 </em> and
			 *  	the pseudoinverse of <em> (SA)^T * (SA) </em> is called <em> M2 </em>.
			 *  	<em> M^.5 </em> denotes the "square root" of the matrix /a M. It is
			 *  	calculated via the Singular Value Decomposition (SVD): The matrix is decomposed into <em> U*S*V^T = M </em>.
			 *  	The entries of \a S are replaced with its respective square roots. The resulting matrix is called S'.
			 *  	Then <em> M^.5 </em> is equal to <em> U * S' * V^T </em>.
			 *  	The value of this variable is <em> [M1^.5 * M2 * M1^.5] ^ .5 </em>.
			 *  @see trace_var_exact, trace_var_approx, trace_sum */
			double trace_var_mix;

			/** @brief The three other trace attributes are calculated together in this variable.
			 *  @details The value of this variable is equal to
			 *  	<em> trace_var_exact + trace_var_approx - 2*trace_var_mix </em>
			 *  @see trace_var_exact, trace_var_approx, trace_var_mix */
			double trace_sum;

			/** @brief Distance between the optimal solutions
			 *  @details The Euclidean distance between
			 *  	the optimal solution of the original linear system and
			 *  	the optimal solution of the embedded linear system. */
			double mean_dist;

			/** @brief Error of the optimal solution
			 *  @details <em> err_exact = ||b - A*amean|| </em>, where \a amean is the best solution for the linear system <em> (A|b) </em>.
			 *  @see err_approx */
			double err_exact;

			/** @brief Error of the optimal solution of embedded data
			 *  @details <em> err_approx = ||Sb - SA*samean|| </em>, where \a samean is the best solution for the linear system <em> (SA|Sb) </em>.
			 *  @see err_exact */
			double err_approx;

			/** @brief Approximation of the error values in both linear systems
			 *  @details This epsilon is equal to <em> err_approx / err_exact - 1 </em>.
			 *  	So it is the approximation quality of the error values in the original and in embedded linear system.
			 *  @see err_exact, err_approx */
			double eps1;
			
			/** @brief Approximation of the error values in the original linear system
			 * 	@details This epsilon is equal to <em> ||b - A*samean|| / err_exact - 1 </em>,
			 * 		where \a samean is the approximated solution.
			 * 		So it is the approximation quality of the error values of the exact and the approximated solution in the original linear system.
			 * 	@see err_exact */
			double eps2;

			/** @brief Approximation of the distance between the solutions normalized by the error value and smallest singular value
			 *  @details Let <em> simga(M) </em> be the singular values of \a M.
			 *  	Let <em> min{v} </em> be the the minimum of the set v.
			 *  	Then this epsilon is equl to <em> ||amean - samean|| / err_exact * min{sigma(A)} </em>,
			 *  	where \a amean is the exact solution and \a samean is the approximated solution.
			 *  	So the value of this variable is the approximation quality
			 *  	of the euclidian distance between the exact and the approximated solution
			 *  	normalized by the error value for the exact solution and the smallest singular value of A. */
			double eps3;

			/** @brief Approximation of the distance between the exact and approximated solution.
			 * 	@details This epsilon is equal to <em> ||amean - samean|| / (2*||amean||) </em>,
			 * 		where \a amean is the exact solution and \a samean is the approximated solution.
			 * 		So it is the approximation quality of the euklidian distance between the exact and the approximated solution. */
			double eps4;

			/** @brief Approximation of the rotation between the original and embedded matrices.
			 *  @details This epsilon denotes the rotation between the matrix A and SA.
			 *  	This value is equal to <em> trace_sum / trace_var_exact </em>.
			 *  @see trace_var_exact, trace_var_approx, trace_var_mix, trace_sum */
			double eps5;
		};

	// == FILE ACCESS ==
		/** @brief Read data from a file (by reading it twice)
		 * 	@param file_Ab
		 *  	The path of a file containing the matrix \a A and the vector \a b.
		 *  @param file_x
		 *  	The path of a file conatining the real solution called "beta data" or NULL if there is no such vector available
		 *  @returns a structure containing the matrix \a A, the vectors \a b and \a x, the time to read the data and
		 *  	the name of the first file as a std::string.
		 *  @details The head line of \a file_Ab contains the column names of the matrix \a A,
		 *  	followed by the name of the column vector \a b and the name of the error column vector (that is ignored).
		 *  	The next lines contains the values of the data: First, a row of the matrix \a A, then one value of \a b and at
		 *  	the end the error value (that is ignored). The values are separated by space characters. \n\n
		 *  	The head line of \a file_x contains the column name of the vector \a x. The next lines
		 *  	contains the values of \a x (there is one value in each line). \n
		 *  	If the parameter file_x is NULL, then the matrix x is an default constructed (0 x 0)-matrix. \n\n
		 *  	The file is readed twice, because the number of rows is unknown before and during the first run. Therefore this
		 *  	function is slower than {@link read read}.
		 *  @see read, Readed, analyse, Base, write */
		Readed read_slow(const char* file_Ab, const char* file_x = NULL);

		/** @brief Read data from a file
		 *  @param file_Ab
		 *  	The path of a file containing the matrix \a A and the vector \a b.
		 *  @param file_x
		 *  	The path of a file conatining the real solution called "beta data" or NULL if there is no such vector available
		 *  @returns a structure containing the matrix \a A, the vectors \a b and \a x, the time to read the data and
		 *  	the name of the first file as a std::string.
		 *  @details The head line of \a file_Ab contains the column names of the matrix \a A,
		 *  	followed by the name of the column vector \a b and the name of the error column vector (that is ignored).
		 *  	The next lines contains the values of the data: First, a row of the matrix \a A, then one value of \a b and at
		 *  	the end the error value (that is ignored). The values are separated by space characters. \n\n
		 *  	The head line of \a file_x contains the column name of the vector \a x. The next lines
		 *  	contains the values of \a x (there is one value in each line). \n
		 *  	If the parameter file_x is NULL, then the matrix x is an default constructed (0 x 0)-matrix. \n\n
		 *  	The number of rows is unknown while the reading the data. Therefore the data is firstly written to a std::vector
		 *  	and then copied to a matrix. If there is no enough memory for this, you can use the function read_slow. It reads the
		 *  	data twice, however, it directly writes the data into a matrix, because it knows the first dimension due to the first
		 *  	reading opertaion.
		 *  @see read_slow, Readed, analyse, Base, write */
		Readed read(const char* file_Ab, const char* file_x = NULL);

		/** @brief Writes a {@link Base Base} object, so it can be readed by {@link read read} Schreiben von Daten
		 *  @param base
		 *  	The data that will be written
		 *  @param path
		 *  	The path of the file into which \a base is written.
		 *  @details The head line of the written file contains the column names of the matrix \a A,
		 *  	followed by the name of the column vector \a b and the name of the error column vector (that is ignored).
		 *  	The next lines contains the values of the data: First, a row of the matrix \a A, then one value of \a b and at
		 *  	the end the error value (that is ignored). The values are separated by space characters.
		 * base : Enth�lt die Matrix A und den Vektor b, die nebeneinander als Matrix in eine Datei geschrieben werden
		 * path : Dateipfad
		 *  @see read, read_slow, Base	 */
		void write(const Base& base, const char* path);

	// == SKETCH ==

		/** @brief Embeds a linear system with a specified quality
		 *  @param base
		 *  	System of linear equations.
		 *  @param eps
		 *  	The quality of the embedding.
		 *  @returns
		 *  	The embedded linear system.
		 *  @details Embeds a linear system using a specified method.
		 *  	The returned linear system is a (1 + \a eps) approximation.
		 *  @see {@link embed(const Base&, int) embed(const Base&, int)},
		 *  	 {@link embed(const Matrix<T>&, double) embed(const Matrix<T>&, double)},
		 *  	 {@link embed(const Matrix<T>&, int) embed(const Matrix<T>&, int)},
		 *  	 BCH_Sketch, BCH_Null, BCH_Small, SRHT, CW, KN */
		template <template <class T> class Sketch>
		statistik::Base embed(const statistik::Base& base, double eps);

		/** @brief Embeds a  linear system with a specified target dimension
		 *  @param base
		 *  	System of linear equations.
		 *  @param dim_reduced
		 *  	The number of equations of the returnd system.
		 *  @returns
		 *  	The embedded linear system.
		 *  @details Embeds a linear system using a specified method.
		 *  	The returned linear system has \a dim_reduced equations respectively rows.
		 *  @see {@link embed(const Base&, double) embed(const Base&, double)},
		 *  	 {@link embed(const Matrix<T>&, double) embed(const Matrix<T>&, double)},
		 *  	 {@link embed(const Matrix<T>&, int) embed(const Matrix<T>&, int)},
		 *  	 BCH_Sketch, BCH_Null, BCH_Small, SRHT, CW, KN */
		template <template <class T> class Sketch>
		statistik::Base embed(const statistik::Base& base, int dim_reduced);

		/** @brief Embeds a matrix with a specified quality
		 *  @param mat
		 *  	The matrix that will be embedded.
		 *  @param eps
		 *  	The quality of the embedding.
		 *  @returns
		 *  	The embedded matrix.
		 *  @details Embeds a matrix using a specified method.
		 *  	The returned matrix is a (1 + \a eps) approximation.
		 *  @see {@link embed(const Base&, double) embed(const Base&, double)},
		 *  	 {@link embed(const Base&, int) embed(const Base&, int)},
		 *  	 {@link embed(const Matrix<T>&, int) embed(const Matrix<T>&, int)},
		 *  	 BCH_Sketch, BCH_Null, BCH_Small, SRHT, CW, KN */
		template <template <class T> class Sketch, class T>
		Matrix<T> embed(const Matrix<T>& mat, double eps);

		/** @brief Embeds a matrix with a specified target dimension
		 *  @param mat
		 *  	The matrix that will be embedded.
		 *  @param dim_reduced
		 *  	The number of equations of the returnd system.
		 *  @returns
		 *  	The embedded matrix.
		 *  @details Embeds a matrix using a specified method.
		 *  	The returned matrix has \a dim_reduced rows.
		 *  @see {@link embed(const Base&, double) embed(const Base&, double)},
		 *  	 {@link embed(const Base&, int) embed(const Base&, int)},
		 *  	 {@link embed(const Matrix<T>&, double) embed(const Matrix<T>&, double)},
		 *  	 BCH_Sketch, BCH_Null, BCH_Small, SRHT, CW, KN */
		template <template <class T> class Sketch, class T>
		Matrix<T> embed(const Matrix<T>& mat, int dim_reduced);

		/** @brief Calculates the necessary number of rows for the embedding
		 *  @details If you have a linear equation system and you want to reduce the number of equations, you can
		 *  	use this function to calculate the number of equations of the embedded matrix.
		 *  @param eps
		 *  	The quality of the embedding.
		 *  @param dim_data
		 *  	The second dimension of the input data.
		 *  @returns the necessary number of rows of the embedded matrix for an (1 + \a eps) approximation. */
		int calcDim_Reduced(double eps, int dim_data);

	// == ANALYSIS ==

		/** @brief Analysis of a linear system
		 *  @param base
		 *  	The data that is analysed.
		 *  @returns the analysed data containing (among other things) the optimal solution and the covariant matrix.
		 *  @details See {@link Analysed Analysed} for more information.
		 *  @see Analysed, Base, analyse(const Matrix<double>&, const Matrix<double>&, long long), analyse(const Matrix<double>&) */
		Analysed analyse(const Base& base);

		/** @brief Analysis of a linear system
		 *  @param matA
		 *  	The matrix A.
		 *  @param matb
		 *  	The vector b.
		 *  @param time_base
	 	 *		Time for data creation (if unknown, use zero).
		 *  @returns the analysed linear system (<em> Ax - b = 0 </em>) containing (among other things) the optimal solution and the covariant matrix.
		 *  @details See {@link Analysed Analysed} for more information.
		 *  @see Analysed, Base, analyse(const Base&), analyse(const Matrix<double>&) */
		Analysed analyse(const Matrix<double>& matA, const Matrix<double>& matb, long long time_base = 0);

		/** @brief Analysis of a linear system
		 *  @param matA
		 *  	The matrix A.
		 *  @returns the analysed linear system (<em> Ax = 0 </em>) containing (among other things) the optimal solution and the covariant matrix.
		 *  @details See {@link Analysed Analysed} for more information.
		 *  @see Analysed, Base, analyse(const Base&), analyse(const Matrix<double>&, const Matrix<double>&, long long) */
		Analysed analyse(const Matrix<double>& matA);

		/** @brief Calculation of different evaluation criteria for an embedding
		 *  @param exact
		 *  	Analysed original data.
		 *  @param sketch
		 *  	Analysed embedded data.
		 *  @returns a structure containing different evaluation criteria
		 *  @details See {@link Epsilon Epsilon} for more information about the returned criteria.
		 *  	If you have the oritinal and embedded data stored as {@link Base Base} object called \a data respectively \a embedding,
		 *  	then you can calculate the corresponding {@link Epsilon Epsilon} object by calling <em> calcEps(analyse(data), analyse(embedding)) </em>
		 *  @see Analysed, Epsilon, compare */
		Epsilon calcEps(const Analysed& exact, const Analysed& sketch);

	// == OTHERS ==

		/* Implementierung von
		 * "Aprroximating the diagonal statistical leverage scores" */

		/** @brief Approximation of the diagonal statistical leverage scores
		 *  @param matA
		 *  	The matrix which leverage scores will be calculated
		 *  @param eps
		 *  	The quality of the approximation
		 *  @returns the approximated leverage scores.
		 *  @details The returned leverage scores are a (1 + \a eps)-approximation of the real leverage scores.
		 *  @see leverageScores_exact */
		std::vector<double> leverageScores_approx(const Matrix<double>& matA, double eps);

		/** @brief Exact diagonal statistical leverage scores
		 *  @param matA
		 *  	The matrix which leverage scores will be calculated
		 *  @returns the leverage scores of the matrix.
		 *  @see leverageScores_approx */
		std::vector<double> leverageScores_exact(const Matrix<double>& matA);

		/** @brief Comparision between two Analysed objects.
		 *  @param exakt
		 *  	The analysis of the original linear system.
		 *  @param sketch
		 *  	The analysis of the embedded linear system.
		 *  @param x
		 *  	The "real" solution of the linear system, also called "beta data".
		 *  @details Compares the solution of the embedded linear system with
		 *  	the solution of the original linear system and prints the results.
		 *  	The output shows the information saved in an Epsilon object.
		 *  @see analyse, Analysed, calcEps, Epsilon */
		void compare(const Analysed& exakt, const Analysed& sketch, const Matrix<double>& x = Matrix<double>::EMPTY);

		/** @brief Analysis of readed data and its embedding
		 *  @param rd
		 *  	The readed linear system.
		 *  @param eps
		 *  	Quality of the embedding. For each entry of this vector an embedding is calculated.
		 *  @param sa_path
		 *  	The embedded linear systems are saved at this path. If the vector contains no strings, then
		 *  	no file is written.
		 *  @param printSA
		 *  	If true, the progress of the embedding is printed.
		 *  @details The passed data is embedded several times (one time for each entry in \a eps).
		 *  	The embeddings are saved, if an entry with the fitting index in \a sa_path exists.
		 *  	The information printed to the standard output stream are the same printed by {@link compare compare}.
		 *  	For each embedding several evaluation criteria are printed.
		 *  @see compare, Readed, Optional */
		void run(const Readed& rd, const std::vector<double>& eps, const std::vector<std::string>& sa_path, bool printSA = true);

		/** @brief Approximated solution using a weighted sampler
		 *  @param matA
		 *  	The matrix A of the linear system <em> A*x - b = 0 </em>
		 *  @param matb
		 *  	The vector b of the lienar system <em> A*x - b = 0 </em>
		 *  @param levScores
		 *  	The leverage scores (exact or approximated) of \a matA
		 *  @param eps
		 *  	The quality of the weighted sampler
		 *  @returns
		 *  	an approximated solution \a x of the lienar system <em> A*x - b =  0 </em>
		 *  @see leverageScores_exact, leverageScores_approx */
		Matrix<double> underConstrainedRegression(const Matrix<double>& matA, const Matrix<double>& matb, std::vector<double> levScores, double eps);

	// == KLASSEN ==

		/** @brief Weighted Sampler
		 *  @details A weighted sampler selects a specified number of rows from a matrix.
		 *  	The propability to choose a row is equal to its weight.
		 *  	Usually the exact or approximated leverage scores are used as weights. */
		class WeightedSampler {
		public:
			/** @brief Internal structure storing an index and the corresponding weight */
			struct Entry {
				/** @brief Row index of the original matrix */
				int index;

				/** @brief Weight of the respective row */
				double weight;

				/** @brief Default construtur
				 *  @details The index is set to negative value and the weight is set to zero. */
				Entry() :
					index(-1),
					weight(0) {
				}

				/** @brief Standard constructor
				 *  @param index
				 *  	The row index.
				 *  @param weight
				 *  	The corresponding weight */
				Entry(int index, double weight) :
					index(index),
					weight(weight) {
				}
			};


			/** @brief Weighted Sampler containing k entries.
			 *  @param k
			 *  	The number of entries the sampler can store. */
			WeightedSampler(int k);

			/** @brief Weighted Sampler with a specified quality
			 *  @param eps
			 *  	The quality of the sampler
			 *  @param d
			 *  	The number of columns of the sampled matrix.
			 *  @details The number of entries \a k is calculated such that the Weighted Sampler will create
			 *  	a (1 + \a eps) approximation of the data set. */
			WeightedSampler(double eps, int d);

			/** @brief Sampling a new rows
			 *  @param weight
			 *  	The weight of the next row.
			 *  @details The leverage score of the next rows is usually passed to this method.
			 *  @see nextWeights */
			void nextWeight(double weight);

			/** @brief Sampling several new rows
			 *  @param weights
			 *  	Weights of the next rows.
			 *  @details The leverage scores of the next rows are usually passed to this method.
			 *  @see nextWeight */
			void nextWeights(const std::vector<double>& weights);

			/** @brief Returns the choosen row indices of the data and the corresponding weights.
			 *  @returns a vector containing the choosen row indices and weights.
			 *  @see getSampledMatrix(const Matrix<T>&), getSampledMatrix(const Base&, long long) */
			std::vector<Entry> getResult() const;

			/** @brief Creates a result matrix
			 *  @param mat
			 *  	The data set which weights were already passed to the sampler.
			 *  @returns a matrix which containing the rows selected by this sampler. */
			template <class T>
			Matrix<T> getSampledMatrix(const Matrix<T>& mat) const;

			/** @brief Creates a approximation of a linear system
			 *  @param base
			 *  	The linear system which weights were already passed to the sampler.
			 *  @param time
			 *  	The returned object contains the time for creating the sampled data plus this parameter.
			 *  @returns a approximation of \a base containing the rows selected by this sampler */
			Base getSampledMatrix(const Base& base, long long time = 0) const;

		private:
			/** @brief Current row index
			 *  @details The value of this variable is equal to the number of {@link nextWeight nextWeight} calls. */
			int currentIndex_;

			/** @brief Sum of the previous weights
			 *  @details The sum of the weights passed with every {@link nextWeight nextWeight} call */
			double weightSum_;

			/** @brief Choosen row indices and weights
			 *  @details This vector contains the choosen row indices and the corresponding weights. */
			std::vector<Entry> choosen_;
		};

}

// ===========================
// ===== IMPLEMENTIERUNG =====
// ===========================

template<class T>
Matrix<T> statistik::WeightedSampler::getSampledMatrix(const Matrix<T>& a) const {
	int k = choosen_.size();
	int d = a.getDim2();
	Matrix<T> erg(k, d);
	std::vector<Entry> choosen = getResult();
	for (int i = 0; i < k; i++)
		for (int j = 0; j < d; j++)
			if (choosen[i].index >= 0)
				erg(i, j) = a(choosen[i].index, j) / sqrt(k * choosen[i].weight / weightSum_);
			else
				erg(i, j) = 0;
	return erg;
}


template <template <class T> class Sketch>
statistik::Base statistik::embed(const statistik::Base& base, double eps) {
	int d = base.A.getDim2();
	Base result;
	Sketch<double> sk(eps, d + 1);
	sk.updateColumns(base.A);
	sk.updateColumns(base.b, d);
	Matrix<double> sab = sk.getSketch_normalized();
	int k = sab.getDim1();
	result.A = sab.select(0, 0, k, d);
	result.b = sab.select(0, d, k, 1);
	return result;
}

template <template <class T> class Sketch>
statistik::Base statistik::embed(const statistik::Base& base, int dim_reduced) {
	int d = base.A.getDim2();
	Base result;
	Sketch<double> sk(dim_reduced, d + 1);
	sk.updateColumns(base.A);
	sk.updateColumns(base.b, d);
	Matrix<double> sab = sk.getSketch_normalized();
	int k = sab.getDim1();
	result.A = sab.select(0, 0, k, d);
	result.b = sab.select(0, d, k, 1);
	return result;
}

template <template <class T> class Sketch, class T>
Matrix<T> statistik::embed(const Matrix<T>& mat, double eps) {
	Sketch<T> sk(eps, mat.getDim2());
	sk.updateAll(mat);
	return sk.getSketch_normalized();
}

template <template <class T> class Sketch, class T>
Matrix<T> statistik::embed(const Matrix<T>& mat, int dim_reduced) {
	Sketch<T> sk(dim_reduced, mat.getDim2());
	sk.updateAll(mat);
	return sk.getSketch_normalized();
}




#endif /* STATISTIK_H_ */
