/** @file matrix.h
 *  @brief Matrix class with several useful methods
 *  @author Jens Quedenfeld
 */

#ifndef MATRIX_H_
#define MATRIX_H_

template<class T = double>
class Matrix;

#include "util/Useful.h"
#ifndef DONT_USE_LAPACK
#include "include/lapack/cpplapack.h"
#endif
#include <fstream>
#include <iostream>
#include <iterator>
#include <stdexcept>
#include <utility>
#include <vector>

/** @brief Structure for saving the Singular Value Decomposition (SVD) of a matrix
 *
 *  A matrix A is decomposed into the matrices U, S and V so that A = U*S*V^T */
template<class T = double>
struct svdData {
	/** @brief The first template parameter (T) */
	typedef T value_type;

	/** @brief Singular values (downwards sorted)
	 *  @details The matrix S is a diagonal matrix with the values s[1], s[2],... on its diagonal */
	std::vector<T> s;

	/** @brief Orthogonal matrix U */
	Matrix <T> u;

	/** @brief The transpone of the orthogonal matrix V */
	Matrix <T> vt;

	/** @brief Calculation of U*S*V^T
	 *  @returns the original matrix A = U*S*V^T
	 *  @attention Due to rounding errors the result is not exact equal to the original matrix. */
	Matrix <T> usvt() {
		return u.multiplyWithDiagonal(s) * vt;
	}
};

/** @brief Structure for saving the eigen values and vectors
 *  @attention Complex eigen values cannot be saved if T = double. */
template <class T = double>
struct eigenData {
	/** @brief The first template parameter (T) */
	typedef T value_type;

	/** @brief Eigen values (not sorted)
	 *  @attention If T = double, complex eigen values cannot be saved. */
	std::vector<T> values;

	/** @brief Eigen vectors
	 *  @details Each column contains one vector.
	 *  	The i-th column belongs to the i-th eigen value in \a values. */
	Matrix<T> vectors;

	/** @brief Eigen values as (1 x n)-matrix.
	 *  @returns a matrix containing the eigen values as row vector. */
	Matrix<T> getValues() {
		return Matrix<T>::fromVectorRow(values);
	}
};

/** @brief Matrix class with several useful methods.
 *
 *  A matrix consists of a two-dimensional array.
 *  More precisely the main array entries points to arrays
 *  that contains the data of the matrix.
 *  Matrices can be transposed implicitly.
 *
 *  See {@link operators here} for the supported operators.*/
template<class T>
class Matrix {
public:
	/** @brief The first template parameter (T) */
	typedef T value_type;

	// == EXCEPTIONS ==
	/** @name Exception classes */
	/** @{ */

		/** @brief Exception denoting an unfitting dimension.
		 *  @details This exception is thrown when the dimension of the matrices passed in a method does not fit togehter. \n
		 *  @par Examples
		 *  	If you try to multiply a (3 x 4)-matrix with a (5 x 6)-matrix, then this exception is probably thrown */
		class dim_exception : public std::invalid_argument {
		public:
			/** @brief Constructor
			 *  @param str
			 *  	Calling the method \a what() will return this string. */
			dim_exception(const std::string& str = "") : std::invalid_argument(str) {};
		};

		/** @brief Exception denoting a non-square matrix.
		 *  @details This exception is thrown when a method is called that requires a square matrix, however, the passed matrix is not square. */
		class notSquare_exception : public std::invalid_argument {
		public:
			/** @brief Constructor
			 *  @param str
			 *  	Calling the method \a what() will return this string. */
			notSquare_exception(const std::string& str = "") : std::invalid_argument(str) {};
		};

		/** @brief Exception denoting that a matrix is not positive definit.
		 *  @details This exception is thrown when the called method requires a positive definit matrix, but the passed matrix is not positive definit. */
		class notPositiveDefinit : public std::runtime_error {
		public:
			/** @brief Constructor
			 *  @param str
			 *  	Calling the method \a what() will return this string. */
			notPositiveDefinit(const std::string& str = "") : std::runtime_error(str) {};
		};

		/** @brief Exception denoting that a matrix cannot be inverted.
		 *  @details This exception is thrown when the called method requires a invertible matrix, however, the passed matrix is invertible. */
		class notInvertible : public std::runtime_error {
		public:
			/** @brief Constructor
			 *  @param str
			 *  	Calling the method \a what() will return this string. */
			notInvertible(const std::string& str = "") : std::runtime_error(str) {};
		};

		/** @brief Exception denoting that an algorithm does not converge.
		 *  @details If an algorithm does not converge, this exception might be thrown. */
		class noConvergence : public std::runtime_error {
		public:
			/** @brief Constructor
			 *  @param str
			 *  	Calling the method \a what() will return this string. */
			noConvergence(const std::string& str = "") : std::runtime_error(str) {};
		};

	/** @} */
	// == ITERATOR ==
	/** @name Iterator class and operations */
	/** @{ */

		class matrix_iterator;
		friend class matrix_iterator;

		/** @brief Line-by-line or column-by-column random access iterator for matrices.
		 *  @details This iterator supports all necessary operations to be a random access iterator */
		class matrix_iterator : public std::iterator<std::random_access_iterator_tag, T> {
		private:
			/** @brief The iterator iterates over this matrix. */
			Matrix<T> *m;

			/** @brief The current row index. */
			int i;

			/** @brief The current column index. */
			int j;

			/** @brief Switch for line-by-line and column-by-column iteration.
			 *  @details If this variable is false, then the iterator iterates column-by-column over the matrix.
			 *  	If it is true, then the iterator iterates line-by-line. */
			bool line_by_line;
		public:
			/** @name Iterator parameter constants */
			/** @{ */
				/** @brief Line-by-line iteration */
				const static bool LINE_BY_LINE = true;

				/** @brief Column-by-Column iteration */
				const static bool COLUMN_BY_COLUMN = false;

				/** @brief iterator starting at the begin of the matrix */
				const static bool BEGIN = false;

				/** @brief iterator starting at the end of the matrix */
				const static bool END = true;
			/** @}*/

			/** @name Iterator typedefs */
			/** @{ */
				typedef std::random_access_iterator_tag it_cat;
				typedef typename std::iterator<it_cat, T>::value_type value_type;
				typedef typename std::iterator<it_cat, T>::difference_type diff_type;
				typedef typename std::iterator<it_cat, T>::reference reference;
				typedef typename std::iterator<it_cat, T>::pointer pointer;
			/** @}*/

			/** @name Iterator constructors */
			/** @{ */
				/** @brief Standard constructor
				 *  @param mat
				 *  	the iterator iterates over the matrix.
				 *  @param line_by_line
				 *  	if true, the iterator iterates line-by-line over the matrix. Otherwise it iterates column-by-column.
				 *  @param end
				 *  	if false (default value), the iterator starts at the begin of the matrix. Otherwise it start at the end. */
				matrix_iterator(Matrix<T>& mat, bool line_by_line, bool end = END);

				/** @brief Default constructor.
				 *  @attention The matrix pointer is set to NULL, therefore the program will be aborted,
				 *  	if a default constructor is accessed. */
				matrix_iterator();

				/** @brief Copy constructor.
				 * 	@param it
				 * 		The iterator that will be copied. */
				matrix_iterator(const matrix_iterator& it);

				/** @brief Assignment operator.
				 *  @param it
				 *  	This iterator gets the values of \a it.	 */
				matrix_iterator& operator=(const matrix_iterator& it);
			/** @} */

			/** @name Arithmetic operators
			 *  @details If the end of a line respectively column is reached,
			 *  	the iterator continues at the begin of the next column respectively line. */
			/** @{ */
				matrix_iterator& operator++();
				matrix_iterator& operator--();
				matrix_iterator operator++(int);
				matrix_iterator operator--(int);
				matrix_iterator& operator+=(diff_type);
				matrix_iterator& operator-=(diff_type);
				matrix_iterator operator+(diff_type) const;
				matrix_iterator operator-(diff_type) const;
			/** @} */

			/** @name Difference operator
			 *  @attention Ensure that both iterators iterates in the same direction (both line-by-line or both column-by-column). */
			/** @{ */
				diff_type operator-(const matrix_iterator&) const;
			/** @} */

			/** @name Reference operators
			 *  @attention The access is not checked. If the iterator does not point to valid matrix entry, the behaviour is undefined, when you access such an element. */
			/** @{ */
				reference operator*() const;
				pointer operator->() const;
				reference operator[](diff_type) const;
			/** @} */

			/** @name compare operators
			 *  @attention if you use <, >, <= or >=, then ensure that both iterators iterates in the same direction (both line-by-line or both column-by-column). */
			/** @{ */
				bool operator==(const matrix_iterator& it) const;
				bool operator!=(const matrix_iterator& it) const;
				bool operator<(const matrix_iterator& it) const;
				bool operator>(const matrix_iterator& it) const;
				bool operator<=(const matrix_iterator& it) const;
				bool operator>=(const matrix_iterator& it) const;
			/** @} */
		};

		/** @brief Creates a line-by-line iterator pointing to the first element.
		 *  @returns a random access iterator which iterates line-by-line over the matrix.
		 *  @see end, begin_cbc, matrix_iterator */
		matrix_iterator begin();

		/** @brief Creates a line-by-line iterator pointing to the last element.
		 *  @returns a random access iterator which iterates line-by-line over the matrix.
		 *  @see begin, end_cbc, matrix_iterator */
		matrix_iterator end();

		/** @brief Creates a line-by-line iterator pointing to the first element.
		 *  @returns a random access iterator which iterates column-by-column over the matrix.
		 *  @see end_cbc, begin, matrix_iterator */
		matrix_iterator begin_cbc();

		/** @brief Creates a line-by-line iterator pointing to the last element.
		 *  @returns a random access iterator which iterates column-by-column over the matrix.
		 *  @see begin_cbc, end, matrix_iterator */
		matrix_iterator end_cbc();

	/** @} */
	// == KONSTRUKTOREN & DESTRUKTOR ==
	/** @name Constructors, destructor, assignment operator and swap operators */
	/** @{ */

		/** @brief Creates a (0 x 0)-matrix.
		 *  @details This default constructor does not create any dynamic array, therefore it is very fast.
		 *  	Usually the matrix is filled with data by operator=.
		 *  @see Matrix(int, int), Matrix(const Matrix&) */
		Matrix();

		/** @brief Standard-Constructor.
		 *  @details Creates a (dim1 x dim2)-Matrix. The values of the matrix elements are undefined.
		 *  @param dim1
		 *  	number of rows
		 *  @param dim2
		 *  	number of columns
		 *  @see Matrix(), Matrix(const Matrix&), Matrix(int, int, T*), zero(int, int), id(int), fill(T), Matrix(int, int, bool) */
		Matrix(int dim1, int dim2);

		/** @brief Constructor that writes the passed data into the matrix.
		 *  @param dim1
		 *  	number of rows
		 *  @param dim2
		 *  	number of columns
		 *  @param data
		 *  	pointer to a data field which entries are copied line-by-line to the matrix.
		 *  	The field must have a length of at least dim1 x dim2, otherwise the behaviour is undefined.
		 *  @see Matrix(int, int) */
		Matrix(int dim1, int dim2, T *data);

		/** @brief Copy-Constructor.
		 *  @param m
		 *  	The matrix that will be copied.
		 *  @details Creates a deep copy of the passed matrix
		 *  @see Matrix(), Matrix(int, int), swap, set, set_noCopy */
		Matrix(const Matrix<T>& m);

		/** @brief Constructor
		 *  @details Creates a (dim1 x dim2)-Matrix. The values of the matrix elements are undefined.
		 *  	If \a dim1 is bigger than \a dim2 and \a optimize is true, then the data field of the matrix is transposed to
		 *  	minimize the memory usuage.
		 *  @param dim1
		 *  	number of rows
		 *  @param dim2
		 *  	number of columns
		 *  @param optimize
		 *  	determines if the structure of the data field of the matrix will be optimized.
		 *  @see Matrix(int, int) */
		Matrix(int dim1, int dim2, bool optimize);

		/** @brief Destructor
		 *  @details The field \a entries_ is deleted subject to \a the attribut createArray_.\n
		 *  	\a entries_ is deleted completly, if \a createArray_ == \a COMPLETE_ARRAY (default case) \n
		 *  	\a entries_ is deleted shallowly, if \a createArray_ == \a SHALLOW_ARRAY \n
		 *  	\a entries_ is not deleted, if \a createArray_ == \a NO_ARRAY \n
		 *  	A shallow deletion means that only the first array is deleted, however the sub arrays are not deleted.
		 *  @attention If a matrix M accesses the data field of the deleted matrix,
		 *  	then M must not be accessed any more until it gets a new data field (for example via \a swap() or \a operator=).
		 *  	Otherwise the behaviour is undefined.  */
		virtual ~Matrix();

		/** @brief Assignment operator.
		 *  @details This operator copies the data field from the passed matrix to "this" matrix.
		 *  	The created copy is always a deep copy.
		 *  	If the passed matrix is implicitly transposed, "this" matrix will be transposed too.
		 *  @param m
		 *  	The matrix that will be copied.
		 *  @see swap, set, set_noCopy */
		Matrix <T>& operator=(const Matrix <T> & m);

		/** @brief Swaps all attributes with the passed matrix.
		 *  @details Instead of swapping the whole data field,
		 *  	only the pointer to the data fields are swapped,
		 *  	therefore this method runs in O(1)time.
		 *  @param m
		 *  	The matrix whose content will be swapped with "this".
		 *  @see set, set_noCopy */
		void swap(Matrix<T>& m);

		/** @brief Swaps all attributes with the passed matrix.
		 *  @details Instead of swapping the whole data field,
		 *  	only the pointer to the data fields are swapped,
		 *  	therefore this method runs in O(1)time.
		 *  	The difference to \a swap() is that the matrix is passed by value not by reference.
		 *  @param m
		 *  	The matrix whose content will be swapped with "this".
		 *  @see swap, set_noCopy */
		void set(Matrix<T> m);

		/** @brief Sets the matrix' content to the passed matrix implicitly.
		 *  @details The matrix accesses the data field of the passed matrix.
		 *  	The old data field is deleted. Therefore this operation runs in O(1) time plus the time for deleting the old array.
		 *  @attention If the passed matrix or its the data field is deleted, the matrix must not be accessed in any way,
		 *  	until it points to a new data field (for example via \a swap, \a operator= or something like this).
		 *  	Preferable use this operation only, if you need a fast copy and if you are sure,
		 *  	that the passed will be deleted after "this" matrix.
		 *  @param m
		 *  	The matrix whose data field will be accessed by "this" matrix.
		 *  @see swap, set */
		void set_noCopy(const Matrix<T>& m);

	/** @} */
	// == STATISCHE KONSTRUKTOREN ==
	/** @name static constructors */
	/** @{ */

		/** @brief Creates an identity matrix
		 *  @param dim
		 *  	The number of rows and columns of the returned matrix.
		 *  @return a (dim x dim) identity matrix.
		 *  @details The returned matrix conatins only one on the main diagonal and zeros elsewhere.
		 *  @see zero(int, int), Matrix(int, int) */
		static Matrix <T> id(int dim);

		/** @brief Creates a zero matrix.
		 *  @param dim1
		 *  	The number of rows of the returned matrix.
		 *  @param dim2
		 *  	The number of columns of the returned matrix.
		 *  @param optimize
		 *  	determines if the structure of the data field of the matrix will be optimized.
		 *  @return a (dim1 x dim2) zero matrix.
		 *  @details All entries of the returned matrix are zero.
		 *  	If \a dim1 is bigger than \a dim2 and \a optimize is true,
		 *  	then the data field of the matrix is transposed to
		 *  	minimize the memory usuage.
		 *  @see id(int), Matrix(int, int), fill(T) */
		static Matrix <T> zero(int dim1, int dim2, bool optimize = false);

	/** @} */
	// == DIMENSIONS ==
	/** @name dimension access operations */
	/** @{ */

		/** @brief Returns the dimensions of the matrix
		 *  @returns a pair whose first entry contains the number of rows and whose second entry contains the number of columns
		 *  @see getDim1, getDim2 */
		std::pair<int, int> getDimensions() const;

		/** @brief Returns the first dimension.
		 *  @returns the number of rows.
		 *  @see getDim2, getDimensions */
		int getDim1() const;

		/** @brief Returns the first dimension.
		 *  @returns the number of columns.
		 *  @see getDim1, getDimensions */
		int getDim2() const;

		/** @brief Enlarges or reduces the size of the matrix. Vergr��ert und verkleinert die Matrix.
		 *  @param dim1
		 *  	the new number of rows.
		 *  @param dim2
		 *  	the new number of columns.
		 *  @details The matrix is scaled up or down.
		 *  	If the matrix is enlarged, the value of the new entries is undefined respectively default constructed.
		 *  	If the matrix is reduced, the old values are lost.
		 *  	This operation takes O(dim1 x dim2) time, unless only the primary array is affected. Then O(dim1) or O(dim2) is possible.
		 *  @attention If another matrix \a m accesses the entries of "this" matrix, then \a m is invalid. Accessing \a m's elements will cause undefined behaviour. */
		void resize(int dim1, int dim2);

		/** @brief Returns true, if both dimensions are zero.
		 *  @returns true, if the matrix is a (0 x 0)-matrix. Otherwise false is returned.
		 *  @details If this is function is called on a default constructed matrix, it will return true. */
		bool isEmpty() const;


	/** @} */
	// == ELEMENT-ZUGRIFF ==
	/** @name element access operations */
	/** @{ */

		/** @brief Sets the entry aij to a specified value.
		 *  @param index1
		 *  	index of the first dimension.
		 *  @param index2
		 *  	index of the second dimension.
		 *  @param value
		 *  	the new value of the specified entry.
		 *  @details After this operation the call \a getEntry(\a index1, \a index2) will return the value of \a value.
		 *  @attention The range is not checked. Be sure that both indices are not negative
		 *  	and that they are lower than the dimension of the matrix.
		 *  	Otherwise the behaviour is undefined.
		 *  @see updateEntryCheck, addEntry, getEntry, {@link operator()(int, int) operator()} */
		void updateEntry(int index1, int index2, T value);

		/** @brief Sets the entry aij to a specified value checking the indices.
		 *  @param index1
		 *  	index of the first dimension.
		 *  @param index2
		 *  	index of the second dimension.
		 *  @param value
		 *  	the new value of the specified entry.
		 *  @details If the entry (index1, index2) is not within the matrix, nothing will happen.
		 *  	If the indices are valid, the call \a getEntry(\a index1, \a index2) will return the value of \a entry after this operation.
		 *  	This operation is a bit slower than \a updateEntry(), because the range is checked.
		 *  @see upateEntry, getEntryCheck */
		void updateEntryCheck(int index1, int index2, T value);

		/** @brief Adds a specified value to an entry.
		 *  @param index1
		 *  	index of the first dimension.
		 *  @param index2
		 *  	index of the second dimension.
		 *  @param value
		 *  	this value is added to the entry specified by the indices.
		 *  @details Calling this method on a matrix \a m has the same effect as \a m(\a index1, \a index2) += \a value.
		 *  @attention The range is not checked. Be sure that both indices are not negative
		 *  	and that they are lower than the dimension of the matrix.
		 *  	Otherwise the behaviour is undefined.
		 *  @see getEntry, updateEntry, {@link operator()(int, int) operator()} */
		void addEntry(int index1, int index2, T value);

		/** @brief Returns the value of the specified entry.
		 *  @param index1
		 *  	index of the first dimension.
		 *  @param index2
		 *  	index of the second dimension.
		 *  @returns the value at (index1, index2).
		 *  @attention The range is not checked. Be sure that both indices are not negative
		 *  	and that they are lower than the dimension of the matrix.
		 *  	Otherwise the behaviour is undefined.
		 *  @see getEntryCheck, updateEntry, addEntry, {@link operator()(int, int) operator()}, {@link operator()(int, int)const operator() const} */
		T getEntry(int index1, int index2) const;

		/** @brief Returns the value of the specified entry.
		 *  @param index1
		 *  	index of the first dimension.
		 *  @param index2
		 *  	index of the second dimension.
		 *  @returns the value at (index1, index2) or zeros, if the indices are not within the matrix.
		 *  @details This operation is a bit slower than \a getEntry(), because the range is checked.
		 *  @see getEntry, updateEntryCheck */
		T getEntryCheck(int index1, int index2) const;

		/** @brief Returns a reference to the specified entry
		 *  @param index1
		 *  	index of the first dimension.
		 *  @param index2
		 *  	index of the second dimension.
		 *  @returns a reference to (index1, index2).
		 *  @attention The range is not checked. Be sure that both indices are not negative
		 *  	and that they are lower than the dimension of the matrix.
		 *  	Otherwise the behaviour is undefined.
		 *  @details This operator can be used for read and write access to the matrix. \n
		 *  @par Examples
		 *  	"m.updateEntry(i, j, v);" has the same effect as "m(i, j) = v;".
		 *  @see getEntry, updateEntry, addEntry, getEntryCheck, updateEntryCheck, {@link operator[](size_t) operator[]}, {@link operator()(int, int)const operator() const} */
		T& operator()(int index1, int index2);

		/** @brief Returns a const reference to the specified entry
		 *  @param index1
		 *  	index of the first dimension.
		 *  @param index2
		 *  	index of the second dimension.
		 *  @returns a const reference to (index1, index2).
		 *  @attention The range is not checked. Be sure that both indices are not negative
		 *  	and that they are lower than the dimension of the matrix.
		 *  	Otherwise the behaviour is undefined.
		 *  @details This operator is the const version of {@link operator()(int, int) operator()}.
		 *  @see getEntry, getEntryCheck, {@link operator[](size_t)const operator[] const}, {@link operator()(int, int) operator()} */
		const T& operator()(int index1, int index2) const;

		/** @brief Provides an easy way to read and write elements of vectors.
		 *  @param idx
		 *  	The index of the returned reference.
		 *  @returns a reference to the element with the index \a idx.
		 *  @attention The range is not checked. Be sure that the index is not negative
		 *  	and that it is lower than the product of both dimensions of the matrix.
		 *  	Otherwise the behaviour is undefined.
		 *  @details This operator can be used for both read and write access.
		 *  	If the object is not a vector but a matrix, the entries are accessed line-by-line. \n
		 *  @par Examples
		 *  	The call "m[idx]" returns the same reference as "m(idx / m.getDim1(), idx % m.getDim1())".
		 *  @see {@link operator()(int, int) operator()}, {@link operator[](size_t)const operator[] const} */
		T& operator[](size_t idx);

		/** @brief Provides an easy way to read elements of vectors.
		 *  @param idx
		 *  	The index of the returned reference.
		 *  @returns a const reference to the element with the index \a idx.
		 *  @attention The range is not checked. Be sure that the index is not negative
		 *  	and that it is lower than the product of both dimensions of the matrix.
		 *  	Otherwise the behaviour is undefined.
		 *  @details If the object is not a vector but a matrix, the entries are accessed line-by-line.
		 *  	This operator is the const version of {@link operator[](size_t) operator[]}.
		 *  @see {@link operator()(int, int)const operator() const}, {@link operator[](size_t) operator[]} */
		const T& operator[](size_t idx) const;


	/** @} */
	// == MATRIX-ZUGRIFF ==
	/** @name matrix access operations */
	/** @{ */

		/** @brief Sets all entries to the specified value.
		 *  @param value
		 *  	The new value of all entries.
		 *  @details After this operation all elements of the matrix have the value \a value. */
		void fill(T value);

		/** @brief Adds the specified value to each matrix entry.
		 *  @param value
		 *  	The value that will be added. */
		void addtoAll(T value);

		/** @brief Multiplies the specified value to each matrix entry.
		 *  @param value
		 *  	The value that will be multiplied. */
		void multiplytoAll(T value);

		/** @brief Applies the specified function to all matrix elements.
		 *  @param func
		 *  	The function that will be applied.
		 *  @details Each element \a aij of the matrix gets the new value \a func(\a aij).
		 *  @par Examples
		 *  	Takes the root of each element: \n \a transform(sqrt)
		 *  @see {@link map map(T(T,T))}, {@link transform(F) transform(F)}, \ref anchorTransformMatrix "transform(const Matrix<T>&, T(T,T))" */
		void transform(T (*func)(T));

		/** @brief Applies the callable object to all matrix elements.
		 *  @param func
		 *  	Callable object which defines \a T \a operator()(\a T).
		 *  @details Each element \a aij gets the new value \a func(\a aij).
		 *  @par Examples
		 *  	Multipies each element of an int-matrix with itself: \n \a transform([](int x){return x*x;})
		 *  @see {@link map(F)const map(F)}, {@link transform transform(T(T,T))}, {@link transform(const Matrix<T>&, F) transform(const Matrix<T>&, F)} */
		template <class F> void transform(F func);

		/** @brief Applies the specified function to all matrix elements and returns the result matrix.
		 *  @param func
		 *  	The function that will be applied.
		 *  @returns a new matrix \a B in which each element \a Bij has the value \a func(\a this->(i,j)).
		 *  @details the passed function is applied to all matrix elements. The result matrix is returned. The matrix on which this method is called does not change.
		 *  @see {@link transform transform(T(T,T))}, {@link map(F)const map(F)}, \ref anchorMapMatrix "map(const Matrix<T>&, T(T,T))" */
		Matrix<T> map(T (*func) (T)) const;

		/** @brief Applies the callable object to all matrix elements and returns the result matrix.
		 *  @param func
		 *  	Callable object which defines \a T \a operator()(\a T).
		 *  @returns a new matrix \a B in which each element \a Bij has the value \a func(\a this->(i,j)).
		 *  @details the passed callable object is applied to all matrix elements. The result matrix is returned. The matrix on which this method is called does not change.
		 *  @see {@link transform(F) transform(F)}, {@link map map(T(T,T))}, {@link map(const Matrix<T>&, F)const map(const Matrix<T>&, F)} */
		template <class F> Matrix<T> map(F func) const;

		/** @brief Applies the specified function to the elements of two matrices.
		 *  @param func
		 *  	The function that will be applied.
		 *  @param mat
		 *  	The elements of this matrix are passed to \a func as the second parameter.
		 *  @details Each element <em> a(i,j) </em> of this matrix gets
		 *  	the new value <em> func(a(i,j), m(i,j)) </em>. So the entries
		 *  	of both matrices are calculated together via the passed function.
		 *  	The result is stored to this matrix.
		 *  @see {@link transform(const Matrix<T>&, F) transform(const Matrix<T>&, F)}, \ref anchorMapMatrix "map(const Matrix<T>&, T(T,T))", {@link transform transform(T(T,T))}
		 *  @anchor anchorTransformMatrix */
		void transform(const Matrix<T>& mat, T (*func) (T, T));

		/** @brief Applies the callable object to the elements of two matrices.
		 *  @param func
		 *  	Callable object which defines \a T \a operator()(\a T, \a T).
		 *  @param mat
		 *  	The elements of this matrix are passed to \a func as the second parameter.
		 *  @details Each element <em> a(i,j) </em> of this matrix gets
		 *  	the new value <em> func(a(i,j), m(i,j)) </em>. So the entries
		 *  	of both matrices are calculated together via the passed function.
		 *  	The result is stored to this matrix.
		 *  @see \ref anchorTransformMatrix "transform(const Matrix<T>&, T(T,T))", {@link map(const Matrix<T>&, F)const map(const Matrix<T>&, F)}, {@link transform(F) tranform(F)} */
		template <class F> void transform(const Matrix<T>& mat, F func);

		/** @brief Applies the specified function to the elements of two matrices and returns the result matrix.
		 *  @param func
		 *  	The function that will be applied.
		 *  @param mat
		 *  	The elements of this matrix are passed to \a func as the second parameter.
		 *  @returns a new matrix \a B in which each element \a Bij has the value \a func(this->(i,j), mat(i,j)).
		 *  @details The entries of both matrices are calculated together via the passed function.
		 *  	The result matrix is returned.
		 *  @see {@link map(const Matrix<T>&, F)const map(const Matrix<T>&, F)},  \ref anchorTransformMatrix "transform(const Matrix<T>&, T(T,T))", {@link map map(T(T,T))}
		 *  @anchor anchorMapMatrix */
		Matrix<T> map(const Matrix<T>& mat, T (*func) (T, T)) const;

		/** @brief Applies the callable object to the elements of two matrices and returns the result matrix.
		 *  @param func
		 *  	Callable object which defines \a T \a operator()(\a T, \a T).
		 *  @param mat
		 *  	The elements of this matrix are passed to \a func as the second parameter.
		 *  @returns a new matrix \a B in which each element \a Bij has the value \a func(this->(i,j), mat(i,j)).
		 *  @details The entries of both matrices are calculated together via the callable object \a func.
		 *  	The result matrix is returned.
		 *  @see \ref anchorMapMatrix "map(const Matrix<T>&, T(T,T))", {@link transform(const Matrix<T>&, F) transform(const Matrix<T>&, F)}, {@link map(F)const, map(F)}
		 *  @anchor anchorMapMatrix */
		template <class F> Matrix<T> map(const Matrix<T>& mat, F func) const;

		/** @brief Calculates all elements together.
		 *  @details This method implements a fold operator: First of all the specified function \a func is applied to the initial value \a begin
		 *  	and the first matrix element. Afterwards the function is applied to the result and the second matrix element and so on.
		 *  	At the end each element of the matrix was accessed one time. The result of the last \a func()-call is returned. This method iterates line-by-line over the matrix.
		 *  @param func
		 *  	The function that will be used for the fold-operation.
		 *  	The initial value and the intermediate results are passed to the first parameter, the matrix elements to the second parameter.
		 *  @param begin
		 *  	The intial value.
		 *  @see {@link fold(F, T)const fold(F, T)} */
		T fold(T (*func) (T, T), T begin) const;

		/** @brief Calculates all elements together.
		 *  @details This method implements a fold operator: First of all the specified callable object \a func is applied to the initial value \a begin
		 *  	and the first matrix element. Afterwards the callable object is applied to the result and the second matrix element and so on.
		 *  	At the end each element of the matrix was accessed one time. The result of the last call is returned. This method iterates line-by-line over the matrix.
		 *  @param func
		 *  	The callable object that will be used for the fold-operation. The object must implement \a T \a operator()(\a T, \a T).
		 *  	The initial value and the intermediate results are passed to the first parameter, the matrix elements to the second parameter.
		 *  @param begin
		 *  	The intial value.
		 *  @par Examples
		 *  	Adding all values of a double-matrix: \n \a fold(std::plus<double>(), 0.0)
		 *  @par
		 *  	Calculation of the maximum of an int-matrix: \n \a fold( [](int a, int b){return a < b ? b : a;}, std::numeric_limits<int>::min() )
		 *  @see {@link fold fold(T(T,T), T)} */
		template <class F> T fold(F func, T begin) const;

	/** @} */
	// == GRUNDLEGENDE MATRIX-OPERATIONEN ==
	/** @name Basic matrix operations */
	/** @{ */

		/** @brief Transposes the matrix implicitly.
		 *  @details The matrix is implicitly transposed in O(1) time.
		 *  @see getTranspose */
		void transpose();

		/** @brief Returns a transpose matrix.
		 *  @returns a transpose matrix that accesses the data field of "this" matrix.
		 *  @attention If "this" matrix is deleted, then the returned matrix is invalid, so accessing its elements will cause undefined behaviour.
		 *  	If you change entries of the returned matrix, the corresponding entries of the original matrix are changed too.
		 *  @see transpose */
		const Matrix<T> getTranspose() const;

		/** @brief Creates an submatrix.
		 *  @param off1
		 *  	row offset.
		 *  @param off2
		 *  	column offset.
		 *  @param dim1
		 *  	number of rows of the returned matrix.
		 *  @param dim2
		 *  	number of columns of the returned matrix.
		 *  @returns a (\a dim1 x \a dim2)-submatrix that accesses the data field of "this" matrix.
		 *  @details The element T(i,j) of the submatrix T corresponds to the element A(i + \a off1, j + \a off2) of "this" matrix.
		 *  	The runtime of this operation is linear in the primary dimension or even constant if possible.
		 *  @pre The offset plus the passed dimension must be lower or equal than the dimension of "this" matrix: \n
		 *  	<em> off1 + dim1 <= dim1_ </em> \n
		 *  	<em> off2 + dim2 <= dim2_ </em>
		 *  @attention If "this" matrix is deleted, then the returned matrix is invalid, so accessing its elements will cause undefined behaviour.
		 *  	If you change entries of the returned matrix, the corresponding entries of the original matrix are changed too. */
		const Matrix <T> select(int off1, int off2, int dim1, int dim2) const;

		/** @brief Inserts a matrix into another.
		 *  @param mat
		 *  	The matrix that will be inserted.
		 *  @param off1
		 *  	row offset.
		 *  @param off2
		 *  	column offset.
		 *  @details Inserts the matrix \a mat into "this" matrix.
		 *  	After this operation the entry (off1 + i, off2 + j) of "this" matrix is equal to mat(i,j) for valid values of i and j.
		 *  	If the matrix \a mat is too big, only the fitting part of \a mat is inserted.
		 *  @see {@link insert(const Matrix<T>&, int, int, int, int) insert}*/
		void insert(const Matrix<T>& mat, int off1, int off2);

		/** @brief Inserts a matrix into another.
		 *  @param mat
		 *  	The matrix that will be inserted.
		 *  @param off1
		 *  	row offset.
		 *  @param off2
		 *  	column offset.
		 *  @param size1
		 *  	the number of rows that will be copied.
		 *  @param size2
		 *  	the number of columns that will be copied.
		 *  @details Inserts the matrix \a mat into "this" matrix.
		 *  	After this operation the entry (off1 + i, off2 + j) of "this" matrix is equal to mat(i,j) for valid values of i < \a size1 and j < \a size2.
		 *  	If the matrix \a mat is too big or \a size1 or \a size2, only the fitting part of \a mat is inserted.
		 *  @see {@link insert(const Matrix<T>&, int, int) insert}*/
		void insert(const Matrix<T>& mat, int off1, int off2, int size1, int size2);

	/** @} */
	// == GRUNDLEGENDE MATRIX-MATRIX-OPERATIONEN ==
	/** @name Basic matrix-matrix-operations */
	/** @{ */

		/** @brief Adds the specified matrix to the current matrix.
		 *  @param mat
		 *  	The matrix that will be added.
		 *  @details Both matrices "this" and \a mat must have the same size.
		 *  	Each element this->(i,j) gets the new value this->(i,j) + mat(i,j).
		 *  @throw dim_exception if the matrices does not have the same size.
		 *  @see subMatrix, {@link addMatrix(const Matrix&, int, int, int, int, int, int)const addMatrix} */
		void addMatrix(const Matrix<T>& mat);

		/** @brief Subtracts the specified matrix from the current matrix.
		 *  @param mat
		 *  	The matrix that will be subtracted.
		 *  @details Both matrices "this" and \a mat must have the same size.
		 *  	Each element this->(i,j) gets the new value this->(i,j) - mat(i,j).
		 *  @throw dim_exception if the matrices does not have the same size.
		 *  @see addMatrix, {@link subMatrix(const Matrix&, int, int, int, int, int, int)const addMatrix}*/
		void subMatrix(const Matrix<T>& mat);

		/** @brief Adds the current matrix with the passed matrix. The offsets of both matrices can be specified.
		 *  @param data
		 *  	The matrix which will be added to "this".
		 *  @param dim1
		 *  	The number of rows of the returned matrix.
		 *  @param dim2
		 *  	The number of columns of the returned matrix.
		 *  @param off_a1
		 *  	Row offset of "this" matrix.
		 *  @param off_a2
		 *  	Column offset of "this" matrix.
		 *  @param off_b1
		 *  	Row offset of the passed matrix.
		 *  @param off_b2
		 *  	Column offset of the passed matrix.
		 *  @returns a (\a dim1 x \a dim2)-matrix \a M where each entry \a M(i,j) is equal to \n
		 *  	this->(\a off_a1 + i, \a off_a2 + j) + \a data(\a off_b1 + i, \a off_b2 + j). \n
		 *  	If an element of this formula is out of range, the value 0 is used instead.
		 *  @see {@link subMatrix(const Matrix&, int, int, int, int, int, int)const addMatrix}, addMatrix */
		Matrix<T> addMatrix(const Matrix<T>& data, int dim1, int dim2, int off_a1, int off_a2, int off_b1, int off_b2) const;

		/** @brief Subtract the passed matrix from the current matrix. The offsets of both matrices can be specified.
		 *  @param data
		 *  	The matrix which will be subtracted from "this".
		 *  @param dim1
		 *  	The number of rows of the returned matrix.
		 *  @param dim2
		 *  	The number of columns of the returned matrix.
		 *  @param off_a1
		 *  	Row offset of "this" matrix.
		 *  @param off_a2
		 *  	Column offset of "this" matrix.
		 *  @param off_b1
		 *  	Row offset of the passed matrix.
		 *  @param off_b2
		 *  	Column offset of the passed matrix.
		 *  @returns a (\a dim1 x \a dim2)-matrix \a M where each entry \a M(i,j) is equal to \n
		 *  	this->(\a off_a1 + i, \a off_a2 + j) - \a data(\a off_b1 + i, \a off_b2 + j). \n
		 *  	If an element of this formula is out of range, the value 0 is used instead.
		 *  @see {@link addMatrix(const Matrix&, int, int, int, int, int, int)const addMatrix}, subMatrix */
		Matrix<T> subMatrix(const Matrix<T>& data, int dim1, int dim2, int off_a1, int off_a2, int off_b1, int off_b2) const;

		/** @brief Naive (slow) unchecked matrix multiplication.
		 *  @param mat
		 *  	The matrix that will be multiplied (second argument).
		 *  @returns "this" multiplied by \a mat.
		 *  @details This method undertakes no security checks.
		 *  	If the number of columns of "this" is not equal to the number of rows of \a mat, the lower value is used.
		 *  	This method uses the naive way of multiplication. The runtime is O(n^3).
		 *  @see multiplyNaive, multiplyMixUnchecked */
		Matrix<T> multiplyNaiveUnchecked(const Matrix<T>* mat) const;

		/** @brief Naive (slow) matrix multiplication.
		 *  @param mat
		 *  	The matrix that will be multiplied (second argument).
		 *  @returns "this" multiplied by \a mat.
		 *  @details This method undertakes security checks and will throw exceptions if the parameters are not valid.
		 *  	This method uses the naive way of multiplication. The runtime is O(n^3).
		 *  @throws std::invalid_argument if mat is NULL.
		 *  @throws dim_exception if the number of columns of "this" is not equal to the number of rows of \a mat.
		 *  @see multiplyNaiveUnchecked, multiplyMix */
		Matrix<T> multiplyNaive(const Matrix<T>* mat) const;

		/** @brief Unchecked matrix multiplication using the (fast) Strassen algorithm.
		 *  @param mat
		 *  	The matrix that will be multiplied (second argument).
		 *  @returns "this" multiplied by \a mat.
		 *  @details This method undertakes no security checks.
		 *  	If the number of columns of "this" is not equal to the number of rows of \a mat, the lower value is used.
		 *  	This method uses the Strassen algorithm. The runtime is O(n^(lb 7)) = O(n^2.807...).
		 *  	This method needs more space than the  {@link multiplyNaive naive way of multiplication}.
		 *  	If the input matrices are small (or will get small enough during the algorithm) the naive matrix multiplication is used.
		 *  @see multiplyNaiveUnchecked, multiplyMix */
		Matrix<T> multiplyMixUnchecked(Matrix<T>* mat) const;

		/** @brief Matrix multiplication using the (fast) Strassen algorithm.
		 *  @param mat
		 *  	The matrix that will be multiplied (second argument).
		 *  @returns "this" multiplied by \a mat.
		 *  @details This method undertakes security checks and will throw exceptions if the parameters are not valid.
		 *  	This method uses the Strassen algorithm. The runtime is O(n^(lb 7)) = O(n^2.807...).
		 *  	This method needs more space than the  {@link multiplyNaive naive way of multiplication}.
		 *  	If the input matrices are small (or will get small enough during the algorithm) the naive matrix multiplication is used.
		 *  @throws std::invalid_argument if mat is NULL.
		 *  @throws dim_exception if the number of columns of "this" is not equal to the number of rows of \a mat.
		 *  @see multiplyNaiveUnchecked, multiplyMix */
		Matrix<T> multiplyMix(const Matrix<T>* mat) const;

		/** @brief Multiplies the matrix with a diagonal matrix \a D.
		 *  @param data
		 *  	Pointer to a field of the length \a size. Each element of the field represents an entry of the main diagonal of \a D.
		 *  @param size
		 *  	Number of elements of the data array.
		 *  @param dim2
		 *  	Number of columns of the result matrix.
		 *  @returns the result of the multiplication of "this" matrix by the diagonal matrix \a D which elements are specified by \a data.
		 *  @details The number of rows of \a D is equal to the number of columns of "this".
		 *  	The number of columns of \a D is equal to \a dim2.
		 *  	The entry (i,j) of \a D is equal to data[i] if i == j and i < size. Else the entry is equal 0.
		 *  @see {@link multiplyWithDiagonal(const std::vector<T>&)const multiplyWithDiagonal} */
		Matrix <T> multiplyWithDiagonal(const T* data, int size, int dim2) const;

		/** @brief Multiplies the matrix with a diagonal matrix \a D.
		 *  @param data
		 *  	Vector that represents the values of the main diagonal of \a D.
		 *  @returns the result of the multiplication of "this" matrix by the diagonal matrix \a D which elements are specified by \a data.
		 *  @details The number of rows of \a D is equal to the number of columns of "this".
		 *  	The number of columns of \a D is equal to its number of rows.
		 *  	The entry (i,j) of \a D is equal to data[i] if i == j. Else the entry is equal 0.
		 *  @see {@link multiplyWithDiagonal(const T*, int, int)const multiplyWithDiagonal} */
		Matrix <T> multiplyWithDiagonal(const std::vector<T>& data) const;

		/** @brief Calculates the product A^T*A.
		 *  @returns the transpose matrix of "this" multiplied by "this".
		 *  @details the transpose matrix is not created explicitly in this operation. */
		Matrix<T> calcATA() const;

		/** @brief Calculates the product A*A^T.
		 *  @returns "this" multiplied by the transpose matrix of "this".
		 *  @details the transpose matrix is not created explicitly in this operation. */
		Matrix<T> calcAAT() const;

		/** @brief Calculates the relative deviation to a specified matrix
		 *  @param b
		 *  	The matrix that will be compared to "this" matrix
		 *  @returns a matrix whose entry cij is equal to std::max(aij / bij, bij / aij).
		 *  @throws dim_excpetion
		 *  	if "this" and \a b have different dimensions */
		Matrix <T> relDeviation(const Matrix<T>& b) const;

	/** @} */
	// == BERECHNUNGEN ==
	/** @name Calculations */
	/** @{ */

		/** @brief Returns the sum of all entries on the main diagonal
		 *  @returns the trace of the matrix.
		 *  @see sum */
		T trace() const;

		/** @brief Calculates the Frobenius norm.
		 *  @returns the Frobenius norm of the matrix.
		 *  @param square
		 *  	if this parameter is true, the squared Frobenius norm is returned.
		 *  @details The Frobeinus norm is the root of the sum of the square of all entries. */
		double frobeniusnorm(bool square = false) const;

		/** @brief Calculates the sum of all entries.
		 *  @returns the sum of all entries of the matrix.
		 *  @see trace, mean */
		T sum() const;

		/** @brief Returns the arithmetic mean of all entries.
		 *  @returns the arithmetic mean of all entries of the matrix.
		 *  @details The result of the call <em> m.mean() </em> is equivalent to <em> m.sum() / m.getDim1() / m.getDim2() </em>
		 *  @see sum, median */
		double mean() const;

		/** @brief Calculates the median of all entries.
		 *  @returns the median of all entries of the matrix.
		 *  @details The median is the value separating the higher half of all elements from the lower half.
		 *  @see mean */
		double median() const;

		/** @brief Calculates the determinant using the Laplace expansion.
		 *  @returns the determinant of the matrix.
		 *  @details This operation has a runtime of O(\a n!) where \a n is the number of rows respectively columns.
		 *  	If the matrix is big, you should use the method {@link detGauss()} instead.
		 *  @throws notSquare_exception if the matrix is not a square matrix
		 *  @see detGauss */
		double detLaplace() const;

		/** @brief Calculates the determinant using Gaussian elimination.
		 *  @returns the determinant of the matrix.
		 *  @details This operation has a runtime of O(\a n^3) where \a n is the number of rows respectively columns.
		 *  @throws notSquare_exception if the matrix is not a square matrix */
		double detGauss() const;

		/** @brief Returns the Moore–Penrose pseudoinverse.
		 *  @details This method uses the singular value decomposition (@ref svd) to calculate the pseudoinverse.
		 *  	The runtime is O(n^3) if the Lapack algorithm is used.
		 *  	If this program or library is compiled defining the macro \a DONT_USE_LAPACK, then the runtime is about O(n^4). */
		Matrix <T> pseudo_inv() const;

		/** @brief Returns the inverse.
		 *  @returns the inverse of the matrix.
		 *  @throws notInvertible if the matrix is not invertible.
		 *  @throws notSquare_exception if the matrix is not a square matrix.
		 *  @details This algorithm uses the Gauss-Jordan elimination to calculate the inverse. The runtime is O(n^4). */
		Matrix <T> invert() const;

	/** @} */
	// == ZERLEGUNGEN ==
	/** @name Decompositions */
	/** @{ */

		/** @brief Cholesky decomposition (A = G * G^T)
		 *  @details This method calculated the cholesky decomposition of a passed matrix.
		 *  @pre The matrix must be symmetric and positiv definite.
		 *  	If this is not the case, a default constructed (0 x 0)-matrix is returned.
		 *  @param newMatrix
		 *  	determines if a new matrix is returned (\a true) or if the current matrix is edited (\a false).
		 *  @param clear
		 *  	If this parameter is false, the values above the main diagonal are not changed.
		 *  	If it is true (default value), the values are set to zero.
		 *  @returns
		 *  	a matrix \a G, so that G * G^T is equal to original "this" matrix. If there exists no such matrix,
		 *  	a (0 x 0)-matrix is returned instead.
		 *  @attention if \a newMatrix is true, you have to delete the returned matrix manually.
		 *  @details The source of the implemented algorithm is http://de.wikipedia.org/wiki/Cholesky-Zerlegung#Pseudocode */
		Matrix <T>* cholesky(bool newMatrix=true, bool clear=true);

		/** @brief Singular value decomposition (SVD)
		 *  @details This method decomposes the matrix \a A into the matrices U, S and V so that A = U*S*V^T.
		 *  	U and V are orthogonal matrices, S is an diagonal matrix. The entries on the main diagonal of S are called singular values.
		 *  	If \a A is an (n x m)-matrix, then U is (n x n), V^T is (m x m) and S is (n x m). \n
		 *  	This method returns the same result in the same runtime like <em> svd(Matrix<>::SVD_BEST); </em>.
		 *  @returns a struct that contains the matrix U, the transpose matrix of V and the singular values.
		 *  @see {@link svd(unsigned int)const svd(unsigned)}, svd_small, svd_lapack, \ref svdOptions */
		svdData<T> svd() const;

		/** @brief Singular value decomposition (SVD)
		 *  @details This method decomposes the matrix \a A into the matrices U, S and V so that A = U*S*V^T.
		 *  	U and V are orthogonal matrices, S is an diagonal matrix. The entries on the main diagonal of S are called singular values.
		 *  	The difference to \ref svd is that U, S and V are as small as possible: If \a A is an (n x m)-matrix and t is defined as std::min(n, m),
		 *  	then U is (n x t), V^T is (t x m) and S is (t x t). \n
		 *  	This method returns the same result in the same runtime like <em> svd(Matrix<>::SVD_BEST | Matrix<>::SVD_OPT_SMALL); </em>.
		 *  @returns a struct that contains the matrix U, the transpose matrix of V and the singular values.
		 *  @see {@link svd(unsigned int)const svd(unsigned)}, svd, svd_lapack, \ref svdOptions */
		svdData<T> svd_small() const;

		/** @brief Singular value decomposition (SVD)
		 *  @details This method decomposes the matrix \a A into the matrices U, S and V so that A = U*S*V^T.
		 *  	U and V are orthogonal matrices, S is an diagonal matrix. The entries on the main diagonal of S are called singular values.
		 *  @param system
		 *  	determines the way the SVD is calculated. See \ref svdOptions for more information.
		 *  @returns a struct that contains the matrix U, the transpose matrix of V and the singular values.
		 *  @see svd, svd_small, svd_lapack, \ref svdOptions */
		svdData<T> svd(unsigned int system) const;

		/** @brief Singular value decomposition (SVD)
		 *  @details This method decomposes the matrix \a A into the matrices U, S and V so that A = U*S*V^T.
		 *  	U and V are orthogonal matrices, S is an diagonal matrix. The entries on the main diagonal of S are called singular values. \n
		 *  	The SVD is calculated via a function of the Lapack library. The results are more accurate than
		 *  	those from the built-in algorithm and the calculation is faster. \n
		 *  	This method returns the same result in the same runtime like <em> svd(system | Matrix<>::SVD_CPPL); </em>.
		 *  @param system
		 *  	additional options. See \ref svdOptions for more information.
		 *  @returns a struct that contains the matrix U, the transpose matrix of V and the singular values.
		 *  @see {@link svd(unsigned int)const svd(unsigned)}, svd, svd_small, \ref svdOptions */
		svdData<T> svd_lapack(unsigned int system = 0) const;

		/** @brief QR decomposition
		 *  @details This method decomposes the matrix \a A into a product A = QR of an orthogonal matrix Q and an upper triangular matrix R.
		 *  @returns a pair which first entry corresponds to Q and which second entry corresponds to R. */
		std::pair<Matrix <T>, Matrix <T> > qr() const;

	/** @} */
	// == LINEARE GLEICHUNGSSYSTE ==
	/** @name Linear systems of equations */
	/** @{ */

		/** @brief Calculates the row echelon form
		 *  @details The matrix is converted to its row echelon form using the gaussian elimination.
		 *  	This is an intermediate step for solving the linear equation "this*x = b"
		 *  @param b
		 *  	The "right" site of the linear equation system.The parameter can be NULL or default constructed
		 *  	to denote that b is a zero vector or matrix.
		 *  @see rref */
		void ref(Matrix <T> *b = NULL);

		/** @brief Solves the linear equation system using the gaussian elimination
		 *  @details This method solves the equation system "this*x = b1...bn". After calculation
		 *  	"this" matrix will be in the reduced row echelon form, so \a b contains the solution.
		 *  	The vector \a b can have several columns to solve several systems at once. If the
		 *  	equation has exactly one solution, "this" matrix is an identity matrix and the vector \a b
		 *  	contains the solution \a x.
		 *  @param b
		 *  	The "right" site of the linear equation system. The parameter can be NULL or default constructed
		 *  	to denote that b is a zero vector or matrix.
		 *  @see ref, invert, pseudo_inv */
		void rref(Matrix <T> *b = NULL);

		/** @brief Solves the linear equation system using the Jacobi method
		 * 	@pre "This" matrix must be a square matrix.\n
		 * 		The matrix \a b must have one column with the same number of rows like "this".
		 *  @param b
		 *  	The "right" site of the linear equation system.
		 *  @returns the solution \a x of the equation "this * x = b".
		 *  @throws noConvergence if the algorithm does not converge. */
		std::vector<T> jacobi(const Matrix <T> *b) const;

		/** @brief Solves the linear equation system using the SOR method
		 * 	@pre "This" matrix must be a square matrix.\n
		 * 		The matrix \a b must have one column with the same number of rows like "this".
		 *  @param b
		 *  	The "right" site of the linear equation system.
		 *  @param omega
		 *  	relaxation factor. This parameter must be between 0 and 2.
		 *  @returns the solution \a x of the equation "this * x = b".
		 *  @throws noConvergence if the algorithm does not converge. */
		std::vector<T> sor(const Matrix <T> *b, T omega) const;

	/** @} */
	// == EIGEN-ALGORITHMENE ==
	/** @name Eigen algorithms */
	/** @{ */

		/** @brief Transforms the matrix to the hessenberg form
		 *  @details This method uses Householder transformations to transform "this" matrix to the hessenberg form.
		 *  	If "this" matrix is symmetric, the result of this function is a tridiagonal matrix.
		 *  @param system
		 *  	If the input matrix is symmetric and if (\a system | \ref MAT_SYM) is bigger than zero, the returned matrix will be symmetric too.
		 *  	If the input matrix is symmetric, but system is zero, there could possibly be inaccuracies, so the returned matrix is not exactly symmetric.
		 *	@see MAT_SYM, qr_algo */
		void hessenberg(unsigned int system);

		/** @brief QR algorithm (to calculate the eigenvalues)
		 *  @details The QR algorithm uses the QR decomposition to calculate the eigenvalues and optionally the eigenvectors.
		 *  @param system
		 *  	determines the way the QR algorithm is calculated, for example pre processing and shifts. See \ref qraOptions for more information.
		 *  @returns a pair which first element contains the eigenvalues on its main diagonal and which second element contains the eigenvectors
		 *  	(unless the option \ref QRA_FASTQR is used. Then instead of the eigenvectors a default constructed (0 x 0)-matrix is returned).
		 *  @see qr, {@link svd(unsigned int)const svd}, hessenberg, \ref qraOptions */
		std::pair<Matrix <T>, Matrix<T> > qr_algo(unsigned int system) const;

		/* Eigenwert- und -vektorberechnung
		 * Die Eigenwerte m�ssen reell sei! */
		/** @brief Calculation of the eigenvalues
		 *  @details Only the real eigenvalues are calculated and returned.
		 *  @attention If the matrix has complex, not real eigenvalues, the result of this function is undefined.
		 *  @param system
		 *  	determines the way the eigenvalues are calculated. See \ref svdOptions for more information.
		 *  @see eigen, qr_algo */
		std::vector<T> eigenvalues(unsigned int system) const;

		/** @brief Calculation of the eigenvalues and eigenvectors.
		 *  @details This method can only calculated real eigenvalues.
		 *  @attention If the matrix has complex, not real eigenvalues, the result of this function is undefined.
		 *  @param system
		 *  	determines the way the eigenvalues and eigenvectors are calculated. See \ref svdOptions for more information.
		 *  @see eigenvalues, eigenData */
		eigenData<T> eigen(unsigned int system) const;

	/** @} */
	// == SONSTIGES ==
	/** @name Miscellaneous */
	/** @{ */

		/** @brief Householder transformation
		 *  @details This method returns a (\a size x \a size)-matrix H = I - 2 / (v^T * v) * (v * v^T),
		 *  	where I is an (\a size x \a size) identity matrix and v is a vector specified by the parameter \a vector.
		 *  @param vector
		 *  	pointer to an array of a least \a size elements
		 *  @param size
		 *  	The number of elements saved in \a vector
		 *  @returns a householder transformation matrix.
		 *  @see hessenberg */
		static Matrix <T> householder(const T* vector, int size);

		/** @brief Linear regression
		 *  @details This method calculated x, so that ||this*x - b|| is minimal.
		 *  	The algorithm works as follows: First, the singular value decompositon (see {@link svd(unsigned int)const svd})
		 *  	is computed to determine the pseudo inverse of "this" matrix.
		 *  	Afterwards the result x is computed by the multiplication of the pseudo inverse by \a b.
		 *  @pre The number of rows of \a b must be equal to the number of rows of "this" matrix. \n
		 *  	The matrix \a b must have exactly one column.
		 *  @param b
		 *  	a column vector that represents the right site of the equation this*x = b which error will be minimized.
		 *  @returns the column vector x that minimizes the term ||this*x - b||
		 *  @see {@link svd(unsigned int)const svd} */
		Matrix <T> regression(const Matrix <T> *b) const;

		/** @brief Calculates the covariance matrix
		 *  @details This method calculates the covariance matrix of an (n x d)-matrix, where n is the number of data points and d is its dimension.
		 *  @param normIn
		 *  	Input normalization. If true, the arithmetic mean of each column is subtracted from all entries of that column.
		 *  @param normOut
		 *  	Output normalization. If true, every entry of the covariance matrix is devided by (n-1).
		 *  @returns the covariance matrix of the input matrix. */
		Matrix <T> covariance(bool normIn = true, bool normOut = true) const;

	/** @} */
	// == UMWANDLUNGEN ==
	/** @name Conversions */
	/** @{ */

#ifndef DONT_USE_LAPACK
		/** @brief Converts the CPPL-Matrix into the standard matrix format of this library.
		 *  @param cppl_matrix
		 *  	The CPPL-Matrix that will be converted
		 *  @returns a matrix with the same content as the passed matrix.
		 *  @details Usually the template parameter \a T is \a double. If another type is used, \a T has to be implicitly convertible from \a double.
	 	 *  @see toCPPL */
		static Matrix <T> fromCPPL(const CPPL::dgematrix& cppl_matrix);

		/** @brief Converts the matrix into a CPPL-double-matrix.
		 *  @returns a CPPL-matrix of the type "double" with the same content as "this" matrix.
		 *  @details Usually the template parameter \a T is \a double. If another type is used, \a T has to be implicitly convertible to \a double.
		 *  @see fromCPPL */
		CPPL::dgematrix toCPPL() const;
#endif

		/** @brief Converts the std::vector to an row vector respectively matrix.
		 *  @param vec
		 *  	The std::vector whose content will be copied to a matrix object.
		 *  @returns a (1 x n)-matrix with the same content as the passed std::vector.
		 *  @see fromVectorColumn, toVector, {@link operator[](size_t) operator[]} */
		static Matrix<T> fromVectorRow(const std::vector<T>& vec);

		/** @brief Converts the std::vector to an column vector respectively matrix.
		 *  @param vec
		 *  	The std::vector whose content will be copied to a matrix object.
		 *  @returns a (n x 1)-matrix with the same content as the passed std::vector.
		 *  @see fromVectorRow, toVector, {@link operator[](size_t) operator[]} */
		static Matrix<T> fromVectorColumn(const std::vector<T>& vec);

		/** @brief Converts the matrix to a std::vector.
		 *  @returns a std::vector with the same content as the matrix.
		 *  @throw dim_exception if neither the number of rows nor the number of columns of the matrix is equals to one.
		 *  @see fromVectorRow, fromVectorColumn, toCPPL, {@link operator[](size_t) operator[]} */
		std::vector<T> toVector() const;

	/** @} */
	// == DATEI-OPERATIONEN ==
	/** @name File and print operations */
	/** @{ */

		/** @brief Writes a small version of the matrix to the specified stream.
		 *  @param strm
		 *  	The stream in which the matrix will be written. The default value is the standard output stream.
		 *  @param prefix
		 *  	A prefix that will be written first. The default value is an empty string.
		 *  @param max_dim1
		 *  	The maximum number of rows that will be written.
		 *  @param max_dim2
		 *  	The maximum number of columns that will be written.
		 *  @details The prefix is printed first followed by the dimension of the matrix.
		 *  	After this the content is written with the specified maximum number of rows and columns.
		 *  	The stream is flushed after this operation. */
		void print(std::ostream &strm /* = std::cout*/, const char* prefix = "", int max_dim1 = 5, int max_dim2 = 5) const;

		/** @brief Writes a small version of the matrix to the specified stream.
		 *  @param prefix
		 *  	A prefix that will be written first. The default value is an empty string.
		 *  @param max_dim1
		 *  	The maximum number of rows that will be written.
		 *  @param max_dim2
		 *  	The maximum number of columns that will be written.
		 *  @param strm
		 *  	The stream in which the matrix will be written. The default value is the standard output stream.
		 *  @details The prefix is printed first followed by the dimension of the matrix.
		 *  	After this the content is written with the specified maximum number of rows and columns.
		 *  	The stream is flushed after this operation. */
		void print(const char* prefix, int max_dim1 /*= 5*/, int max_dim2/* = 5*/, std::ostream &strm /*= std::cout*/) const;

		/** @brief Writes the matrix into a file.
		 *  @param path
		 *  	The path of the file that will be written.
		 *  @param mode
		 *  	The format of the serialized matrix. See \ref rwOptions for the supported modes.
		 *  @see read, writeStandard, writeBinary, writeR, writeCSV */
		void write(const char* path, char mode) const;

		/** @brief Reads a matrix from a file.
		 *  @param path
		 *  	The path of the file that will be read.
		 *  @param mode
		 *  	The format of the serialized matrix. See \ref rwOptions for the supported modes.
		 *  @returns the read matrix.
		 *  @see write, readStandard, readBinary, readR, readCSV */
		static Matrix <T> read(const char* path, char mode);

		/** @brief Writes the matrix into a file in the standard output format.
		 *  @param path
		 *  	The path of the file that will be written.
		 *  @details This method uses the standard output format, so it is human readable. Firstly the dimension of the matrix is written, followed by its content.
		 *  	Only the most important digits are saved, thus the file is smaller, however there will be a loss of precision if you convert the file to a matrix again.
		 *
		 *  @see readStandard, writeBinary, writeR, writeCSV */
		void writeStandard(const char* path) const;

		/** @brief Reads the matrix from a file in the standard output format.
		 *  @param path
		 *  	The path of the file that will be read.
		 *  @returns the matrix which the file contains.
		 *  @details The file must be written with \ref writeStandard to return correct results. Otherwise the behaviour is undefined.
		 *  @see writeStandard, readBinary, readR, readCSV */
		static Matrix<T> readStandard(const char* path);

		/** @brief Writes the matrix into a binary file.
		 *  @param path
		 *  	The path of the file that will be written.
		 *  @details This method writes the matrix in a binary (not human readable) format to a file, therefore the created file is very small.
		 *  @see readBinary, writeStandard, writeR, writeCSV */
		void writeBinary(const char* path) const;

		/** @brief Reads the matrix from a binary file.
		 *  @param path
		 *  	The path of the file that will be read.
		 *  @returns the matrix which the file contains.
		 *  @details The file must be written with \ref writeBinary to return correct results. Otherwise the behaviour is undefined.
		 *  @see writeBinary, readStandard, readR, readCSV */
		static Matrix<T> readBinary(const char* path);

		/** @brief Writes the matrix into a file in the standard format used by the program R.
		 *  @param path
		 *  	The path of the file that will be written.
		 *  @details Matrices written by this method can be easily read by the program R.
		 *  @see readR, writeStandard, writeBinary, writeCSV */
		void writeR(const char* path) const;

		/** @brief Reads the matrix from a file in the R-format.
		 *  @param path
		 *  	The path of the file that will be read.
		 *  @returns the matrix which the file contains.
		 *  @details The file must be written with \ref writeR or with the program R to return correct results. Otherwise the behaviour is undefined.
		 *  @see writeR, readStandard, readBinary, readCSV */
		static Matrix<T> readR(const char* path);

		/** @brief Writes the matrix into a CSV-file.
		 *  @param path
		 *  	The path of the file that will be written.
		 *  @param sep
		 *  	Column separator. Default value is ';'.
		 *  @param comma
		 *  	Decimal Point. Default value is ','.
		 *  @details The columns of the matrix are separated by \a sep. For the decimal point the sign \a comma is used.
		 *  	If you use the default values you can open the created file with the German version of Microsoft Excel.
		 *  @see readCSV, writeStandard, writeBinary, writeR */
		void writeCSV(const char* path, char sep = ';', char comma = ',') const;

		/** @brief Reads the matrix from a CSV-file.
		 *  @param path
		 *  	The path of the file that will be read.
		 *  @param sep
		 *  	Column separator. Default value is ';'.
		 *  @param comma
		 *  	Decimal Point. Default value is ','.
		 *  @returns the matrix which the file contains.
		 *  @details The file must be a valid CSV file without a header. Otherwise the behaviour is undefined.
		 *  @see writeCSV, readStandard, readBinary, readR */
		static Matrix<T> readCSV(const char* path, char sep = ';', char comma = ',');

	/** @} */
	// == KONSTANTEN FÜR PARAMETER ==
	/** @name Constants for parameters */
	/** @{ */
		/** @anchor qraOptions */

		/** @brief Denotes that the input matrix is symmetric
		 *  @details This is an optional flag for algorithms that can run faster or can produce more accurate results. \n
		 *  	If you use this flag, the input matrix \a m should satisfy the following condition for all valid \a i and \a j:
		 *  	m(i,j) == m(j,i)
		 *  @see hessenberg, qr_algo */
		const static unsigned int MAT_SYM = 0x1;

		/** @brief Bit postions for the QR algorithm shift options.
		 *  @details The bits that are set in this constant are used by the function \ref qr_algo to determine the applied shift.
		 *  	A good shift method causes a fast convergent of the QR algorihtm.
		 *  @see qr_algo, QRA_SHIFT_ZERO, QRA_SHIFT_ANN, QRA_SHIFT_SYM */
		const static unsigned int QRA_SHIFT_ = 0x2 | 0x4 | 0x8;

		/** @brief Used shift in the QR algorithm: No shift
		 *  @details Very slow convergent.
		 *  @see qr_algo, QRA_SHIFT_, QRA_SHIFT_ANN, QRA_SHIFT_SYM */
		const static unsigned int QRA_SHIFT_ZERO = 0x2;

		/** @brief Used shift in the QR algorithm: Simple shift
		 *  @details Converges faster than \ref QRA_SHIFT_ZERO. If the input matrix is symmetric, use \ref QRA_SHIFT_SYM and \ref MAT_SYM instead.
		 *  @see qr_algo, QRA_SHIFT_, QRA_SHIFT_ZERO, QRA_SHIFT_SYM */
		const static unsigned int QRA_SHIFT_ANN = 0x4;

		/** @brief Used shift in the QR algorithm: Shift for symmetric matrices.
		 *  @details Converges quadratically. This option only works for symmetric matrices. If the input matrix is not symmetric the convergent of the qr algorithm is not proven.
		 *  @see qr_algo, QRA_SHIFT_, QRA_SHIFT_ZERO, QRA_SHIFT_ANN */
		const static unsigned int QRA_SHIFT_SYM = 0x6;

		/** @brief In the QR algorithm the input matrix is firstly converted into Hessenberg form.
		 *  @details The algorithm converges faster, if this option is used.
		 *  @pre If you use this option, you have to use \ref QRA_FASTQR necessarily too.
		 *  @see qr_algo, QRA_FASTQR */
		const static unsigned int QRA_HESSENBERG = 0x10;

		/** @brief Very fast QR decomposition in the QR algorithm.
		 *  @details If this option is used, the QR decompositon is calculated very fast. However, the eigen vectors cannot be calculated directly.
		 *  @pre If you use this option, you have to use \ref QRA_HESSENBERG and \ref MAT_SYM necessarily too.
		 *  @see qr_algo, QRA_HESSENBERG, MAT_SYM */
		const static unsigned int QRA_FASTQR = 0x20;

		/** @anchor svdOptions */
		/* emfohle Konfigurationen zur Bestimmung der SVD
		 *     SVD_STD:    langsamer als SVD_FAST, direkte Berechnung der Eigenvektoren
		 *     SVD_CPPL:   CPP-Lapack-Funktion benutzen
		 *     SVD_FAST:   sehr schnell und stabil
		 *     SVD_BEST:   [empfohlen] bestes Verfahren, momentan SVD_CPPL */
		/** @brief Standard configuration for the singular value decomposition.
		 *  @details This configuration is obsolete. Use \ref SVD_CPPL (if Lapack is supported) or \ref SVD_FAST instead. Both configurations are faster than this.
		 *  @see {@link svd(unsigned int)const svd}, SVD_CPPL, SVD_FAST, SVD_BEST */
		const static unsigned int SVD_STD    = MAT_SYM | QRA_SHIFT_SYM | QRA_HESSENBERG;

		/** @brief Singular value demposition using Lapack.
		 *  @details If you use this option the singular value decomposition is calculated by the Lapck library.
		 *  	If this package is compiled with the definition DONT_USE_LAPACK, so that there is no function from the
		 *  	Lapack library that be called, the configuration \ref SVD_FAST is used instead.
		 *  @see {@link svd(unsigned int)const svd}, SVD_FAST, SVD_BEST */
		const static unsigned int SVD_CPPL   = 0x40;

		/** @brief Fast configuration for the singular value decomposition.
		 *  @details This configuration uses the internal algorithm to compute the {@link svd(unsigned int)const svd}.
		 *  	However, the Lapck library function for calculating the singular value decomposition is faster,
		 *  	so use \ref SVD_CPPL for faster results.
		 *  @see {@link svd(unsigned int)const svd}, SVD_CPPL */
		const static unsigned int SVD_FAST   = MAT_SYM | QRA_SHIFT_SYM | QRA_HESSENBERG | QRA_FASTQR;

		/** @brief Best configuration for the singular value decomposition.
		 *  @details This option will use the best known configuration. So if the Lapack library functions are available, they will be used.
		 *  	Otherwise the configuration \ref SVD_FAST will be applied.
		 *  @see {@link svd(unsigned int)const svd}, SVD_FAST, SVD_CPPL */
		const static unsigned int SVD_BEST   = SVD_CPPL;

		/** @brief Creation of small matrices in the singular value decompositon.
		 *  @details If this option is used additionally, the returned matrices by {@link svd(unsigned int)const svd} have the following dimensions:
		 *  	If the input matrix is an (m x n)-matrix, then \a U is an (m x min)-matrix and \a V^T is a (min x n)-matrix, where min is the minimum of m and n. \n
		 *  	This options should be used together (connected with bitwise OR) with \ref SVD_FAST, \ref SVD_CPPL or \ref SVD_BEST.
		 *  @see {@link svd(unsigned int)const svd}, SVD_CPPL, SVD_FAST, SVD_BEST */
		const static unsigned int SVD_OPT_SMALL = 0x80;

		/* Read/Write-Optionen */
		/** @anchor rwOptions */

		/** @brief Standard read and write format.
		 *  @details If this option is used for the method \ref read or \ref write, then the behaviour is the same like \ref readStandard ord \ref writeStandard.
		 *  @see read, write, readStandard, writeStandard */
		const static char RW_STANDARD = 'S';

		/** @brief Binary read and write format.
		 *  @details If this option is used for the method \ref read or \ref write, then the behaviour is the same like \ref readBinary ord \ref writeBinary.
		 *  @see read, write, readBinary, writeBinary */
		const static char RW_BINARY = 'B';

		/** @brief Read and write format of the program R.
		 *  @details If this option is used for the method \ref read or \ref write, then the behaviour is the same like \ref readR ord \ref writeR.
		 *  @see read, write, readR, writeR */
		const static char RW_R = 'R';

		/** @brief CSV read and write format.
		 *  @details If this option is used for the method \ref read or \ref write, then the behaviour is the same like \ref readCSV ord \ref writeCSV.
		 *  @see read, write, readCSV, writeCSV */
		const static char RW_CSV = 'C';

	/** @} */
	// == BERECHNUNGS-KONSTANTEN ==
	/** @name Constants for calculations */
	/** @{ */

		/* Genauigkeitskonstanten (Accuracy (ACC))
		 * Definition: Die Elemente der Matrix A haben die Gr��enordnung g := frobnorm() / dim1_ / dim2_
		 *
		 * ACC_SVD_EIGEN_ZERO_DIV: Werte Eigenwerte die (betragsm��ig) kleiner als g / div sind, als null.
		 * ACC_QRA_STOP_DIV:       Sobald alle Werte unterhalb der Hauptdiagonalen (betragsm��ig) kleinre als g / div sind, so breche den Algorithmus ab
		 * ACC_QRA_FASTQR_ONE_DIV: Wenn "sqcur" zwischen 1 +- epsilon^2 / div liegt, so werte es als 1 (und f�hre einen Sonderfall aus. Epsilon ist gleich "g / ACC_SVD_EIGEN_ZERO_DIV"
		 * ACC_REF_EQUAL_DIV:      Wenn die Differenz zweiter Werte fast null ist, so werte es als 0. Fast null bedeutet, dass Quotient aus dem Gr��eren und Kleineren kleiner als 1 + 1/div ist.
		 * ACC_JACOBI_FAC:         Wenn alle Differenzen kleiner als g/fac werden, reicht die Genauigkeit aus. Wenn eine Differenz gr��er als frobnorm()*fac ist, so bricht der Algorithmus ab, da er nicht terminiert.
		 * ACC_PINV_ZERO_DIV:      Singul�rwerte, die kleiner der gr��te Singul�rwert durch div ist, werden als 0 gewertet. */

		/** @brief Accuracy constant determining when an eigen value is set to zero
		 *  @details Eigen values which absolute value is lower than the Frobenius norm of the matrix divided by both dimensions and by the value of this constant, are set to zero.
		 *	@see eigenvalues, eigen, {@link svd(unsigned int)const svd} */
		static double ACC_SVD_EIGEN_ZERO_DIV;

		/** @brief Accuracy constant determining when the QR algorithm terminates.
		 *  @details This constant is used in the QR algorithm: As soon as the absolute values of all entries below main diagonal are lower than
		 *  	the Frobenius norm of the matrix divided by both dimensions and by the value of this constant, the algoritm terminates and
		 *  	returns the calculated eigen values.
		 *  @see qr_algo */
		static double ACC_QRA_STOP_DIV;

		/** @brief Accuracy constant for the Fast QR decomposition.
		 *  @details This constant is used in the QR algorithm, if the option \ref QRA_FASTQR is used.
		 *  	If the variable sqcur is between <em> 1 ± epsilon^2 / div </em>, then set sqcur to 1 (and execute a special case of the algorithm). \a div is the Value of this constant and
		 *  	\a epsilon is equal to the Frobenius norm of the matrix divided by both dimensions and by the value of \ref ACC_SVD_EIGEN_ZERO_DIV.
		 *  @see qr_algo, ACC_QRA_STOP_DIV, ACC_SVD_EIGEN_ZERO_DIV, \ref qraOptions */
		static double ACC_QRA_FASTQR_ONE_DIV;

		/** @brief Accuracy constant determining when the difference of two values is set to zero.
		 *  @details This constant is used in \ref ref and \ref rref. If the difference of two values is almost zero, than the difference is set to zero.
		 *  	"Almost" means in this case, that the bigger value divided by the lower value is lower than <em> 1 + 1/div </em>, where \a div is the value of this constant.
		 *  @see ref, rref */
		static double ACC_REF_EQUAL_DIV;

		/** @brief Accuracy constant for the Jacobi algorithm.
		 *  @details If all differences are lower than the Frobenius norm of the matrix divided by both dimensions and by the value of this constant,
		 *  	the accuracy is high enough and algorithm terminates. If the differences are bigger than the Frobenius norm of the matrix divided by both dimensions and
		 *  	multiplied with the value of this constant, the algorithm aborts, because it does not converge.
		 *  @see jacobi, sor */
		static double ACC_JACOBI_FAC;

		/** @brief Accuracy constant for the calculation of the pseudo inverse
		 *  @details singular values that are lower than the biggest singular value divided by this value of this constant, are set to zero.
		 *  @see pseudo_inv, {@link svd(unsigned int)const svd} */
		static double ACC_PINV_ZERO_DIV;

		/** @brief Constant which determines when the naive multiplication algorithm is used instead of the Strassen algorithm.
		 * 	@details The naive multiplication algorithm is used, if any dimension is lower or equal 1 or if the following condition is true: \n
		 * 		((double) dimI * dimK * dimJ) / (dimI * dimK + dimK * dimJ + dimI * dimJ) <= MULTIPLY_MIX_GRENZE + 1 \n
		 * 		where dimI and dimK are dimensions of the first matrix and dimK and dimJ of the second matrix.
		 * 	@see multiplyMix, multiplyNaive */
		static double MULTIPLY_MIX_GRENZE;

		/** @brief Empty (0 x 0)-matrix.
		 *  @details This constant contains an empty (0 x 0)-matrix created by the {@link Matrix() default constructor}. */
		const static Matrix<T> EMPTY;

	/** @} */

private:

	// == ARRAY-KONSTANTEN ==
	/** @anchor array_constants
	 *  @name Array Constants
	 *
	 *  This constants are used for the private attribute \a createArray_.
	 *  They denote if the matrix has an own data field or if it accesses the data field of another matrix. */
	/** @{ */

		/** @brief The matrix has no own array.
		 *  @details This constant denote that the matrix accesses the data field of another matrix.
		 *  	If the destructor is called, no array is deleted. */
		const static int NO_ARRAY = 0;

		/** @brief The matrix owns the primary array, but uses the secondary arrays of another matrix.
		 *  @details If the destructor is called, only the primary is deleted. */
		const static int SHALLOW_ARRAY = 1;

		/** @brief The matrix owns the complete data field.
		 *  @details If the destructor is called, the whole data field is deleted.
		 *  	If another matrix accesses the data field of this matrix, it will be get unvalid after deletion of this matrix. */
		const static int COMPLETE_ARRAY = 2;

	/** @} */

	// == PRIVATE METHODEN ==

		/** @brief Copies the passed matrix.
		 *  @param m
		 *  	the matrix that will be copied.
		 *  @param createArray
		 *  	determines whether the data field is copied completely, shallowly or not at all.
		 *  @details The value of \a createArray determines if the data field is copied completely (\ref COMPLETE_ARRAY) or
		 *  	if the new matrix accesses the data field of \a m (\ref NO_ARRAY). With the option \ref SHALLOW_ARRAY only a primary array is created.
		 *  @see @ref array_constants */
		Matrix(const Matrix<T>& m, int createArray);

		/** @brief Creates an uninitialized matrix.
		 *  @param dim1
		 *  	The number of rows.
		 *  @param dim2
		 *  	The number of columns.
		 *  @param createArray
		 *  	determines whether the data field is copied completely, shallowly or not at all.
		 *  @details if \a createArray is not equal \ref COMPLETE_ARRAY,
		 *  	then it is not allowed to access the data field of the matrix
		 *  	until \ref entries_ has a correct value.
		 *  	Otherwise the behaviour is undefined.
		 *  @see @ref array_constants */
		Matrix(int dim1, int dim2, int createArray);

		/** @brief Naive unchecked matrix multiplication with no optimizations
		 *  @param matrix
		 *  	The matrix that will be multiplied (second argument).
		 *  @returns "this" multiplied by \a mat.
		 *  @details This method undertakes no security checks.
		 *  	If the number of columns of "this" is not equal to the number of rows of \a mat, the lower value is used.
		 *  	This method uses the naive way of multiplication. The runtime is O(n^3). \n
		 *  	Compared to \ref multiplyNaiveUnchecked this method undertakes no optimizations relating to the order of the data of both matrices.
		 *  	Therefore it is maybe a bit faster when the matrices are very small.
		 *  @see multiplyNaiveUnchecked */
		Matrix<T> multiplyNaiveUncheckedSimple(const Matrix<T>* matrix) const;

	// == ATTRIBUTE ==
		/** @brief Denotes if the matrix has an own array or if it accesses the data field of another matrix.
		 *  @details If the value of this attribute is \ref NO_ARRAY, then the matrix has no own data field.
		 *  	Therefore no array will be deleted if the destructor is called.
		 *  	However, if the destructor of the matrix whose data field is accessed by "this" matrix is called,
		 *  	"this" matrix will be invalid. Accessing its elements will cause undefined behaviour until \a entries_ points
		 *  	to a new valid data field, for example via \ref swap. \n\n
		 *
		 *  	If the value of this attribute is \ref SHALLOW_ARRAY, then only the primary array belongs to this matrix.
		 *  	The primary array contains pointer to the secondary arrays which belongs to another matrix in this case.
		 *  	When the destructor of this matrix is called, only the primary array is deleted. The destruction of the accessed matrix
		 *  	will cause the same behaviour respectively problems as if this attribute is equal to \ref NO_ARRAY. \n\n
		 *
		 *  	The value \ref COMPLETE_ARRAY denotes that this matrix owns an own whole data field. If the destructor is called,
		 *  	both the primary and the secondary arrays are deleted. Accessing this matrix after the destruction of other matrices cannot cause any problems.
		 *  @see ~Matrix */
		int createArray_;

		/** @brief The number of rows of the matrix */
		int dim1_;

		/** @brief The number of columns of the matrix */
		int dim2_;

		/** @brief Data field.
		 *  @details The data field consists of a primary array which entries points to secondary arrays that contains the data.
		 *  	The data field can be implicitly transposed. If \ref transposed_ is false, then the primary array has a dimension of \ref dim1_ and
		 *  	the secondary arrays a dimension of \ref dim2_. If it is true, the dimensions are swapped.
		 *  @attention Do not access this attribute directly because of \ref transposed_.
		 *  	Use \ref getEntry, \ref updateEntry or {@link operator()(int, int) operator()} instead. */
		T** entries_;

		/** @brief Denotes if the matrix is implicitly transposed.
		 *  @details If this attribute is true, the data field \ref entries_ is firstly accessed by the column number
		 *  	and secondly by the row number. If it is false, the data field is normally accessed (row number first, then column number) */
		bool transposed_;

	// == STATISCHE HILFSFUNKTIONEN ==
		/** @brief Creates a data field.
		 *  @details This function creates a data field for matrices. A data field consists of a primary array with pointers to the first elements of the secondary arrays.
		 *  	The parameter \a createArray determines if all arrays are created (\ref COMPLETE_ARRAY) or if only the primary array is created (\ref SHALLOW_ARRAY) or if no array is created (\ref NO_ARRAY).
		 *  @param dim1
		 *  	Primary dimension of the array-array-structure.
		 *  @param dim2
		 *  	Secondary dimension of the array-array-structure.
		 *  @param createArray
		 *  	determines whether the data field is created completely, shallowly or not at all.
		 *  @returns a pointer to the first element of the primary array or \a NULL if createArray is equal to \ref NO_ARRAY.
		 *  @throw std::bad_alloc if there is not enough memory to create the specified arrays.
		 *  @see cleanup, entries_ */
		static T** createField(int dim1, int dim2, int createArray = COMPLETE_ARRAY);

		/** @brief Cleans up an array created by \ref createField.
		 *  @details This function deletes the array-array-structure created by the function \ref createField. If \a createArray is equal to \ref COMPLETE_ARRAY
		 *  	the whole array (primary and secoundary arrays) is deleted. If \a createArray is equal to \ref SHALLOW_ARRAY only the primary array is deleted.
		 *  	This can cause memory leaks, if there is no matrix that can access the secondary arrays after deletion.
		 *  	If \a createArray is equal to \ref NO_ARRAY, this function does nothing.
		 *  @param data
		 *  	The data field that will be deleted.
		 *  @param size
		 *  	The size of the primary array.
		 *  @param createArray
		 *  	determines whether the data field is deleted completely, shallowly or not at all.
		 *  @attention
		 *  	The parameter \a size is not always the first dimension of the matrix, since the matrix can be implicitly transposed.
		 *  @attention
		 *  	If another matrix accesses the data field that is deleted by this function, the matrix will get invalid. Accessing its elements after the call of
		 *  	this function cause undefined behaviour.
		 *  @see createField, entries_, transposed_ */
		static void cleanup(T** data, int size, int createArray = COMPLETE_ARRAY);

};


// == OPERATORS ==
/** @anchor operators */

/** @brief Prints the matrix to the specified stream
 *  @details First, the dimension is printed, followed by the data field.
 *  	Each line is surronded by square bracket. The values are separated with commas.
 *  @param strm
 *  	The matrix is printed to this stream
 *  @param m
 *  	The matrix that will be printed
 *  @returns the stream passed with the parameter
 *  @see Matrix::print */
template <class T>
std::ostream& operator<<(std::ostream &strm, const Matrix<T> &m);

/** @brief Checks if two matrices are equal.
 *  @returns true, if and only if both matrices have the same dimension and the same content. */
template <class T>
bool operator==(const Matrix<T> &a, const Matrix<T> &b);

/** @brief Checks if two matrices are not equal.
 *  @return false, if and only if both matrices have the same dimension and the same content. */
template <class T>
bool operator!=(const Matrix<T> &a, const Matrix<T> &b);

/** @brief Adds matrices.
 *  @details This function adds two matrics and returns the result. If the passed
 *  	matrices have different dimensions, the result matrix get the bigger ones
 *  	and the missing values are interpreted as zero.
 *  @returns a matrix c, so that c.getEntryCheck(i,j) = a.getEntryCheck(i,j) + b.getEntryCheck(i,j)
 *  @see Matrix::getEntryCheck, {@link Matrix::addMatrix(const Matrix&, int, int, int, int, int, int)const Matrix::addMatrix} */
template <class T>
Matrix<T> operator+(const Matrix<T> &a, const Matrix<T> &b);

/** @brief Subtracts matrices.
 *  @details This function subtracts two matrics and returns the result. If the passed
 *  	matrices have different dimensions, the result matrix get the bigger ones
 *  	and the missing values are interpreted as zero.
 *  @returns a matrix c, so that c.getEntryCheck(i,j) = a.getEntryCheck(i,j) - b.getEntryCheck(i,j)
 *  @see Matrix::getEntryCheck, {@link Matrix::subMatrix(const Matrix&, int, int, int, int, int, int)const Matrix::subMatrix} */
template <class T>
Matrix<T> operator-(const Matrix<T> &a, const Matrix<T> &b);

/** @brief Matrix multiplication.
 *  @returns the same result like a.multiplyMix(&b);
 *  @see Matrix::multiplyMix */
template <class T>
Matrix<T> operator*(const Matrix<T> &a, const Matrix<T> &b);

/** @brief Adds a matrix to another.
 *  @pre Both matrices must have the same dimension.
 *  @details Adds the matrix b to the matrix a.
 *  @returns a reference to a.
 *  @see Matrix::addMatrix */
template <class T>
Matrix<T> & operator+=(Matrix <T> &a, const Matrix<T> &b);

/** @brief Subtracts a matrix from another.
 *  @pre Both matrices must have the same dimension.
 *  @details Subtracts the matrix b from the matrix a.
 *  @returns a reference to a.
 *  @see Matrix::subMatrix */
template <class T>
Matrix<T> & operator-=(Matrix <T> &a, const Matrix<T> &b);

/** @brief Matrix multiplication.
 *  @returns the same result like a = a.multiplyMix(&b);
 *  @see Matrix::multiplyMix */
template <class T>
Matrix<T> & operator*=(Matrix<T> &a, const Matrix<T> &b);

/** @brief Returns the transpose matrix of the specified matrix.
 *  @details Transposes the matrix implicitly. This means that the
 *  	returned matrix accesses the data field of the passed matrix.
 *  @returns the same result like a.getTranspose();
 *  @see Matrix::getTranspose */
template <class T>
const Matrix<T> operator~(const Matrix<T> &a);

/** @brief Pseudo inverse.
 *  @details Calculates and returns the pseudo inverse of the passed matrix.
 *		See \ref Matrix::pseudo_inv for more information.
 *	@see Matrix::pseudo_inv */
template <class T>
Matrix<T> operator!(const Matrix<T> &a);

/** @brief Adds a vector and a matrix
 *  @pre One dimension of the matrix must be one.
 *  @returns a matrix c, so that c[i] = a[i] + v[i].
 *  @see Matrix::operator[], Matrix::toVector, Matrix::fromVectorRow, Matrix::fromVectorColumn */
template <class T>
Matrix<T> operator+(const Matrix<T> &a, const std::vector<T> &v);

/** @brief Substracts a vector from a matrix
 *  @pre One dimension of the matrix must be one.
 *  @returns a matrix c, so that c[i] = a[i] - v[i].
 *  @see Matrix::operator[], Matrix::toVector, Matrix::fromVectorRow, Matrix::fromVectorColumn */
template <class T>
Matrix<T> operator-(const Matrix<T> &a, const std::vector<T> &v);

/** @brief Multiplies a matrix with a vector.
 *  @details If the second dimension of the matrix is one, then the scalar product is calculated. \n
 *  	If the first dimension of the matrix is one, then the tensor product is calculated. \n
 *  	If no dimension of the matrix is one, then the matrix multiplied with a row vector.
 *  @returns a matrix which is the result of \a a multiplied with \a v.
 *  @see Matrix::toVector, Matrix::fromVectorRow, Matrix::fromVectorColumn */
template <class T>
Matrix<T> operator*(const Matrix<T> &a, const std::vector<T> &v);

/** @brief Adds a value to all matrix entries.
 *  @returns a matrix c, so that c(i,j) = mat(i,j) + a; */
template <class T>
Matrix<T> operator+(const Matrix<T> &mat, T a);

/** @brief Subtracts a value from all matrix entries.
 *  @returns a matrix c, so that c(i,j) = mat(i,j) - a; */
template <class T>
Matrix<T> operator-(const Matrix<T> &mat, T a);

/** @brief Multiplies all matrix entries with the specified value.
 *  @returns a matrix c, so that c(i,j) = mat(i,j) * a; */
template <class T>
Matrix<T> operator*(const Matrix<T> &mat, T a);

/** @brief Divides all matrix entries by the specified value.
 *  @returns a matrix c, so that c(i,j) = mat(i,j) / a; */
template <class T>
Matrix<T> operator/(const Matrix<T> &mat, T a);

/** @brief Adds a value to all matrix entries.
 *  @returns a reference to the changed input matrix.
 *  @details The value \a a is added to all entries of the matrix. */
template <class T>
Matrix<T> & operator+=(Matrix<T> &mat, T a);

/** @brief Subtracts a value from all matrix entries.
 *  @returns a reference to the changed input matrix.
 *  @details The value \a a is subtracted from all entries of the matrix. */
template <class T>
Matrix<T> & operator-=(Matrix<T> &mat, T a);

/** @brief Multiplies all matrix entries with a value.
 *  @returns a reference to the changed input matrix.
 *  @details All entries of the matrix are multiplied with the value \a a. */
template <class T>
Matrix<T> & operator*=(Matrix<T> &mat, T a);

/** @brief Divides all matrix entries by a value.
 *  @returns a reference to the changed input matrix.
 *  @details All entries of the matrix are divided by the value \a a. */
template <class T>
Matrix<T> & operator/=(Matrix<T> &mat, T a);

///** @brief Applies the specified function to all matrix elements and returns the result matrix.
// *  @param func
// *  	The function that will be applied.
// *  @param mat
// *  	The input matrix.
// *  @returns a new matrix \a B in which each element \a Bij has the value \a func(\a this->(i,j)).
// *  @details the passed function is applied to all matrix elements. The result matrix is returned.
// *  	The matrix on which this method is called does not change. */
//template <class T>
//Matrix<T> operator<<(const Matrix<T>& mat, T (*func)(T));
//
///** @brief Applies the callable object to all matrix elements and returns the result matrix.
// *  @param func
// *  	Callable object which defines \a T \a operator()(\a T).
// *  @param mat
// *  	The input matrix.
// *  @returns a new matrix \a B in which each element \a Bij has the value \a func(\a this->(i,j)).
// *  @details the passed callable object is applied to all matrix elements. The result matrix is returned. The matrix on which this method is called does not change. */
//template <class T, class F>
//Matrix<T> operator<<(const Matrix<T>& mat, F func);

/** @brief Applies the specified function to all matrix elements.
 *  @param func
 *  	The function that will be applied.
 *  @param mat
 *  	The input matrix.
 *  @returns the a reference to the changed input matrix.
 *  @details Each element \a aij of the matrix gets the new value \a func(\a aij). */
template <class T>
Matrix<T> & operator<<(Matrix<T>& mat, T (*func)(T));

/** @brief Applies the callable object to all matrix elements.
 *  @param func
 *  	Callable object which defines \a T \a operator()(\a T).
 *  @param mat
 *  	The input matrix.
 *  @returns the a reference to the changed input matrix.
 *  @details Each element \a aij gets the new value \a func(\a aij). */
template <class T, class F>
Matrix<T> & operator<<(Matrix<T>& mat, F func);

// ===========================
// ===== IMPLEMENTIERUNG =====
// ===========================

#include "matrix.impl.h"

#endif /* MATRIX_H_ */

