/** @file matrix.impl.h
 *  @brief Implementation of matrix.h
 *  @author Jens Quedenfeld
 */

#include "matrix.h"

#include <cmath>
#include <cstddef>
#include <cstdlib>
#include <ctime>
#include <fstream>
#include <stdexcept>
#include <sstream>
#include <iomanip>

namespace {
	template <class T>
	int compare(const void *a, const void *b) {
		const T *da = (const T *) a;
		const T *db = (const T *) b;
		return (*da < *db) - (*da > *db);
	}
}

// Die Werte der folgenden Konstanten sind recht gut gew�hlt
template<class T>
double Matrix<T>::MULTIPLY_MIX_GRENZE = 50.0; // 100

template<class T>
double Matrix<T>::ACC_SVD_EIGEN_ZERO_DIV = 1e9;

template<class T>
double Matrix<T>::ACC_QRA_STOP_DIV = 1e18; // 1e9

template<class T>
double Matrix<T>::ACC_QRA_FASTQR_ONE_DIV = 1e9;

template<class T>
double Matrix<T>::ACC_REF_EQUAL_DIV = 1e9;

template<class T>
double Matrix<T>::ACC_JACOBI_FAC = 1e9;

template<class T>
double Matrix<T>::ACC_PINV_ZERO_DIV = 1e13; // 1e9

template<class T>
const Matrix<T> Matrix<T>::EMPTY = Matrix<T>();

template<class T>
Matrix<T>::Matrix() {
	createArray_ = NO_ARRAY;
	transposed_ = false;
	dim1_ = 0;
	dim2_ = 0;
	entries_ = NULL;
}

template<class T>
Matrix<T>::Matrix(int dim1, int dim2) {
	createArray_ = COMPLETE_ARRAY;
	transposed_ = false;
	dim1_ = dim1;
	dim2_ = dim2;
	entries_ = createField(dim1_, dim2_);
}

template<class T>
Matrix<T>::Matrix(int dim1, int dim2, int createArray) {
	createArray_ = createArray;
	transposed_ = false;
	dim1_ = dim1;
	dim2_ = dim2;
	entries_ = createField(dim1_, dim2_, createArray);
}

template<class T>
Matrix<T>::Matrix(int dim1, int dim2, T* data) {
	createArray_ = COMPLETE_ARRAY;
	transposed_ = false;
	dim1_ = dim1;
	dim2_ = dim2;
	entries_ = createField(dim1_, dim2_);
	try {
		for (int i = 0; i < dim1; i++)
			for (int j = 0; j < dim2; j++)
				entries_[i][j] = data[i * dim2 + j];
	} catch (...) {
		cleanup(entries_, dim1_);
		throw;
	}
}

template<class T>
Matrix<T>::Matrix(const Matrix<T>& m) {
	createArray_ = COMPLETE_ARRAY; // die Matrix wird Werte-m��ig komplett kopiert
	transposed_ = m.transposed_;
	dim1_ = m.dim1_;
	dim2_ = m.dim2_;
	int dim_first = transposed_ ? dim2_ : dim1_;
	int dim_second = transposed_ ? dim1_ : dim2_;
	entries_ = createField(dim_first, dim_second);
	try {
		for (int i = 0; i < dim_first; i++)
			for (int j = 0; j < dim_second; j++)
				entries_[i][j] = m.entries_[i][j];
	} catch (...) {
		cleanup(entries_, dim_first);
		throw;
	}
}

template <class T>
Matrix<T>::Matrix(int dim1, int dim2, bool optimize) {
	createArray_ = COMPLETE_ARRAY;
	dim1_ = dim1;
	dim2_ = dim2;
	if (optimize && dim1 > dim2) {
		entries_ = createField(dim2_, dim1_);
		transposed_ = true;
	} else {
		entries_ = createField(dim1_, dim2_);
		transposed_ = false;
	}
}

template<class T>
Matrix<T>::Matrix(const Matrix<T>& m, int createArray) {
	createArray_ = createArray;
	transposed_ = m.transposed_;
	dim1_ = m.dim1_;
	dim2_ = m.dim2_;
	int dim_first = transposed_ ? dim2_ : dim1_;
	int dim_second = transposed_ ? dim1_ : dim2_;
	if (createArray == NO_ARRAY) {
		entries_ = m.entries_;
	} else if (createArray == SHALLOW_ARRAY) {
		entries_ = createField(dim_first, dim_second, createArray);
		for (int i = 0; i < dim_first; i++)
			entries_[i] = m.entries_[i];
	} else if (createArray == COMPLETE_ARRAY) {
		entries_ = createField(dim_first, dim_second, createArray);
		try {
			for (int i = 0; i < dim_first; i++)
				for (int j = 0; j < dim_second; j++)
					entries_[i][j] = m.entries_[i][j];
		} catch (...) {
			cleanup(entries_, dim_first);
			throw;
		}
	} else throw std::invalid_argument("Matrix: unknown createArray");
}

template <class T>
void Matrix<T>::cleanup(T** data, int size, int createArray) {
	if (createArray != NO_ARRAY) {
		if (createArray == COMPLETE_ARRAY)
			for (int i = 0; i < size; i++)
				delete[] data[i];
		delete[] data;
	}

}

template <class T>
T** Matrix<T>::createField(int dim1, int dim2, int createArray) {
	if (createArray == NO_ARRAY) return NULL;
	T** newData = NULL;
	int i = 0;
	try {
		newData = new T*[dim1];
		if (createArray == COMPLETE_ARRAY)
			for (i = 0; i < dim1; i++)
				newData[i] = new T[dim2];
	} catch (...) {
		cleanup(newData, i-1);
		std::stringstream ss;
		ss << "Matrix: createField(" << dim1 << "," << dim2 << "," << createArray << "): i=" << i;
		throw std::runtime_error(ss.str());
	}
	return newData;
}

template <class T>
Matrix<T>& Matrix<T>::operator=(const Matrix<T> &m) {
	if (this != &m) {
		// Dimensionen
		int dim_first = m.transposed_ ? m.dim2_ : m.dim1_;
		int dim_second = m.transposed_ ? m.dim1_ : m.dim2_;

		// Neues Datenfeld erstellen
		T** newData = NULL;
		int i = 0;
		try {
			newData = new T*[dim_first];
			for (i = 0; i < dim_first; i++)
				newData[i] = new T[dim_second];

			// Daten kopieren
			for (int i = 0; i < dim_first; i++) {
				for (int j = 0; j < dim_second; j++) {
					newData[i][j] = m.entries_[i][j];
				}
			}
		} catch (...) {
			cleanup(newData, i-1);
			throw;
		}

		// Altes Array l�schen
		if (createArray_ != NO_ARRAY) {
			if (createArray_ == COMPLETE_ARRAY)
				for (int i = 0; i < (transposed_ ? dim2_ : dim1_); i++)
					delete[] entries_[i];
			delete[] entries_;
		}

		// Werte �bernehmen
		createArray_ = COMPLETE_ARRAY; // die Matrix wird Werte-m��ig komplett kopiert
		transposed_ = m.transposed_;
		dim1_ = m.dim1_;
		dim2_ = m.dim2_;
		entries_ = newData;
	}
	return *this;
}

template <class T>
void Matrix<T>::swap(Matrix<T>& m) {
	std::swap(dim1_, m.dim1_);
	std::swap(dim2_, m.dim2_);
	std::swap(transposed_, m.transposed_);
	std::swap(entries_, m.entries_);
	std::swap(createArray_, m.createArray_);
}

template <class T>
void Matrix<T>::set(Matrix<T> m) {
	std::swap(dim1_, m.dim1_);
	std::swap(dim2_, m.dim2_);
	std::swap(transposed_, m.transposed_);
	std::swap(entries_, m.entries_);
	std::swap(createArray_, m.createArray_);
}

template <class T>
void Matrix<T>::set_noCopy(const Matrix<T>& m) {
	// Altes Array löschen
	if (createArray_ != NO_ARRAY) {
		if (createArray_ == COMPLETE_ARRAY)
			for (int i = 0; i < (transposed_ ? dim2_ : dim1_); i++)
				delete[] entries_[i];
		delete[] entries_;
	}

	dim1_ = m.dim1_;
	dim2_ = m.dim2_;
	createArray_ = NO_ARRAY;
	transposed_ = m.transposed_;
	entries_ = m.entries_;

}

template<class T>
Matrix<T>::~Matrix() {
	if (createArray_ != NO_ARRAY) {
		if (createArray_ == COMPLETE_ARRAY)
			for (int i = 0; i < (transposed_ ? dim2_ : dim1_); i++)
				delete[] entries_[i];
		delete[] entries_;
	}
}

//template<class T>
//Matrix <T>* Matrix<T>::pointer() {
//	Matrix <T> *erg = new Matrix(*this, NO_ARRAY);
//	erg->createArray_ = COMPLETE_ARRAY;
//	createArray_ = NO_ARRAY;
//	return erg;
//}

template <class T>
Matrix <T> Matrix<T>::id(int dim) {
	Matrix <T> m(dim, dim);
	for (int i = 0; i < dim; i++)
		for (int j = 0; j < dim; j++)
			m.updateEntry(i, j, i == j ? 1 : 0);
	return m;
}

template <class T>
Matrix <T> Matrix<T>::zero(int dim1, int dim2, bool optimize) {
	Matrix <T> m(dim1, dim2, optimize);
	m.fill(0);
	return m;
}

template<class T>
std::pair<int, int> Matrix<T>::getDimensions() const {
	return std::pair<int, int> (dim1_, dim2_);
}

template<class T>
inline int Matrix<T>::getDim1() const {
	return dim1_;
}
template<class T>
inline int Matrix<T>::getDim2() const {
	return dim2_;
}

template<class T>
inline void Matrix<T>::updateEntry(int index1, int index2, T entry) {
	assert2(index1 >= 0 && index1 < dim1_ && index2 >= 0 && index2 < dim2_,
			"(m x n) = (" << dim1_ << " x " << dim2_ << "), " << "index1: " << index1 << ", index2: " << index2)
	if (transposed_) entries_[index2][index1] = entry;
	else entries_[index1][index2] = entry;
}

template<class T>
inline void Matrix<T>::updateEntryCheck(int index1, int index2, T entry) {
	if (index1 < 0 || index1 >= dim1_ || index2 < 0 || index2 >= dim2_) return;
	if (transposed_) entries_[index2][index1] = entry;
	else entries_[index1][index2] = entry;
}

template<class T>
inline void Matrix<T>::addEntry(int index1, int index2, T entry) {
	assert2(index1 >= 0 && index1 < dim1_ && index2 >= 0 && index2 < dim2_,
			"(m x n) = (" << dim1_ << " x " << dim2_ << "), " << "index1: " << index1 << ", index2: " << index2)
	if (transposed_) entries_[index2][index1] += entry;
	else entries_[index1][index2] += entry;
}

template<class T>
inline void Matrix<T>::addtoAll(T entry) {
	for (int i = 0; i < dim1_; i++){
		for (int j = 0; j < dim2_; j++){
			addEntry(i, j, entry);
		}
	}
}

template<class T>
inline void Matrix<T>::multiplytoAll(T entry) {
	for (int i = 0; i < dim1_; i++){
		for (int j = 0; j < dim2_; j++){
			(*this)(i,j) *= entry;
		}
	}
}

template<class T>
void Matrix<T>::fill(T entry) {
	for (int i = 0; i < dim1_; i++) {
		for (int j = 0; j < dim2_; j++) {
			updateEntry(i, j, entry);
		}
	}
}

template<class T>
inline T Matrix<T>::getEntry(int index1, int index2) const {
	assert2(index1 >= 0 && index1 < dim1_ && index2 >= 0 && index2 < dim2_,
			"(m x n) = (" << dim1_ << " x " << dim2_ << "), " << "index1: " << index1 << ", index2: " << index2)
	if (transposed_) return entries_[index2][index1];
	else return entries_[index1][index2];
}

template<class T>
inline T Matrix<T>::getEntryCheck(int index1, int index2) const {
	if (index1 < 0 || index1 >= dim1_ || index2 < 0 || index2 >= dim2_) return 0;
	if (transposed_) return entries_[index2][index1];
    else return entries_[index1][index2];
}

template<class T>
T& Matrix<T>::operator()(int index1, int index2) {
	assert2(index1 >= 0 && index1 < dim1_ && index2 >= 0 && index2 < dim2_,
			"(m x n) = (" << dim1_ << " x " << dim2_ << "), " << "index1: " << index1 << ", index2: " << index2)
	if (transposed_) return (entries_[index2][index1]);
	else return (entries_[index1][index2]);
}

template<class T>
const T& Matrix<T>::operator()(int index1, int index2) const {
	assert2(index1 >= 0 && index1 < dim1_ && index2 >= 0 && index2 < dim2_,
			"(m x n) = (" << dim1_ << " x " << dim2_ << "), " << "index1: " << index1 << ", index2: " << index2)
	if (transposed_) return (entries_[index2][index1]);
	else return (entries_[index1][index2]);
}

template <class T>
T& Matrix<T>::operator[](size_t x) {
	assert2(x >= 0 && x < (size_t) dim1_ * (size_t) dim2_,
			"(m x n) = (" << dim1_ << " x " << dim2_ << "), " << "x: " << x)
	int i = x % dim1_;
	int j = x / dim1_;
	if (transposed_) return (entries_[j][i]);
	else return (entries_[i][j]);
}

template <class T>
const T& Matrix<T>::operator[](size_t x) const {
	assert2(x >= 0 && x < (size_t) dim1_ * (size_t) dim2_,
			"(m x n) = (" << dim1_ << " x " << dim2_ << "), " << "x: " << x)
	int i = x % dim1_;
	int j = x / dim1_;
	if (transposed_) return (entries_[j][i]);
	else return (entries_[i][j]);
}

template <class T>
bool Matrix<T>::isEmpty() const {
	return dim1_ == 0 && dim2_ == 0;
}

template <class T>
T Matrix<T>::trace() const {
	T sum = 0;
	for (int i = 0; i < (dim1_ < dim2_ ? dim1_ : dim2_); i++)
		sum += getEntry(i,i);
	return sum;
}

template <class T>
void Matrix<T>::transform(T (*func)(T)) {
	for (int i = 0; i < dim1_; i++)
		for (int j = 0; j < dim2_; j++)
			updateEntry(i, j, func(getEntry(i, j)));
}

template <class T>
Matrix<T> Matrix<T>::map(T (*func)(T)) const {
	Matrix<T> erg(dim1_, dim2_);
	for (int i = 0; i < dim1_; i++)
		for (int j = 0; j < dim2_; j++)
			erg(i, j) = func(getEntry(i, j));
	return erg;
}

template <class T>
T Matrix<T>::fold(T (*func)(T, T), T erg) const {
	for (int i = 0; i < dim1_; i++)
		for (int j = 0; j < dim2_; j++)
			erg = func(erg, getEntry(i,j));
	return erg;
}

template <class T>
void Matrix<T>::transform(const Matrix<T>& m, T (*func) (T, T)) {
	if (m.dim1_ != dim1_ || m.dim2_ != dim2_)
		throw dim_exception("compute: dim");
	for (int i = 0; i < dim1_; i++)
		for (int j = 0; j < dim2_; j++)
			updateEntry(i, j, func(getEntry(i,j), m.getEntry(i,j)));
}

template <class T>
Matrix<T> Matrix<T>::map(const Matrix<T>& m, T (*func) (T, T)) const {
	if (m.dim1_ != dim1_ || m.dim2_ != dim2_)
		throw dim_exception("compute: dim");
	Matrix<T> erg(dim1_, dim2_);
	for (int i = 0; i < dim1_; i++)
		for (int j = 0; j < dim2_; j++)
			erg(i, j) = func(getEntry(i,j), m.getEntry(i,j));
	return erg;
}


template<class T>
void Matrix<T>::transpose() {
	transposed_ = !transposed_;
	int dim = dim1_;
	dim1_ = dim2_;
	dim2_ = dim;
}

template<class T>
const Matrix <T> Matrix<T>::getTranspose() const {
	Matrix<T> mat;
	mat.createArray_ = NO_ARRAY;
	mat.dim2_ = dim1_;
	mat.dim1_ = dim2_;
	mat.transposed_ = !transposed_;
	mat.entries_ = entries_;
	return mat;
}

template <class T>
void Matrix<T>::resize(int dim1, int dim2) {
	if (dim1 == dim1_ && dim2 == dim2_) return;
	int dim_fst = transposed_ ? dim2_ : dim1_;
	int dim_snd = transposed_ ? dim1_ : dim2_;
	int dim_fst_new = transposed_ ? dim2 : dim1;
	int dim_snd_new = transposed_ ? dim1 : dim2;
	if (dim_snd == dim_snd_new && (createArray_ == COMPLETE_ARRAY || dim_fst_new <= dim_fst)) {
		// nur Änderung des primären Arrays
		T** newData = createField(dim_fst_new, dim_snd_new, SHALLOW_ARRAY);
		for (int i = 0; i < std::min(dim_fst, dim_fst_new); i++)
			newData[i] = entries_[i];
		if (createArray_ == COMPLETE_ARRAY)
			for (int i = dim_fst_new; i < dim_fst; i++)
				delete[] entries_[i];
		if (dim_fst_new > dim_fst) { // COMPLETE_ARRAY nach Voraussetzung!
			int i = dim_fst;
			try {
				for (i = dim_fst; i < dim_fst_new; i++)
					newData[i] = new T[dim_snd_new];
			} catch (...) {
				for (int k = dim_fst; k < i-1; k++)
					delete[] newData[k];
				delete[] newData;
				std::stringstream ss;
				ss << "Matrix: resize(" << dim1 << "," << dim2 << "): i=" << i;
				throw std::runtime_error(ss.str());
			}
		}
		delete[] entries_;
		if (createArray_ == NO_ARRAY)
			createArray_ = SHALLOW_ARRAY;
		entries_ = newData;
		dim1_ = dim1;
		dim2_ = dim2;
	} else {
		// Änderung der sekundären Arrays => neues Array
		T** newData = createField(dim1, dim2);
		try {
			for (int i = 0; i < std::min(dim1_, dim1); i++) {
				for (int j = 0; j < std::min(dim2_, dim2); j++)
					newData[i][j] = getEntry(i, j);
			}
		} catch (...) {
			cleanup(newData, dim1);
			throw;
		}
		cleanup(entries_, transposed_ ? dim2_ : dim1_, createArray_);
		entries_ = newData;
		createArray_ = COMPLETE_ARRAY;
		transposed_ = false;
		dim1_ = dim1;
		dim2_ = dim2;
	}
}

template<class T>
void Matrix<T>::addMatrix(const Matrix<T>& matrix) {
	// Sicherheitsprüfung
	if (dim1_ != matrix.dim1_ || dim2_ != matrix.dim2_)
		throw dim_exception("addMatrix: dim");

	// Addition
	for (int i = 0; i < dim1_; i++)
		for (int j = 0; j < dim2_; j++)
			addEntry(i, j, matrix.getEntry(i, j));
}

template<class T>
void Matrix<T>::subMatrix(const Matrix<T>& matrix) {
	// Sicherheitsprüfung
	if (dim1_ != matrix.dim1_ || dim2_ != matrix.dim2_)
		throw dim_exception("subMatrix: dim");

	// Subtraktion
	for (int i = 0; i < dim1_; i++)
		for (int j = 0; j < dim2_; j++)
			addEntry(i, j, - matrix.getEntry(i, j));
}

//template<class T>
//Matrix<T> Matrix<T>::addMatrix(const Matrix<T>* data, int dim1, int dim2, int off_a1, int off_a2, int off_b1, int off_b2) const {
//	Matrix<T> erg = Matrix<T>(dim1, dim2);
//	for (int i = 0; i < dim1; i++) {
//		T* ergDataI = erg.entries_[i];
//		int i_plus_off_a1 = i + off_a1;
//		int i_plus_off_b1 = i + off_b1;
//		for (int j = 0; j < dim2; j++) {
//			ergDataI[j] = getEntryCheck(i_plus_off_a1, j + off_a2) + data->getEntryCheck(i_plus_off_b1, j + off_b2);
//		}
//	}
//	return erg;
//}
template<class T>
Matrix<T> Matrix<T>::addMatrix(const Matrix<T>& data, int dim1, int dim2, int off_a1, int off_a2, int off_b1, int off_b2) const {
	Matrix<T> erg = Matrix<T>(dim1, dim2);
	for (int i = 0; i < dim1; i++) {
		T* ergDataI = erg.entries_[i];
		int i_plus_off_a1 = i + off_a1;
		int i_plus_off_b1 = i + off_b1;
		for (int j = 0; j < dim2; j++) {
			ergDataI[j] = getEntryCheck(i_plus_off_a1, j + off_a2) + data.getEntryCheck(i_plus_off_b1, j + off_b2);
		}
	}
	return erg;
}

//template<class T>
//Matrix<T> Matrix<T>::subMatrix(const Matrix<T>* data, int dim1, int dim2, int off_a1, int off_a2, int off_b1, int off_b2) const {
//	Matrix<T> erg = Matrix<T>(dim1, dim2);
//	for (int i = 0; i < dim1; i++) {
//		T* ergDataI = erg.entries_[i];
//		int i_plus_off_a1 = i + off_a1;
//		int i_plus_off_b1 = i + off_b1;
//		for (int j = 0; j < dim2; j++) {
//			ergDataI[j] = getEntryCheck(i_plus_off_a1, j + off_a2) - data->getEntryCheck(i_plus_off_b1, j + off_b2);
//		}
//	}
//	return erg;
//}
template<class T>
Matrix<T> Matrix<T>::subMatrix(const Matrix<T>& data, int dim1, int dim2, int off_a1, int off_a2, int off_b1, int off_b2) const {
	Matrix<T> erg = Matrix<T>(dim1, dim2);
	for (int i = 0; i < dim1; i++) {
		T* ergDataI = erg.entries_[i];
		int i_plus_off_a1 = i + off_a1;
		int i_plus_off_b1 = i + off_b1;
		for (int j = 0; j < dim2; j++) {
			ergDataI[j] = getEntryCheck(i_plus_off_a1, j + off_a2) - data.getEntryCheck(i_plus_off_b1, j + off_b2);
		}
	}
	return erg;
}

template<class T>
Matrix<T> Matrix<T>::multiplyNaive(const Matrix<T>* matrix) const {
	// Sicherheitspr�fung
	if (matrix == NULL) throw std::invalid_argument("multiplyNaive: NULL");
	if (dim2_ != matrix->dim1_) throw dim_exception("multiplyNaive: dim");

	return multiplyNaiveUnchecked(matrix);
}

template<class T>
Matrix<T> Matrix<T>::multiplyNaiveUncheckedSimple(const Matrix<T>* matrix) const {
	// Dimensionen: (I x K)*(K x J) = (I x J)
	// Falls Dimensionen nicht zusammenpassen, so nehme die gr��ere und f�lle IMPLIZIT mit Nullen auf.
	int dimI = dim1_,
	    dimK = dim2_ <= matrix->dim1_ ? dim2_ : matrix->dim1_,
	    dimJ = matrix->dim2_;

	// Neue Matrix
	Matrix<T> erg = Matrix(dimI, dimJ);

	if (dimK == 0) {
		for (int i = 0; i < dimI; i++)
			for (int j = 0; j < dimJ; j++) {
				erg.entries_[i][j] = (transposed_ ? entries_[0][i] : entries_[i][0])
						 			* (matrix->transposed_ ? matrix->entries_[j][0] : matrix->entries_[0][j]);
			}
	} else {
		for (int i = 0; i < dimI; i++)
			for (int j = 0; j < dimJ; j++) {
				T sum = 0;
				for (int k = 0; k < dimK; k++) {
					sum +=  (transposed_ ? entries_[k][i] : entries_[i][k])
						  * (matrix->transposed_ ? matrix->entries_[j][k] : matrix->entries_[k][j]);
				}
				erg.entries_[i][j] = sum;
			}
	}

	return erg;
}

template<class T>
Matrix<T> Matrix<T>::multiplyNaiveUnchecked(const Matrix<T>* matrix) const {
	using namespace useful;

	// Dimensionen: (I x K)*(K x J) = (I x J)
	// Falls Dimensionen nicht zusammenpassen, so nehme die gr��ere und f�lle IMPLIZIT mit Nullen auf.
	int dimI = dim1_,
	    dimK = dim2_ <= matrix->dim1_ ? dim2_ : matrix->dim1_,
	    dimJ = matrix->dim2_;

	if (dimK == 0) {
		Matrix<T> erg(dimI, dimJ);
		erg.fill(0);
		return erg;
	}
	if (dimK == 1) return multiplyNaiveUncheckedSimple(matrix);

	// Neue Matrix
	Matrix<T> erg = Matrix(dimI, dimJ);

	// Vorarbeit
	// Daten von "this"
	T** datA = NULL;
	bool deleteDatA = false;
	if (transposed_) {
		try {
			datA = createField(dim1_, dim2_);
			for (int i = 0; i < dim1_; i++) {
				for (int j = 0; j < dim2_; j++)
					datA[i][j] = getEntry(i,j);
			}
			deleteDatA = true;
		} catch (...) {
			cleanup(datA, dim1_);
			datA = entries_;
			deleteDatA = false;
		}
	} else datA = entries_;

	// transponierte (!) Daten von "matrix"
	T** datBT = NULL;
	bool deleteDatBT = false;
	if (matrix->transposed_) datBT = matrix->entries_;
	else {
		try {
			datBT = createField(matrix->dim2_, matrix->dim1_);
			for (int j = 0; j < matrix->dim2_; j++) {
				for (int i = 0; i < matrix->dim1_; i++)
					datBT[j][i] = matrix->getEntry(i,j);
			}
			deleteDatBT = true;
		} catch (...) {
			cleanup(datBT, matrix->dim2_);
			datBT = matrix->entries_;
			deleteDatBT = false;
		}
	}

	// Multiplikation
	for (int i = 0; i < dimI; i++) {
		T* a = datA[i];
		T* ergI = erg.entries_[i];
		for (int j = 0; j < dimJ; j++) {
			T* b = datBT[j];
			T sum = 0;
			// Alte Schleife
//			for (int k = 0; k < dimK; k++) {
//				sum += a[k]*b[k];
//			}
			// Entrollte Schleife
			int k = 0;
			for (; k < dimK % 8; k++) {
				sum += a[k]*b[k];
			}
			while (k < dimK) {
				sum += a[k]*b[k]; k++;
				sum += a[k]*b[k]; k++;
				sum += a[k]*b[k]; k++;
				sum += a[k]*b[k]; k++;

				sum += a[k]*b[k]; k++;
				sum += a[k]*b[k]; k++;
				sum += a[k]*b[k]; k++;
				sum += a[k]*b[k]; k++;
			}
			ergI[j] = sum;
		}
	}

	// ggf. l�schen
	if (deleteDatA) {
		for (int i = 0; i < dim1_; i++) delete[] datA[i];
		delete[] datA;
	}
	if (deleteDatBT) {
		for (int j = 0; j < matrix->dim2_; j++) delete[] datBT[j];
		delete[] datBT;
	}

	// R�ckgabe
	return erg;
}


template<class T>
Matrix<T> Matrix<T>::multiplyMix(const Matrix<T>* matrix) const {
	using namespace useful;

	// Sicherheitspr�fung
	if (matrix == NULL) throw std::invalid_argument("multiplyMix: NULL");
	if (dim2_ != matrix->dim1_) {
		std::ostringstream ss;
		ss << "multiplyMix: dim: ";
		ss << "(" << dim1_ << " x " << dim2_ << ") * (" << matrix->dim1_ << " x " << matrix->dim2_ << ")";
		throw dim_exception(ss.str().c_str());
	}

	timeMultiply_naive = 0;
	timeMultiply_other = 0;
	timeMultiply_add = 0;
	timeMultiply_last = 0;
	Matrix <T> mat(*matrix, NO_ARRAY);
	return multiplyMixUnchecked(&mat);
}

namespace {
	long long timeMultiply_t0 = 0;
}

template<class T>
Matrix<T> Matrix<T>::multiplyMixUnchecked(Matrix<T>* matrix) const {
	using namespace useful;

	// Dimensionen: (I x K)*(K x J) = (I x J)
	// Falls Dimensionen nicht zusammenpassen, so nehme die gr��ere und f�lle IMPLIZIT mit Nullen auf.
	timeMultiply_t0 = clock();
	int dimI = dim1_,
		dimK = dim2_ > matrix->dim1_ ? dim2_ : matrix->dim1_,
		dimJ = matrix->dim2_;

	// Abbruch-Kriterium
	if (dimI == 1 && dimK == 1 && dimJ == 1) {
		Matrix <T> erg = Matrix <T> (1, 1);
		(erg).updateEntry(0, 0, getEntry(0, 0) * matrix->getEntry(0, 0));
		return erg;
	}
	timeMultiply_other += clock() - timeMultiply_t0;
	timeMultiply_t0 = clock();

	// Falls die Matrizen klein genug sind, so benutze Naive statt Strassen
	// Volumen / Oberfl�che
	if (((double) dimI * dimK * dimJ) / (dimI * dimK + dimK * dimJ + dimI * dimJ) <= MULTIPLY_MIX_GRENZE + 1 || // +1, wegen Rundungsfehlern bei double
			dimI <= 1 || dimK <= 1 || dimJ <= 1) {

		Matrix<T> erg = multiplyNaiveUnchecked(matrix);
		timeMultiply_naive += clock() - timeMultiply_t0;
		return erg;
	}

	// halbierte (aufgerundete) Dimensionen speichern
	int dimIh = (dimI + 1) >> 1,
		dimKh = (dimK + 1) >> 1,
		dimJh = (dimJ + 1) >> 1;

	// Erstellen von x11 und y11 (oben linker Teil von X und Y)
	// Dazu wird entries_ entsprechend gesetzt und NO_ARRAY gesetzt, damit die Original-Matrix nicht zerst�rt wird
	Matrix <T> x11 = Matrix <T> (dimIh, dimKh, NO_ARRAY);
	Matrix <T> y11 = Matrix <T> (dimKh, dimJh, NO_ARRAY);
	x11.entries_ = entries_;
	x11.transposed_ = transposed_;
	y11.entries_ = matrix->entries_;
	y11.transposed_ = matrix->transposed_;

	// Erstellen von x22 und y22 (unten rechter Teil von X und Y)
	// Es wird IMPLIZIT mit Nullen aufgef�llt, falls erforderlich, durch Angabe der "zu kleinen" Dimension.
	Matrix <T> x22 = Matrix <T> (dim1_ - dimIh, dim2_ - dimKh, NO_ARRAY);
	Matrix <T> y22 = Matrix <T> (matrix->dim1_ - dimKh, matrix->dim2_ - dimJh, NO_ARRAY);
	x22.createArray_ = SHALLOW_ARRAY;
	x22.transposed_ = transposed_;
	x22.entries_ = createField(x22.transposed_ ? x22.dim2_ : x22.dim1_, 0, SHALLOW_ARRAY);
	y22.createArray_ = SHALLOW_ARRAY;
	y22.transposed_ = matrix->transposed_;
	y22.entries_ = createField(y22.transposed_ ? y22.dim2_ : y22.dim1_, 0, SHALLOW_ARRAY);
	if (x22.transposed_) {
		for (int i = 0; i < x22.dim2_; i++)
			x22.entries_[i] = &entries_[i + dimKh][dimIh];
	} else {
		for (int i = 0; i < x22.dim1_; i++)
			x22.entries_[i] = &entries_[i + dimIh][dimKh];
	}
	if (y22.transposed_) {
		for (int i = 0; i < y22.dim2_; i++)
			y22.entries_[i] = &matrix->entries_[i + dimJh][dimKh];
	} else {
		for (int i = 0; i < y22.dim1_; i++)
			y22.entries_[i] = &matrix->entries_[i + dimKh][dimJh];
	}

	timeMultiply_other += clock() - timeMultiply_t0;
	timeMultiply_t0 = clock();

	// Erstellen der Hilfsmatrizen f�r die Multiplikation
	// mxa steht f�r den a-ten Faktor des x-ten Produkts
	// transpose() wird aufgerufen, um multiplyNaive() ggf. effizient zu berechnen
	Matrix <T> m1, m2, m3, m4, m5, m6, m7;
	{
		Matrix <T> m11 = subMatrix(*this, dimIh, dimKh, 0, dimKh, dimIh, dimKh);	//x12; m11.subMatrix(&x22);
		matrix->transpose();
		Matrix <T> m12 = matrix->addMatrix(*matrix, dimJh, dimKh, 0 , dimKh, dimJh, dimKh); //y21; m12.addMatrix(&y22);
		m12.transpose();
		matrix->transpose();
		m1 = m11.multiplyMixUnchecked(&m12); //, dimIh, dimJh);
	}{
		Matrix <T> m21 = addMatrix(*this, dimIh, dimKh, 0, 0, dimIh, dimKh); //x11; m21.addMatrix(&x22);
		matrix->transpose();
		Matrix <T> m22 = matrix->addMatrix(*matrix, dimJh, dimKh, 0, 0, dimJh, dimKh); //y11; m22.addMatrix(&y22);
		m22.transpose();
		matrix->transpose();
		m2 = m21.multiplyMixUnchecked(&m22); //, dimIh, dimJh);
	}{
		Matrix <T> m31 = subMatrix(*this, dimIh, dimKh, dimIh, 0, 0, 0); //x21; m31.subMatrix(&x11);
		matrix->transpose();
		Matrix <T> m32 = matrix->addMatrix(*matrix, dimJh, dimKh, 0, 0, dimJh, 0); //y11; m32.addMatrix(&y12);
		m32.transpose();
		matrix->transpose();
		m3 = m31.multiplyMixUnchecked(&m32); //, dimIh, dimJh);
	}{
		Matrix <T> m41 = addMatrix(*this, dimIh, dimKh, 0, 0, 0, dimKh); //x11; m41.addMatrix(&x12);
		m4 = m41.multiplyMixUnchecked(&y22); //, dimIh, dimJh);
	}{
		matrix->transpose();
		Matrix <T> m52 = matrix->subMatrix(*matrix, dimJh, dimKh, dimJh, 0, dimJh, dimKh); //y12; m52.subMatrix(&y22);
		m52.transpose();
		matrix->transpose();
		m5 = x11.multiplyMixUnchecked(&m52); //, dimIh, dimJh);
	}{
		matrix->transpose();
		Matrix <T> m62 = matrix->subMatrix(*matrix, dimJh, dimKh, 0, dimKh, 0, 0); //y21; m62.subMatrix(&y11);
		m62.transpose();
		matrix->transpose();
		m6 = x22.multiplyMixUnchecked(&m62); //, dimIh, dimJh);
	}{
		Matrix <T> m71 = addMatrix(*this, dimIh, dimKh, dimIh, 0, dimIh, dimKh); //x21; m71.addMatrix(&x22);
		m7 = m71.multiplyMixUnchecked(&y11); //, dimIh, dimJh);
	}

	timeMultiply_add += clock() - timeMultiply_t0;
	timeMultiply_t0 = clock();

	// Rekursiver Aufruf (Multiplikation)

	// Erstellen der Ergebnismatrix (indem die mx-Matrizen korrekt zusammen addiert bzw. subtrahiert)
	Matrix <T> erg = Matrix<T>(dimI, dimJ);
	// Wissen verwenden:
	// mx ist nie transponiert
	// Es wird nicht auf negative Indizes zugegriffen
	int dimIh_min = dimI >> 1;
	int dimJh_min = dimJ >> 1;
	for (int i = 0; i < dimI; i++) {
		T* ergDataI = erg.entries_[i];
		int i2 = i - dimIh;
		if (i2 < 0) {
			T* m1I = m1.entries_[i];
			T* m2I = m2.entries_[i];
			T* m4I = m4.entries_[i];
			T* m5I = m5.entries_[i];
			T* m6I = m6.entries_[i];
			for (int j = 0; j < dimJ; j++) {
				int j2 = j - dimJh;
				if (j2 < 0) {
					// z11 = m1 + m2 - m4 + m6
					if (i < dimIh_min && j < dimJh_min) {
						ergDataI[j] = m1I[j] + m2I[j] - m4I[j] + m6I[j];
					} else {
						ergDataI[j] = m1.getEntryCheck(i, j) + m2.getEntryCheck(i, j) - m4.getEntryCheck(i, j) + m6.getEntryCheck(i, j);
					}
				} else {
					// z12 = m4 + m5
					if (i < dimIh_min && j2 < dimJh_min) {
						ergDataI[j] = m4I[j2] + m5I[j2];
					} else {
						ergDataI[j] = m4.getEntryCheck(i, j2) + m5.getEntryCheck(i, j2);
					}
				}
			}
		} else {
			T* m2I = m2.entries_[i2];
			T* m3I = m3.entries_[i2];
			T* m5I = m5.entries_[i2];
			T* m6I = m6.entries_[i2];
			T* m7I = m7.entries_[i2];
			for (int j = 0; j < dimJ; j++) {
				int j2 = j - dimJh;
				if (j2 < 0) {
					// z21 = m6 + m7
					if (i2 < dimIh_min && j < dimJh_min) {
						ergDataI[j] = m6I[j] + m7I[j];
					} else {
						ergDataI[j] = m6.getEntryCheck(i2, j) + m7.getEntryCheck(i2, j);
					}
				} else {
					// z22 = m2 + m3 + m5 - m7
					if (i2 < dimIh_min && j2 < dimJh_min) {
						ergDataI[j] = m2I[j2] + m3I[j2] + m5I[j2] - m7I[j2];
					} else {
						ergDataI[j] = m2.getEntryCheck(i2, j2) + m3.getEntryCheck(i2, j2) + m5.getEntryCheck(i2, j2) - m7.getEntryCheck(i2, j2);
					}
				}
			}
		}
	}
	timeMultiply_last += clock() - timeMultiply_t0;
	return erg;
}

template<class T>
Matrix <T> Matrix<T>::multiplyWithDiagonal(const T* data, int size, int dim2) const {
	Matrix <T> erg = Matrix <T> (dim1_, dim2);
	for (int i = 0; i < dim1_; i++)
		for (int j = 0; j < dim2; j++) {
			if (j < size)
				erg.updateEntry(i, j, getEntry(i, j) * data[j]);
			else
				erg.updateEntry(i, j, 0);
		}
	return erg;
}
template<class T>
Matrix <T> Matrix<T>::multiplyWithDiagonal(const std::vector<T>& data) const {
	return multiplyWithDiagonal(&data[0], data.size(), data.size());
}

template<class T>
Matrix<T> Matrix<T>::calcATA() const {
	Matrix <T> trans = Matrix <T> (dim2_, dim1_, NO_ARRAY);
	trans.entries_ = entries_;
	trans.transposed_ = !transposed_;
	return trans.multiplyMix(this);
}

template<class T>
Matrix<T> Matrix<T>::calcAAT() const {
	Matrix <T> trans = Matrix <T> (dim2_, dim1_, NO_ARRAY);
	trans.entries_ = entries_;
	trans.transposed_ = !transposed_;
	return multiplyMix(&trans);
}

template <class T>
void Matrix<T>::insert(const Matrix<T>& mat, int off1, int off2) {
	insert(mat, off1, off2, mat.dim1_, mat.dim2_);
}

template <class T>
void Matrix<T>::insert(const Matrix<T>& mat, int off1, int off2, int size1, int size2) {
	if (mat.dim1_ < size1) size1 = mat.dim1_;
	if (mat.dim2_ < size2) size2 = mat.dim2_;
	if (size1 + off1 > dim1_) size1 = dim1_ - off1;
	if (size2 + off2 > dim2_) size1 = dim2_ - off2;

	for (int i = 0; i < size1; i++)
		for (int j = 0; j < size2; j++)
			updateEntry(off1 + i, off2 + j, mat(i,j));
}

template <class T>
const Matrix<T> Matrix<T>::select(int off1, int off2, int dim1, int dim2) const {
	Matrix <T> erg = Matrix <T> (dim1, dim2, NO_ARRAY);
	if (!transposed_) {
		if (off2 == 0) {
			erg.entries_ = &entries_[off1];
		} else {
			erg.createArray_ = SHALLOW_ARRAY;
			erg.entries_ = new T*[dim1];
			for (int i = 0; i < dim1; i++)
				erg.entries_[i] = &entries_[i + off1][off2];
		}
	} else {
		erg.transposed_ = true;
		if (off1 == 0) {
			erg.entries_ = &entries_[off2];
		} else {
			erg.createArray_ = SHALLOW_ARRAY;
			erg.entries_ = new T*[dim2];
			for (int i = 0; i < dim2; i++)
				erg.entries_[i] = &entries_[i + off2][off1];
		}
	}
	return erg;
}

template<class T>
double Matrix<T>::frobeniusnorm(bool square) const {
	// = sqrt(sum(sum(|mij|^2)))
	double d = 0;
	for (int i = 0; i < dim1_; i++) {
		for (int j = 0; j < dim2_; j++) {
			T abs = (T) fabs(getEntry(i,j));
			d += abs*abs;
		}
	}
	return square ? d : sqrt(d);
}

template<class T>
T Matrix<T>::sum() const {
	T sum = 0;
	for (int i = 0; i < dim1_; i++)
		for (int j = 0; j < dim2_; j++)
			sum += getEntry(i,j);
	return sum;
}


template<class T>
double Matrix<T>::mean() const {
	double m = 0;
	for (int i = 0; i < dim1_; i++) {
		for (int j = 0; j < dim2_; j++) {
			m += getEntry(i,j);
		}
	}
	return m / dim1_ / dim2_;
}

template<class T>
double Matrix<T>::median() const {
	int n = dim1_*dim2_;
	std::vector<double> data(n);
	int count = 0;
	for (int i = 0; i < dim1_; i++) {
		for (int j = 0; j < dim2_; j++, count++) {
			data[count] = getEntry(i, j);
		}
	}
	qsort(&data[0], n, sizeof (double), ::compare<T>);

	double m = data[n / 2];
	if (n % 2 == 0) m = (m + data[n / 2 - 1]) / 2;
	return m;
}

template<class T>
double Matrix<T>::detLaplace() const {
	// Summe (i = 1, n, (-1)^(i+j)*aij*detAij
	// Zeilen k�nnen effizient gestrichen werden => Entwickelt werden sollte nach der letzten Spalte

	// Sicherheitsabfrage
	if (dim1_ != dim2_) throw notSquare_exception("detLaplace: nicht quadratisch");

	// Rekursionsabruch
	if (dim1_ == 1 && dim2_ == 1) return getEntry(0,0);

	// Rekursion
	double sum = 0;
	int j = dim2_ - 1; // letzte Spalte
	int fac = j % 2 == 0 ? 1 : -1;
	for (int i = 0; i < dim1_; i++, fac *= -1) {
		// Streichen der i-ten Zeile --> entries_[] bearbeiten
		//        und letzten Spalte --> geht "automatisch"
		Matrix <T> mat = Matrix <T> (dim1_ - 1, dim2_ - 1, SHALLOW_ARRAY);
		int c = 0;
		for (int k = 0; k < dim1_; k++) {
			if (k != i) mat.entries_[c++] = entries_[k];
		}

		sum += fac * getEntry(i, j) * mat.detLaplace();
	}

	return sum;
}

template<class T>
double Matrix<T>::detGauss() const {
	// Sicherheitsabfrage
	if (dim1_ != dim2_) throw std::invalid_argument("detGauss: nicht quadratisch");

	double prod = 1;

	// Dreiecksmatrix erstellen
	Matrix <T> mat = Matrix(*this);

	// F�r jede Zeile i
	for (int i = 0; i < dim1_; i++) {
		// Falls m(i,i) == 0, so vertausche zwei Zeilen
		for (int j = i; j <= dim1_; j++) {
			// Die Spalte enth�lt nur Nullen => det(M) = 0
			if (j == dim1_) return 0;
			// Suche die erste Zeile (beginnend bei i), die ungleich Null ist
			if (mat.getEntry(i,j) != 0) {
				// Falls in der ersten Zeile (beginnend bei i) keine Null steht:
				if (i != j) {
					// Vertausche Zeile i und j
					T* t = mat.entries_[i];
					mat.entries_[i] = mat.entries_[j];
					mat.entries_[j] = t;
					prod *= -1;
				}
				break;
			}
		}
		// f�r jede folgende Zeile
		for (int j = i + 1; j < dim1_; j++) {
			// Ziehe i*fac von j ab, sodass an in Spalte i ab Zeile j nur noch Nullen stehen
			T fac = mat.getEntry(j,i) / mat.getEntry(i,i);
			for (int k = 0; k < dim2_; k++) {
				mat.updateEntry(j,k, mat.getEntry(j,k) - mat.getEntry(i,k) * fac);
			}
		}

	}

	// Produkt bilden
	for (int i = 0; i < dim1_; i++) {
		prod *= mat.getEntry(i, i);
	}

	return prod;
}

template<class T>
Matrix <T>* Matrix<T>::cholesky(bool newMatrix, bool clear) {
	// Sicherheitsabfrage
	if (dim1_ != dim2_) throw notSquare_exception("cholesky: nicht quadratisch");

	// ggf. neue Matrix erstellen
	Matrix <T> *mat;
	if (newMatrix) mat = new Matrix <T>(*this);
	else mat = this;

	// Algorithmus nach Wikipedia
	for (int i = 0; i < dim1_; i++) {
		for (int j = 0; j < i; j++) {
			T sum = mat->getEntry(i, j);
			for (int k = 0; k < j; k++)
				sum -= mat->getEntry(i, k) * mat->getEntry(j, k);
			mat->updateEntry(i, j, sum / mat->getEntry(j,j));
		}
		T sum = mat->getEntry(i, i);
		for (int k = 0; k < i; k++) {
			sum = sum - mat->getEntry(i, k) * mat->getEntry(i, k);
		}
		if (sum <= 0) { // mat ist nicht positiv definit
			if (newMatrix) delete mat;
			throw notPositiveDefinit("cholesky: Matrix ist nicht positiv definit");
			//return new Matrix <T>();
		}
		T erg = (T) sqrt(sum);
		mat->updateEntry(i, i, erg);
	}

	// Mit Nullen auff�llen
	if (clear) {
		for (int i = 0; i < dim1_; i++) {
			for (int j = i+1; j < dim2_; j++) {
				mat->updateEntry(i, j, 0);
			}
		}
	}

	return mat;
}

template <class T>
svdData<T> Matrix<T>::svd() const {
	return svd(SVD_BEST);
}

template <class T>
svdData<T> Matrix<T>::svd_small() const {
	return svd(SVD_BEST | SVD_OPT_SMALL);
}

template <class T>
svdData<T> Matrix<T>::svd(unsigned int system) const {
	using namespace useful;

	if (system & SVD_CPPL) return svd_lapack(system & ~SVD_CPPL);
	/* SVD: A = USVT
	 * 1. ata = ATA von A
	 * 2. cp = CP von ata // Charakteristisches Polynom, siehe Lesezeichen
	 * 3. nst = NST von cp // Nullstellen, siehe Lesezeichen
	 * 4. nst = sort (sqrt nst)
	 * 5. s = MAT nst // Matrix
	 * 6. s^-1 // trivial!
	 * 7. for each nst n: xi = solve (ata - MAT n)*x=0
	 * 8. v = x1...xn; vt = v^T;
	 * 9. u = AVS^-1
	 */

	long long t0 = clock();
	// 1. Berechne A^T * A (� dim2_ x dim2_)
	Matrix <T> ata = calcATA();
	if (debug_) {useful::printTime("--> SVD_1 (ATA): ", t0);}

	// 2. Bestimme Eigenwerte und Vektoren
	eigenData<T> eig = ata.eigen(system);
	int anz = eig.values.size();
	T* nst = &eig.values[0];
	Matrix <T>& v = eig.vectors;
	if (debug_) {useful::printTime("--> SVD_2 (eigen): ", t0);}

	// 3. Erstelle (implizit) S^-1-Matrix
	std::vector<T> nstinv(anz);
	for (int i = 0; i < anz; i++)
		nstinv[i] = (T) (1 / sqrt(fabs(nst[i])));
	if (debug_) {useful::printTime("--> SVD_3 (conv): ", t0);}

	// 4. u = AVS^-1
	Matrix <T> u = multiplyMix(&v).multiplyWithDiagonal(&nstinv[0], anz,
			(system & SVD_OPT_SMALL) ? anz : dim1_);

	// 5. Return
	for (int i = 0; i < anz; i++)
		nst[i] = (T) sqrt(fabs(nst[i]));
	v.transpose();

	std::vector<T> val = eig.values;
	svdData<T> sd = {val, u, v};
	//if (debug_) {useful::printTime("--> SVD_4 (finish): ", t0); std::cout << std::endl;}
	return sd;
}

template <class T>
svdData<T> Matrix<T>::svd_lapack(unsigned int system) const {
	using namespace std;

#ifndef DONT_USE_LAPACK
	// CPPL-Matrix erstellen
	CPPL::dgematrix mat = toCPPL();
	CPPL::dgematrix u_, vt_;
	CPPL::dcovector s_;

	// SVD
	if (system & SVD_OPT_SMALL) {
		// Kleine U- und VT-Matrizen
		// Ver�nderter CPPL-Code
		int x = std::min(dim1_, dim2_);
		char JOBU('S'), JOBVT('S');
		long LDA(dim1_), LDU(dim1_), LDVT(x),
		LWORK(std::max(3*std::min(dim1_,dim2_)+std::max(dim1_,dim2_),5*std::min(dim1_,dim2_))), INFO(1);
		vector<double> WORK(LWORK);
		CPPL::dgematrix a = toCPPL();

		s_.resize(x); u_.resize(LDU,x); vt_.resize(LDVT,dim2_);

		dgesvd_(JOBU, JOBVT, dim1_, dim2_, a.array, LDA, s_.array, u_.array,
				LDU, vt_.array, LDVT, &WORK[0], LWORK, INFO);
//		LWORK = (3*min(dim1_, dim2_) + max(max(dim1_, dim2_), 4*min(dim1_, dim2_)*min(dim1_, dim2_) +4*min(dim1_, dim2_)));
//		vector<double> WORK(LWORK);
//		vector<long> IWORK(8*min(dim1_, dim2_));
//		dgesdd_(JOBU, dim1_, dim2_, a.array, LDA, s_.array, u_.array,
//				LDU, vt_.array, LDVT, WORK, &LWORK[0], &IWORK[0], INFO);

		if(INFO!=0){
			stringstream ss;
			ss << "error: svd_lapack: lapack: Serious trouble happend. INFO = " << INFO << "." << endl;
			throw runtime_error(ss.str());
		}
	} else {
		// Gro�e U- und VT-Matrizen
		mat.dgesvd(s_, u_, vt_);
	}

	// Zielmatrizen erstellen
	Matrix <T> u = Matrix <T>::fromCPPL(u_);
	Matrix <T> vt = Matrix <T>::fromCPPL(vt_);
	vector<T> s(s_.l);
	for (unsigned int i = 0; i < s.size(); i++)
		s[i] = (T) s_.array[i];

	// R�ckgabe
	svdData<T> sd = {s, u, vt};
	return sd;
#else
	return svd(SVD_FAST | system);
#endif
}

#ifndef DONT_USE_LAPACK
template <class T>
Matrix <T> Matrix<T>::fromCPPL(const CPPL::dgematrix& cppl) {
	Matrix <T> m = Matrix <T> (cppl.m, cppl.n);
	for (int i = 0; i < m.dim1_; i++)
		for (int j = 0; j < m.dim2_; j++)
			m.updateEntry(i, j, (T) cppl.darray[j][i]);
	return m;
}

template <class T>
CPPL::dgematrix Matrix<T>::toCPPL() const {
	CPPL::dgematrix mat(dim1_, dim2_);
	for (int i = 0; i < dim1_; i++)
		for (int j = 0; j < dim2_; j++)
			mat.set(i, j, getEntry(i,j));
	return mat;
}
#endif


template <class T>
Matrix <T> Matrix<T>::fromVectorRow(const std::vector<T>& v) {
	Matrix <T> m(1, v.size());
	for (unsigned int i = 0; i < v.size(); i++)
		m(0, i) = v[i];
	return m;
}

template <class T>
Matrix <T> Matrix<T>::fromVectorColumn(const std::vector<T>& v) {
	Matrix <T> m(v.size(), 1);
	for (unsigned int i = 0; i < v.size(); i++)
		m(i, 0) = v[i];
	return m;
}

template <class T>
std::vector<T> Matrix<T>::toVector() const {
	if (dim1_ == 1) {
		std::vector<T> v(dim2_);
		for (int i = 0; i < dim2_; i++)
			v[i] = getEntry(0, i);
		return v;
	} else if (dim2_ == 1) {
		std::vector<T> v(dim1_);
		for (int i = 0; i < dim1_; i++)
			v[i] = getEntry(i, 0);
		return v;
	} else {
		throw dim_exception("toVector: matrix is not a vector");
	}
	// Warnung verhindern:
	return std::vector<T>();
}

//template<class T>
//void Matrix<T>::ref() {
//	Matrix <T> b = Matrix <T>();
//	ref(&b);
//}
//template<class T>
//void Matrix<T>::rref() {
//	Matrix <T> b = Matrix <T>();
//	rref(&b);
//}

namespace {
	bool almostEqual(const double a, const double b, const double epsilon) {
		const double diff = a >= b ? a - b : b - a;
		const double sum = (a >= 0 ? a : -a) + (b >= 0 ? b : -b);
		if (diff <= epsilon * sum) return true;
		return false;
	}
}

template<class T>
void Matrix<T>::ref(Matrix <T> *b) {
	Matrix<T> b_empty;
	if (b == NULL) b = &b_empty;

	// Sicherheitspr�fung
	if (dim1_ != b->dim1_ && b->dim2_ != 0) throw dim_exception("ref: dim");

	// Matrizen explizit r�ck-transponieren, falls erforderlich
	if (transposed_) {
		T** old = entries_;
		try {
			entries_ = createField(dim1_, dim2_);
			for (int i = 0; i < dim1_; i++) {
				for (int j = 0; j < dim2_;j++) {
					entries_[i][j] = old[j][i];
				}
			}
			transposed_ = false;
			for (int i = 0; i < dim2_; i++)
				delete old[i];
			delete old;
		} catch (...) {
			cleanup(entries_, dim1_);
			entries_ = old;
		}
	}
	if (b->transposed_ && b->dim2_ != 0) {
		T** old = b->entries_;
		try {
			b->entries_ = new T*[b->dim1_];
			for (int i = 0; i < b->dim1_; i++) {
				b->entries_[i] = new T[b->dim2_];
				for (int j = 0; j < b->dim2_;j++) {
					b->entries_[i][j] = old[j][i];
				}
			}
			b->transposed_ = false;
			for (int i = 0; i < b->dim2_; i++)
				delete old[i];
			delete old;
		} catch (...) {
			cleanup(b->entries_, b->dim1_);
			b->entries_ = old;
		}

	}

	// Betrachte Elemente mij mit i <= zeile und j <= spalte
	int zeile = 0;
	int spalte = 0;
	while (zeile < dim1_ && spalte < dim2_) {
		// Suche das betragsm��ig gr��te Element der aktuellen Spalte, das ungleich null ist
		int biggest = -1;
		T value = -1;
		for (int i = zeile; i < dim1_; i++)
			if (getEntry(i, spalte) != 0 && fabs(getEntry(i, spalte)) > value) {
				biggest = i;
				value = (T) fabs(getEntry(i, spalte));
			}
		if (biggest == -1) {
			// Alle Elemente der Spalte sind gleich null => n�chste Spalte betrachten
			spalte++;
			continue;
		}

		// Das erste Element der Spalte ist ungleich null => Vertauschen
		if (biggest != zeile) {
			// Vertausche Zeilen bei "this"
			if (transposed_) {
				for (int j = 0; j < dim2_; j++) {
					T t = getEntry(zeile, j);
					updateEntry(zeile, j, getEntry(biggest, j));
					updateEntry(biggest, j, t);
				}
			} else {
				// Effizientes Vertauschen bei nicht-transponierten Matrizen
				T* t = entries_[zeile];
				entries_[zeile] = entries_[biggest];
				entries_[biggest] = t;
			}

			// Vertausche Zeilen bei "b" (analog zu "this")
			if (b->dim2_ != 0) {
				if (b->transposed_) {
					for (int j = 0; j < b->dim2_; j++) {
						T t = b->getEntry(zeile, j);
						b->updateEntry(zeile, j, b->getEntry(biggest, j));
						b->updateEntry(biggest, j, t);
					}
				} else {
					T* t = b->entries_[zeile];
					b->entries_[zeile] = b->entries_[biggest];
					b->entries_[biggest] = t;
				}
			}
		}

		// Sorge daf�r, dass das erste Element der Zeile gleich 1 ist
		T div = getEntry(zeile, spalte);
		for (int j = spalte; j < dim2_; j++)
			updateEntry(zeile, j, getEntry(zeile, j) / div);
		for (int j = 0; j < b->dim2_; j++)
			b->updateEntry(zeile, j, b->getEntry(zeile, j) / div);

		// Sorge daf�r, dass in der Spalte nach dem ersten Element nur Nullen stehen
//		T div = getEntry(zeile, spalte);
		const bool fast = true;
		const double epsilon = 1 / ACC_REF_EQUAL_DIV;
		for (int i = zeile + 1; i < dim1_; i++) {
			const T fac = getEntry(i, spalte); // / div;
			updateEntry(i, spalte, 0);
			if (fast) {
				T *ez = entries_[zeile],
				  *ei = entries_[i];
				for (int j = spalte+1; j < dim2_; j++) {
					// Handoptimierter Code
					T* eij = &ei[j];
					const T d = *eij,
							e = fac * ez[j];
					const T diff = d >= e ? d - e : e - d;
					const T sum = (d >= 0 ? d : -d) + (e >= 0 ? e : -e);
					if (diff <= epsilon * sum) *eij = 0;
					else *eij = d - e;

//					const T d = entries_[i][j],
//							e = fac * entries_[zeile][j];
//					entries_[i][j] = almostEqual((double) d,(double) e, epsilon) ? 0 : d - e;
				}
				for (int j = 0; j < b->dim2_; j++) {
					T d = b->entries_[i][j],
					  e = fac * b->entries_[zeile][j];
					b->entries_[i][j] = almostEqual((double) d,(double) e, epsilon) ? 0 : d - e;
				}
			} else {
				for (int j = spalte+1; j < dim2_; j++) {
					T d = getEntry(i, j),
					  e = fac * getEntry(zeile, j);
					updateEntry(i, j, almostEqual((double) d,(double) e, epsilon) ? 0 : d - e);
				}
				for (int j = 0; j < b->dim2_; j++) {
					T d = b->getEntry(i, j),
					  e = fac * b->getEntry(zeile, j);
					b->updateEntry(i, j, almostEqual((double) d,(double) e, epsilon) ? 0 : d - e);
				}
			}

		}

		// Bearbeite die n�chste Zeile und Spalte
		spalte++;
		zeile++;
	}
}

template<class T>
void Matrix<T>::rref(Matrix <T> *b) {
	/* Zitat: http://de.wikipedia.org/wiki/Gau�-Jordan-Algorithmus
	 * 1.) Man w�hlt die erste Spalte von links, in der mindestens ein von Null verschiedener Wert steht.
	 * 2.) Ist die oberste Zahl der gew�hlten Spalte eine Null, so vertauscht man die erste Zeile mit einer anderen Zeile, in der in dieser Spalte keine Null steht.
	 * 3.) Man dividiert die erste Zeile durch das nun oberste Element der gew�hlten Spalte.
	 * 4.) Man subtrahiere von den �brigen Zeilen entsprechende Vielfache der ersten Zeile mit dem Ziel, dass das erste Element jeder Zeile (au�er der ersten) Null wird.
	 * 5.) Durch Streichen der ersten Zeile und Spalte erh�lt man eine Restmatrix auf die man diese Verfahren wieder anwendet. Das f�hrt man solange durch, bis die Matrix in Zeilenstufenform ist.
	 * 6.) Man zieht danach von den dar�berliegenden Zeilen entsprechende Vielfache ab, sodass �ber einer f�hrenden 1 nur Nullen stehen. */

	Matrix<T> b_empty;
	if (b == NULL) b = &b_empty;

	// Sicherheitspr�fung
	if (dim1_ != b->dim1_ && b->dim2_ != 0) throw dim_exception("rref: dim");

	ref(b);

	// Die Matrix befindet sich jetzt in Stufenform
	// Bringe die Matrix auf die reduzierte Stufenform
	const double epsilon = 1 / ACC_REF_EQUAL_DIV;
	for (int zeile = dim1_ - 1; zeile >= 0; zeile--) {
		// Suche das erste Element der Zeile, in dem keine Null steht (sondern eine Eins)
		int nonzero = -1;
		for (int j = 0; j < dim2_; j++) {
			if (getEntry(zeile, j) != 0) {
				nonzero = j;
				break;
			}
		}
		// Falls in der Zeile nur Nullen stehen, so mache mit der n�chsten Zeile weiter
		if (nonzero == -1) continue;

		// Sorge daf�r, dass �ber dem ersten Element (das gleich eins ist) nur Nullen stehen
		for (int i = 0; i < zeile; i++) {
			T fac = getEntry(i, nonzero);
			for (int j = nonzero; j < dim2_; j++) {
				T d = getEntry(i, j),
				  e = fac * getEntry(zeile, j);
				updateEntry(i, j, almostEqual((double) d, (double) e, epsilon) ? 0 : d - e);
			}
			for (int j = 0; j < b->dim2_; j++) {
				T d = b->getEntry(i, j),
				  e = fac * b->getEntry(zeile, j);
				b->updateEntry(i, j, almostEqual((double) d,(double) e, epsilon) ? 0 : d - e);
			}
		}
	}
}

template <class T>
Matrix <T> Matrix<T>::pseudo_inv() const {
	// SVD-Berechnung
	svdData<T> sd = svd(SVD_CPPL | SVD_OPT_SMALL);

    Matrix<T> ut = sd.u;
    ut.transpose();
    Matrix<T> v = sd.vt;
    v.transpose();
    double zero = sd.s[0];
    zero /= ACC_PINV_ZERO_DIV;

	for (unsigned int i = 0; i < sd.s.size(); i++) sd.s[i] = sd.s[i] < zero ? 0 : 1 / sd.s[i]; // Fast-null-Werte auf null setzen
	Matrix <T> erg = v.multiplyWithDiagonal(&sd.s[0], sd.s.size(), ut.dim1_).multiplyMix(&ut);

	return erg;
}

template<class T>
Matrix <T> Matrix<T>::invert() const {
	// Sicherheitspr�fung
	if (dim1_ != dim2_) throw notSquare_exception("invert: nicht quadratisch");

	Matrix <T> mat = Matrix <T> (*this);
	Matrix <T> b = Matrix<T> (dim1_, dim2_);
	for (int i = 0; i < dim1_; i++)
		for (int j = 0; j < dim2_; j++)
			b.updateEntry(i, j, i == j ? 1 : 0);

	mat.rref(&b);

	bool inv = true; // invertierbar
	for (int i = 0; i < dim1_; i++)
		for (int j = 0; j < dim2_; j++)
			if (mat.getEntry(i,j) != (i == j ? 1 : 0)) inv = false;

	if (inv) return b;
	else throw notInvertible("invert: not invertible");
}

template <class T>
Matrix <T> Matrix<T>::relDeviation(const Matrix<T>& b) const {

	if (dim1_ != b.dim1_ || dim2_ != b.dim2_) throw dim_exception("compare: dim");
	Matrix <T> c(dim1_, dim2_);
	for (int i = 0; i < dim1_; i++)
		for (int j = 0; j < dim2_; j++) {
			T t = (T) (fabs(b.getEntry(i, j)) / fabs(getEntry(i, j)));
			c.updateEntry(i, j, t < 1 ? 1 / t : t);
		}
	return c;
}

template <class T>
std::vector<T> Matrix<T>::jacobi(const Matrix <T> *b) const {
	// Quelle: http://de.wikipedia.org/wiki/Jacobi-Verfahren
	// Sicherheitspr�fung
	if (dim1_ != dim2_) throw notSquare_exception("jacobi: nicht quadratisch");
	if (b->dim1_ != dim1_) throw dim_exception("jacobi: 'b' nicht passend");

	// Initialisierung
	int n = dim1_;
	std::vector<T> erg_vec(n);
	T* erg = &erg_vec[0];
	for (int i = 0; i < n; i++) {
		// Semi-willk�rlicher Startwert (0 w�re schlecht (triviale L�sung))
		erg[i] = 1;
	}

	T epsilon = (T) (frobeniusnorm() / dim1_ / dim2_ / ACC_JACOBI_FAC);
	T delta = (T) (frobeniusnorm() * ACC_JACOBI_FAC);

	// Iterationen
	bool stopE = false, stopD = false;
	while (!stopE && !stopD) {
		T x[n];
		stopE = true;

		// Iterationsschritt
		for (int i = 0; i < n; i++) {
			x[i] = 0;
			for (int j = 0; j < n; j++) {
				if (i != j) x[i] += + getEntry(i,j) * erg[j];
			}
			x[i] = (b->getEntry(i,0) - x[i]) / getEntry(i,i);
		}

		// Abbruchsauswertung
		for (int i = 0; i < n; i++) {
			// Genauigkeit hoch genug?
			T diff = erg[i] - x[i];
			if (diff < 0) diff = -diff;
			if (diff > epsilon) stopE = false; // falls eine Werte-Differenz gr��er als (das sehr kleine) Epsilon ist, so fahre den Algorithmus fort
			if (diff > delta) stopD = true; // fals eine Werte-Differenz gr��er als (das sehr gro�e) Delta ist, so breche ab, denn der Algorithmus terminiert nicht

			// Werte �bertragen
			erg[i] = x[i];
		}
	}

	// R�ckgabe (null-pointer, falls das Verfahren nicht konvergiert)
	if (stopD) throw noConvergence("jacobi: keine Konvergenz");
	return erg_vec;
}

template <class T>
std::vector<T> Matrix<T>::sor(const Matrix <T> *b, T omega) const {
	// Quelle: http://de.wikipedia.org/wiki/SOR-Verfahren
	// Sicherheitspr�fung
	if (dim1_ != dim2_) throw notSquare_exception("sor: nicht quadratisch");
	if (b->dim1_ != dim1_) throw dim_exception("sor: 'b' nicht passend");

	// Initialisierung
	int n = dim1_;
	std::vector<T> erg_vec(n);
	T* erg = &erg_vec[0];
	for (int i = 0; i < n; i++) {
		// Semi-willk�rlicher Startwert (0 w�re schlecht (triviale L�sung))
		erg[i] = 1;
	}

	T epsilon = (T) (frobeniusnorm() / dim1_ / dim2_ / ACC_JACOBI_FAC);
	T delta = (T) (frobeniusnorm() * ACC_JACOBI_FAC);

	// Iterationen
	bool stopE = false, stopD = false;
	while (!stopE && !stopD) {
		T x[n];
		stopE = true;

		// Iterationsschritt
		for (int k = 0; k < n; k++) {
			x[k] = 0;
			for (int i = 0; i < k; i++)
				x[k] += getEntry(k,i) * x[i];
			for (int i = k+1; i < n; i++)
				x[k] += getEntry(k,i) * erg[i];
			x[k] = omega * (b->getEntry(k,0) - x[k]) / getEntry(k,k);
			x[k] += (1-omega) * erg[k];
		}

		// Abbruchsauswertung
		for (int i = 0; i < n; i++) {
			// Genauigkeit hoch genug?
			T diff = erg[i] - x[i];
			if (diff < 0) diff = -diff;
			if (diff > epsilon) stopE = false; // falls eine Werte-Differenz gr��er als (das sehr kleine) Epsilon ist, so fahre den Algorithmus fort
			if (diff > delta) stopD = true; // fals eine Werte-Differenz gr��er als (das sehr gro�e) Delta ist, so breche ab, denn der Algorithmus terminiert nicht

			// Werte �bertragen
			erg[i] = x[i];
		}
	}

	// R�ckgabe (null-pointer, falls das Verfahren nicht konvergiert)
	if (stopD) throw noConvergence("jacobi: keine Konvergenz");
	return erg_vec;
}

template <class T>
Matrix<T> Matrix<T>::householder(const T* vector, int size) {
	// H = I - 2 / vtv * vvt
	Matrix <T> h = Matrix <T> (size, size);
	T vtv = 0;
	for (int i = 0; i < size; i++)
		vtv += vector[i] * vector[i];
	for (int i = 0; i < size; i++)
		for (int j = 0; j < size; j++)
			h.updateEntry(i, j, (i == j ? 1 : 0) - 2 / vtv * vector[i]*vector[j]);
	return h;
}

template<class T>
std::pair<Matrix <T>, Matrix <T> > Matrix<T>::qr() const {
	Matrix <T> q;
	Matrix <T> r;

	// Quelle: http://de.wikipedia.org/wiki/Givens-Rotation

	q = Matrix <T> (dim1_, dim1_);
	for (int i = 0; i < dim1_; i++)
		for (int j = 0; j < dim1_; j++)
			q.updateEntry(i,j, i == j ? 1 : 0);
	r = Matrix <T> (dim1_, dim2_);
	for (int i = 0; i < dim1_; i++)
		for (int j = 0; j < dim1_; j++)
			r.updateEntry(i,j, getEntry(i,j));

	// Q = G1T*G2T*... = (...*G2*G1)T
	// R = ...*G2*G1*A = QT*A

	// f�r jede Spalte
	for (int j = 0; j < dim2_; j++) {
		T rjj = r.getEntry(j,j);
		// f�r jede Zeile
		for (int i = j+1; i < dim1_; i++) {
			T rij = r.getEntry(i, j);
			if (rij != 0) {
				T sgn = rjj > 0 ? 1 : rjj < 0 ? -1 : 0;
				T p = sgn * (T) sqrt(rjj*rjj + rij*rij);
				T c = rjj / p;
				T s = rij / p;
				// G * X = Z
				// zkl = { xkl, falls k != i, k != j
				//         f�r k = i: gki * xil + gkj * xjl
				//         f�r k = j: gkj * xjl + gki * xil
				// gii = gjj = c, gji = s, gij = -s
				for (int l = 0; l < dim1_; l++) {
					// handoptimierter Code: q darf nicht transponieren sein!
					T qil = c * q.entries_[i][l] - s * q.entries_[j][l];
					q.entries_[j][l] = c * q.entries_[j][l] + s * q.entries_[i][l];
					q.entries_[i][l] = qil;
				}
				for (int l = 0; l < dim2_; l++) {
					// handoptimierter Code: r darf nicht transponieren sein!
					T ril = c * r.entries_[i][l] - s * r.entries_[j][l];
					r.entries_[j][l] = c * r.entries_[j][l] + s * r.entries_[i][l];
					r.entries_[i][l] = ril;
				}
			}
		}
	}
	q.transpose();

	return std::pair<Matrix <T>, Matrix <T> > (q,r);
}

template <class T>
void Matrix<T>::hessenberg(unsigned int system) {
	using namespace useful;
	using namespace std;

	int n = dim1_;
	// Hessenberg-Form
	/* for k=1 to n-1
	 * 	u = A(k+1:n, k)
	 * 	Pk = I - 2vvT
	 * 	Pk = (Ik 0, 0, Pk)
	 * A = PkAPk
	 * endfor
	 * Beschleunigung durch schnelle matrix-vektor-mult mit H (?)
	 */
	//long long t1;
	//if (debug_) {t1 = clock();}
	//if (debug_) {cout << "Hessenberg-Transformation: [" << n << "]" << endl;}
	vector<T> u_vec(n-1);
	T* u = &u_vec[0];
	for (int k = 0; k < n-1; k++) {
		// Erstelln von u = A(k+1:n, k)
		int size = n - k - 1;
		T length = 0;
		for (int i = 0; i < size; i++) {
			u[i] = getEntry(k+1+i, k);
			length += u[i]*u[i];
		}
		length = (T) sqrt(length);
		if (length == 0) continue;

		// Anpassen von der ersten Koordinate f�r die HHT
		u[0] += length * (u[0] < 0 ? -1 : 1);

		// Normieren von u
		length = 0;
		for (int i = 0; i < size; i++) {
			length += u[i]*u[i];
		}
		length = (T) sqrt(length);
		for (int i = 0; i < size; i++)
			u[i] /= length;

		// Berechnen der Householdertransformation, sodass u auf den ersten Einheitsvektor abgebildet wird
		Matrix <T> h = householder(u, size);

		// Ausf�hren der Transformation
		// Idee: h � k x k, akt' � k x n (letzte Zeilen)
		//     => h * akt' =: akt_p @ k x n
		Matrix <T> akt_kn = select(n - size, 0, size, n);
		Matrix <T> akt_p1 = Matrix(size,n);
		// H*x = (x - <u,x>*u) - <u,x>*u = x - 2*<u,x>*u
		for (int j = 0; j < n; j++) {
			T ux = 0;
			for (int i = 0; i < size; i++)
				ux += u[i] * akt_kn.getEntry(i,j);
			ux *= 2;
			for (int i = 0; i < size; i++)
				akt_p1.updateEntry(i,j, akt_kn.getEntry(i,j) - ux*u[i]);
		}

		for (int i = 0; i < size; i++)
			for (int j = 0; j < n; j++)
				updateEntry(i + n - size,j, akt_p1.getEntry(i,j));
		//     -> akt_p' � n x k (rechter Teil von akt_p und oben rechter Teil von akt'
		//     => akt_p' * h =: akt_pp � n x k
		Matrix <T> akt_nk = select(0,n-size,n,size);
		Matrix <T> akt_p2 = akt_nk.multiplyMix(&h);
		//     -> akt_new = [[A(links oben) akt_pp(oben)] [akt_p(links) akt_pp(unten)]]
		for (int i = 0; i < n; i++)
			for (int j = 0; j < size; j++)
				updateEntry(i,j + n - size, akt_p2.getEntry(i,j));

		// Aufr�umen

		/*if (debug_) {cout << k+1 << ","; cout.flush();
					if ((k+1) % 25 == 0) {useful::printTime("  --> ", t1);}} */
	}
	//if (debug_) {cout << endl;}

	// Ausgleichen von Rechenfehlern
	for (int i = 0; i < n; i++)
		for (int j = 0; j < n; j++) {
			if (i > j+1 || (i < j-1 && (system & MAT_SYM)))
				updateEntry(i,j, 0);
		}
}

template <class T>
std::pair<Matrix <T>, Matrix<T> > Matrix<T>::qr_algo(unsigned int system) const {
	using namespace useful;
	using namespace std;

	if (dim1_ != dim2_) throw notSquare_exception("qr_algo: nicht quadratisch");

	// Variablen deklarieren/initialisieren
	long long t0 = clock();
	int n = dim1_;
	Matrix <T> akt = Matrix <T> (*this);
	Matrix <T> evec;
	T epsilon = (T) (frobeniusnorm() / dim1_ / dim2_ / ACC_QRA_STOP_DIV);
	//if (debug_) {cout << endl << "--> QR-Algo" << endl << "Epsilon: " << epsilon << endl;}

	// ggf. Hessenberg-Transformation
	if (system & QRA_HESSENBERG) {
		akt.hessenberg(system);
		if (debug_) {useful::printTime("--> QR-Algo (hessen): ", t0);}
	}

	// QR-Algorithmus
	int i;
	//if (debug_) {cout << "QR-Algorithmus: " << endl;}
	if ((system & QRA_HESSENBERG) && (system & MAT_SYM) && (system & QRA_FASTQR)) {
		vector<T> a_vec(n+1);
		vector<T> b_vec(n);
		T* a = &a_vec[0];
		T* b = &b_vec[0];
		T lambda, psquare, u, sqpre, sqcur;

		sqpre = 0; // Compiler-Warnung verhindern (eigenlich nicht n�tig)

		// Schreibe Werte der Tridigonalmatrix in zwei Arrays
		for (int k = 0; k < n; k++)
			a[k] = akt.getEntry(k,k);
		for (int k = 1; k < n; k++)
			b[k-1] = akt.getEntry(k, k-1) * akt.getEntry(k, k-1);

		// Iteriere
		for (i = 0;; i++) {
			T ki;

			// QR-Algo mit einfach Shift f�r Sym-Matrizen
			T a_ = a[n-2],
			  b_ = (T) sqrt(b[n-2]),
			  c_ = (T) sqrt(b[n-2]),
			  d_ = a[n-1];
			// Quelle: http://en.wikipedia.org/wiki/Eigenvalue_algorithm
			T eig1 = (a_+d_)/2 + (T) sqrt(((a_+d_)*(a_+d_))/4 + b_*c_ - a_*d_);
			T eig2 = (a_+d_)/2 - (T) sqrt(((a_+d_)*(a_+d_))/4 + b_*c_ - a_*d_);
			ki = fabs(eig1 - d_) < fabs(eig2 - d_) ? eig1 : eig2;
			if (ki != ki) ki = d_; // if (ki is NaN) use QRA_ANN istead

			// Shift ausf�hren
			for (int j = 0; j < n; j++)
				a[j] -= ki;

			// Berechnet "QR := A; A' := RQ" in O(n)-Zeit;
			// Quelle: http://comjnl.oxfordjournals.org/content/6/1/99.full.pdf
			u = 0;
			sqcur = 0;
			b[n-1] = 0;
			a[n] = 0;
			for (int j = 0; j < n; j++) {
				lambda = a[j] - u;
				if (sqcur <= 1 - epsilon*epsilon/ACC_QRA_FASTQR_ONE_DIV || sqcur >= 1 + epsilon*epsilon/ACC_QRA_FASTQR_ONE_DIV)
					psquare = lambda * lambda / (1 - sqcur);
				else
					psquare = (1 - sqpre) * b[j-1];
				if (j != 0) {
					b[j-1] = sqcur * (psquare + b[j]);
				}

				sqpre = sqcur;
				if (b[j] != 0)
					sqcur = b[j] / (psquare + b[j]);
				else if (psquare != 0)
					sqcur = 0;
				else
					sqcur = 1;
				u = sqcur * (lambda + a[j+1]);
				a[j] = lambda + u;
			}

			// Shift r�ckg�ngig machen
			for (int j = 0; j < n; j++)
				a[j] += ki;

			/*if (debug_) {
				if (i % 10000 == 0) {
					T max = 0;
					for (int k = 1; k < n; k++)
						if (fabs(b[k]) > max) max = b[k];
					cout << i << ":  max:" << sqrt(max) << endl;
				}
			}*/

			// Genauigkeit �berpr�fen und ggf. abbrechen
			bool ready = true;
			for (int j = 0; j < n-1 && ready; j++)
				if (fabs(b[j]) >= epsilon * epsilon) {
					ready = false;
				}
			if (ready) {
				break;
			}

		}

		// Array-Werte wieder in Matrix schreiben
		for (int i = 0; i < n; i++)
			for (int j = 0; j < n; j++) {
				if (i == j) akt.updateEntry(i,j, a[i]);
				else if (i == j+1) akt.updateEntry(i,j, (T) sqrt(b[i]));
				else if (i == j-1) akt.updateEntry(i,j, (T) sqrt(b[j]));
				else akt.updateEntry(i,j, 0);
			}

		// Aufr�umen
	} else {
		// Normaler QR-Algorithmus
		evec = Matrix <T> (n, n);
		for (i = 0; i < n; i++)
			for (int j = 0; j < n; j++)
				evec.updateEntry(i, j, i == j ? 1 : 0);

		// Iteriere
		for (i = 0;; i++) {
			// Shift bestimmen
			T ki;
			switch (system & QRA_SHIFT_) {
			case QRA_SHIFT_SYM: {
				// QR-Algo mit einfach Shift f�r Sym-Matrizen
				T a = akt.getEntry(n-2, n-2),
				  b = akt.getEntry(n-2, n-1),
				  c = akt.getEntry(n-1, n-2),
				  d = akt.getEntry(n-1, n-1);
				// Quelle: http://en.wikipedia.org/wiki/Eigenvalue_algorithm
				T eig1 = (a+d)/2 + (T) sqrt(((a+d)*(a+d))/4 + b*c - a*d);
				T eig2 = (a+d)/2 - (T) sqrt(((a+d)*(a+d))/4 + b*c - a*d);
				ki = fabs(eig1 - d) < fabs(eig2 - d) ? eig1 : eig2;
				if (ki != ki) ki = d; // if (ki is NaN) use QRA_ANN istead
				break; }
			case QRA_SHIFT_ANN:
				// QR-Algo mit einfach-Shift:
				ki = akt.getEntry(n-1, n-1);
				break;
			case QRA_SHIFT_ZERO:
				ki = 0;
				break;
			default:
				throw invalid_argument("qr_algo: unknown system");
			}

			// Shift ausf�hren
			for (int j = 0; j < n; j++)
				akt.addEntry(j, j, -ki);

			// QR-Zerlegung
			pair<Matrix <T>, Matrix <T> > qrd = akt.qr();

			// Evec-Aktualisierung
			Matrix <T> t = (evec * qrd.first);
			evec = t;

			// Berechnung von RQ
			if ((system & QRA_HESSENBERG) && (system & MAT_SYM)) {
				// schnelle Multiplikation
				// in R stehen nicht-null-Werte nur in (i,i), (i,i+1), (i,i+2)
				akt = Matrix <T> (n,n);
				for (int i = 0; i < n; i++)
					for (int j = 0; j < n; j++)
						if (i < j-1 || i > j+1) akt.updateEntry(i,j,0);
						else {
							T sum = 0;
							for (int k = i; k <= i+2; k++) {
								sum += qrd.second.getEntryCheck(i,k) * qrd.first.getEntryCheck(k,j);
							}
							akt.updateEntry(i,j, sum);
						}
			} else {
				akt = qrd.second.multiplyMix(&qrd.first);

				// Werte, die theoretisch immer 0 sind, aber praktisch nicht, auf 0 setzen
				if (system & QRA_HESSENBERG) {
					for (int k = 2; k < dim1_; k++)
						for (int l = 0; l <= k-2; l++)
							akt.updateEntry(k,l, 0);
				}
			}


			// Shift r�ckg�ngig machen
			for (int j = 0; j < n; j++)
				akt.addEntry(j, j, ki);

			// Debug-Ausgabe
			/*if (debug_) {
				if (i % 1000 == 0) {
					T max = 0;
					for (int k = 0; k < dim1_; k++)
						for (int l = 0; l < dim2_; l++)
							if (k != l && (T) fabs(akt.getEntry(k,l)) >= max)
								max = (T) fabs(akt.getEntry(k,l));
					cout << i << ":  max: " << max << endl;
				}
			}*/

			// Aufr�umen
//			delete qrd.first;
//			delete qrd.second;

			// Abbruch: Idee: Wenn nicht-Diagonal-Werte klein genug sind -> Abbruch
			bool ready = true;
			for (int k = 0; k < dim1_ && ready; k++)
				for (int l = 0; l < k /*dim2_*/ && ready; l++)
					if (k != l && fabs(akt.getEntry(k,l)) >= epsilon) {
						ready = false;
					}
			if (ready) {
				break;
			}
		}
		//if (debug_) {cout << endl;}
	}

	/*if (debug_) {T max = 0;
				for (int k = 0; k < dim1_; k++)
					for (int l = 0; l < dim2_; l++)
						if (k != l && (T) fabs(akt.getEntry(k,l)) >= max)
							max = (T) fabs(akt.getEntry(k,l));
				cout << i << ":  max:" << max << endl;
//				cout << i << ": " << endl << "   a: ";
//	            for (int i = 0; i < n; i++) cout << akt.getEntry(i,i) << ",";
//	            cout << endl << "   b: ";
//	            for (int i = 1; i < n; i++) cout << akt.getEntry(i,i-1) << ",";
//	            cout << endl;
//	            T max = 0;
//				for (int k = 0; k < dim1_; k++)
//					for (int l = 0; l < dim2_; l++)
//						if (k != l && (T) fabs(akt.getEntry(k,l)) >= max)
//							max = (T) fabs(akt.getEntry(k,l));
//				cout << "   max: " << max << endl;
				}*/
	//if (debug_) {useful::printTime("--> QR-Algo (qra): ", t0);}

	return pair<Matrix <T>, Matrix<T> > (akt, evec);
}

template <class T>
std::vector<T> Matrix<T>::eigenvalues(unsigned int system) const {
	if (system & SVD_CPPL) {
#ifndef DONT_USE_LAPACK
		CPPL::dgematrix mat = toCPPL();
		std::vector<double> wr, wi;
		mat.dgeev(wr, wi);
		std::vector<T> out(wr.size());
		for (unsigned i = 0; i < wr.size(); i++)
			out[i] = (T) wr[i];
		return out;
#else
		system = SVD_FAST;
#endif
	}

	// Sicherheitsabfrage
	if (dim1_ != dim2_) throw notSquare_exception("eigenvalues: nicht quadratisch");

	std::vector<T> nst;

	Matrix <T> t = qr_algo(system).first;
	nst = std::vector<T>(dim2_);
	T epsilon = (T) (t.frobeniusnorm() / dim2_ / dim2_ / ACC_SVD_EIGEN_ZERO_DIV);
	int anz = 0;
	for (int i = 0; i < dim2_; i++) {
		T val = t.getEntry(i,i);
		if (fabs(val) < epsilon) {

		} else nst[anz++] = val;
	}
	nst.resize(anz);

	return nst;
}

template <class T>
eigenData<T> Matrix<T>::eigen(unsigned int system) const {
	using namespace std;
	using namespace useful;


	if (system & SVD_CPPL) {
#ifndef DONT_USE_LAPACK
		CPPL::dgematrix mat = toCPPL();
		vector<double> wr, wi;
		vector<CPPL::dcovector> vrr, vri;
		mat.dgeev(wr, wi, vrr, vri);

		vector<T> evalues(wr.size());
		for (unsigned i = 0; i < wr.size(); i++)
			evalues[i] = (T) wr[i];
		Matrix<T> evectors(vrr[0].l, vrr.size());
		for (int i = 0; i < vrr[0].l; i++) // f�r jeden Eigenvektor
			for (unsigned j = 0; j < vrr.size(); j++) // f�r jedes Element
				evectors(j,i) = (T) vrr[i](j);
		eigenData<T> ed = {evalues, evectors};
		return ed;
#else
		system = SVD_FAST;
#endif
	}


	// Sicherheitsabfrage
	if (dim1_ != dim2_) throw notSquare_exception("eigen: nicht quadratisch");

	if (system & QRA_FASTQR) {
		// 1. Eigenwerte berechnen
		long long t0 = clock();
		vector<T> nst = eigenvalues(system);
		if (debug_) {useful::printTime("--> Eigenwerte: ", t0);}

		// 2. Sortieren
		qsort(&nst[0], nst.size(), sizeof (double), ::compare<T>);

		// 3. for each nst n: xi = solve (this - MAT (n))*x=0
		//if (debug_) {cout << "Solving: [" << nst.size() << "]" << endl;}
		//long long t1 = t0;
		Matrix <T> v = system & SVD_OPT_SMALL ? Matrix <T> (dim2_, nst.size()) : Matrix <T> (dim2_,dim2_);
		for (unsigned int i = 0; i < nst.size(); i++) {
			// ATA bearbeiten
			Matrix <T> mat = Matrix <T> (*this);
			for (int j = 0; j < dim2_; j++) {
				mat.updateEntry(j, j, mat.getEntry(j, j) - nst[i]);
				// Problem, wenn (j,j) == 0
			}

			// sor or standard
			vector<T> lsg(dim2_);
			for (int j = 0; j < dim2_; j++) lsg[j] = 0; // Default-Wert
			const bool useChol = false;
			T* erg_sor = 0;
			if (useChol) {
				// L�uft nicht, da Jacobi nicht immer konvergiert.
				//cout << *this;
				//Matrix <T>* mat_ = mat.cholesky(true, true);
				//if (debug_ && mat_->dim1_ != 0) cout << "NICE" << endl;
				if (erg_sor != 0) {
					//cout << "s";
					for (int i = 0; i < dim2_; i++) lsg[i] = erg_sor[i];
				}
			}
			if (!useChol || erg_sor == 0) {
				// a) Dreiecksform
				mat.ref();
				// b) Eine L�sung erzeugen
				// Analysiere die Struktur (structure[i] := Spalte mit der ersten nicht-null-Zahl in Zeile i)
				vector<int> structure(dim2_);
				int akt = -1;
				bool found = true;
				int zeile = 0;
				for (; zeile < dim2_ && found; zeile++) {
					found = false;
					for (int j = akt + 1; j < dim2_; j++) {
						if (mat.getEntry(zeile,j) != 0) {
							structure[zeile] = akt = j;
							found = true;
							break;
						}
					}
				}

				// Falls ref f�r mat.getEntry(n-1,n-1) == 1 lieferte, so korrigiere, denn
				// die letzte Zeile ist immer null (in der Theorie)
				if (zeile == dim2_) zeile--;
				structure[zeile] = dim2_;
				// F�r jede Zeile
				for (int j = zeile - 1; j >= 0; j--) {
					T sum = 0;
					for (int k = structure[j+1] - 1; k > structure[j]; k--) {
						lsg[k] = 1;
					}
					for (int k = dim2_ - 1; k > structure[j]; k--) {
						sum += lsg[k]*mat.getEntry(j, k);
					}
					lsg[j] = - sum / mat.getEntry(j, structure[j]); // Entry(j,j) ???
				}

				// Fehler-Korrektur, falls die erste Spalte = (0..0)
				bool fehlerKorrektur = true;
				for (int j = 0; j < dim1_; j++)
					if (mat.getEntry(j, 0) != 0)
						fehlerKorrektur = false;
				if (fehlerKorrektur) lsg[0] = 1;
			}

			// c) L�sung normieren
			T erg = 0;
			for (int j = 0; j < dim2_; j++)
				erg += lsg[j]*lsg[j];
			erg = (T) sqrt(erg);
			for (int j = 0; j < dim2_; j++)
				lsg[j] /= erg;

			// d) L�sung in v reinschreiben
			for (int j = 0; j < dim2_; j++) {
				v.updateEntry(j, i, lsg[j]);
			}
			//if (debug_) {cout << i + 1 << ","; cout.flush(); if ((i+1) % 25 == 0) useful::printTime("  --> ", t1);}
		}
		if (!(system & SVD_OPT_SMALL))
			for (int i = nst.size(); i < dim2_; i++)
				for (int j = 0; j < dim2_; j++)
					v.updateEntry(j, i, 0);
		if (debug_) {useful::printTime("\n--> Eigenvektoren: ", t0);}

		eigenData<T> ed = {nst, v};
		return ed;
	} else {
		pair<Matrix <T>, Matrix<T> > pair_ = qr_algo(system);

		Matrix <T>& t = pair_.first;
		std::vector<T> nst(dim2_);
		T epsilon = (T) (t.frobeniusnorm() / dim2_ / dim2_ / ACC_SVD_EIGEN_ZERO_DIV);
		int anz = 0;
		for (int i = 0; i < dim2_; i++) {
			T val = t.getEntry(i,i);
			if (fabs(val) < epsilon) {

			} else nst[anz++] = val;
		}
		nst.resize(anz);

		eigenData<T> ed = {nst, pair_.second};
		return ed;
	}
}

template <class T>
Matrix <T> Matrix<T>::regression(const Matrix <T> *b) const {
	// Sicherheitsabfrage
	if (dim1_ != b->dim1_ || b->dim2_ != 1) throw dim_exception("regression: Matrix b nicht passend");

	// SVD
	svdData<T> svd_ = svd(SVD_CPPL | SVD_OPT_SMALL);
	Matrix <T> ut = svd_.u; // U
	Matrix <T> v = svd_.vt; // VT
	std::vector<T> data = svd_.s;

	ut.transpose(); // U -> UT
	v.transpose(); // VT -> V
	for (unsigned int i = 0; i < data.size(); i++)
		data[i] = 1 / data[i]; // S -> S^-1

	/* Berechne: x = V * S^-1 * UT * b
	 * V:    (d x k)
	 * S^-1: (k x k)
	 * UT:   (k x n)
	 * b:    (n x 1)
	 * x:    (d x 1)
	 */
	int n = dim1_;
	int d = dim2_;
	int k = std::min(dim1_, dim2_);

	// utb = UT * b
	std::vector<T> utb(k);
	for (int i = 0; i < k; i++) { // eigentlich dim1_, aber dim2_ reicht aus!
		utb[i] = 0;
		for (int j = 0; j < n; j++) {
			utb[i] += ut.getEntryCheck(i,j) * b->getEntry(j,0);
		}
	}
	// siutb = S^-1 * utb
	std::vector<T> siutb(k);
	for (int i = 0; i < k; i++) {
		siutb[i] = data[i] * utb[i];
	}

	// erg = V * siutb
	Matrix <T> erg = Matrix <T> (d, 1);
	for (int i = 0; i < d; i++) {
		erg.updateEntry(i, 0, 0);
		for (int j = 0; j < k; j++) {
			erg.addEntry(i, 0, v.getEntry(i,j) * siutb[j]);
		}
	}

	// R�ckgabe
	return erg;
}

template <class T>
Matrix <T> Matrix <T>::covariance(bool normIn, bool normOut) const {
	Matrix <T> data = Matrix <T> (*this);
	int n = dim1_, d = dim2_;

	if (normIn) {
		// Berechnen der Erwartungswerte
		std::vector<T> erw(d);
		for (int i = 0; i < d; i++) {
			erw[i] = 0;
			for (int j = 0; j < n; j++) {
				erw[i] += getEntry(j,i);
			}
			erw[i] /= n;
		}

		// Abziehen der Erw
		for (int i = 0; i < n; i++)
			for (int j= 0; j < d; j++)
				data.addEntry(i, j, - erw[j]);
	}

	// Füllen der co-Matrix
	// cov(x,y) = Sum(xi - X)(yi -Y) / (n-1)
	Matrix <T> co = data.calcATA();

	// Ausgabe durch n-1 teilen
	if (normOut) {
		for (int i = 0; i < d; i++)
			for (int j= 0; j < d; j++)
				co.updateEntry(i, j, co.getEntry(i, j) / (n-1));
	}

	// Rückgabe
	return co;
}

template <class T>
void Matrix<T>::write(const char* path, char mode) const {
	switch (mode) {
	case RW_STANDARD: return writeStandard(path);
	case RW_BINARY: return writeBinary(path);
	case RW_R: return writeR(path);
	case RW_CSV: return writeCSV(path);
	default: throw std::invalid_argument("write: mode");
	}
}

template <class T>
Matrix<T> Matrix<T>::read(const char* path, char mode) {
	switch (mode) {
	case RW_STANDARD: return readStandard(path);
	case RW_BINARY: return readBinary(path);
	case RW_R: return readR(path);
	case RW_CSV: return readCSV(path);
	default: throw std::invalid_argument("read: mode");
	}
}

template <class T>
void Matrix<T>::writeStandard(const char* path) const {
	std::ofstream f;
//	f.exceptions (ios::failbit | ios::badbit);
	f.open(path, std::ios::out);
	if (!f) useful::err("error: write: f.open()");
	f << *this;
	f.close();
}

template <class T>
Matrix<T> Matrix<T>::readStandard(const char* path) {
	std::ifstream f;
//	f.exceptions (ios::failbit | ios::badbit);
	f.open(path, std::ios::in);
	if (!f) useful::err("error: read: f.open()");

	// Dimension auslesen: "m x n :"
	int dim1, dim2;
	std::string line;
	getline(f, line);
	int begin = 0;
	int end = line.find_first_of(' ', 0);
	dim1 = atoi(line.substr(begin, end - begin).data());
	begin = line.find_first_of(' ', end + 1) + 1;
	end = line.find_first_of(' ', begin + 1);
	dim2 = atoi(line.substr(begin, end - begin).data());
	Matrix <T> mat = Matrix <T> (dim1, dim2);

	mat.fill(0);

	// Daten auslesen
	for (int i = 0; i < dim1 && !f.eof(); i++) {
		getline(f, line);
		begin = line.find_first_of('[', 0) + 1;
		for (int j = 0; j < dim2; j++) {
			end = line.find_first_of(' ', begin);
			T dat = (T) atof(line.substr(begin, end - begin - 1).data());
			mat.updateEntry(i, j, dat);
			begin = end + 1;
			end = line.find_first_of(' ', begin);
		}
	}

	f.close();
	return mat;
}

template <class T>
void Matrix<T>::writeBinary(const char* Filename) const {
	using namespace std;
	using namespace useful;

	//Vorbereiten der Datei
	ofstream f;												//nur zum Schreiben von Daten
//	f.exceptions (ios::failbit | ios::badbit);
	f.open(Filename, ios::out | ios::binary | ios::trunc);	//Bin�rmodus f�r Ausgabe, alte Datei wird gel�scht
	if (!f) err("error: write: f.open()");

	//Speichern der Dimension
	f.write( (const char*) (&dim1_), sizeof(dim1_));
	f.write( (const char*) (&dim2_), sizeof(dim2_));

	//Speichern der Eintr�ge
	for (int i = 0; i < dim1_; i++){
		for (int j = 0; j < dim2_; j++){
			T Entry = this->getEntry(i, j);
			f.write( (const char*) (&Entry), sizeof(Entry) );
		}
	}
	f.close();
}

template<class T>
Matrix<T> Matrix<T>::readBinary(const char* Filename) {
	using namespace std;
	using namespace useful;

	ifstream f;
//	f.exceptions (ios::failbit | ios::badbit);
	f.open(Filename, ios::in | ios::binary);
	if (!f) err("error: readBinary: f.open()");

	//Einlesen der Dimensionen
	int dim1 = 0;
	f.read(reinterpret_cast<char *>(&dim1), sizeof(dim1) );
	//cout << dim1 << endl;

	int dim2 = 0;
	f.read(reinterpret_cast<char*>(&dim2), sizeof(dim2) );
	//cout << dim2 << endl;


	//Erstellen der Matrix
	Matrix<T> erg = Matrix<T>(dim1, dim2);

	//Einlesen der Eintr�ge
	for (int i = 0; i < erg.getDim1(); i++){
		for (int j = 0; j < erg.getDim2(); j++){
			T Entry;
			f.read(reinterpret_cast<char*> (&Entry), sizeof(T) );
			erg.updateEntry(i, j, Entry);
		}
	}

	return erg;
}


template <class T>
void Matrix<T>::writeR(const char* path) const {
	using namespace std;
	using namespace useful;

	ofstream f;
//	f.exceptions (ios::failbit | ios::badbit);
	f.open(path);
	if (!f) err("error: writeR: f.open()");
	f << scientific << setprecision(14);

	// Spalten bennenen
	for (int j = 0; j < dim2_; j++) {
		if (j != 0) f << " ";
		f << "\"x" << j << "\"";
	}

	// Werte schreiben
	for (int i = 0; i < dim1_; i++) {
		f << "\n";
		for (int j = 0; j < dim2_; j++) {
			if (j != 0) f << " ";
			f << getEntry(i,j);
		}
	}
	f << endl;
}

template<class T>
Matrix<T> Matrix<T>::readR(const char* path) {
	using namespace std;
	using namespace useful;

	ifstream f;
//	f.exceptions (ios::failbit | ios::badbit);
	f.open(path);
	if (!f) err("error: readR: f.open()");

	int dim1 = 0, dim2 = 0;
	string line, t;

	// Spaltenanzahl
	getline(f, line);
	istringstream iss(line);
	while (!iss.eof()) {
		dim2++;
		iss >> t;
	}

	// Zeilenanzahl
	getline(f, line);
	while (!f.eof()) {
		dim1++;
		getline(f, line);
	}

	// Daten lesen
	f.clear();
	f.seekg(0);
	getline(f, line);
	Matrix<T> mat(dim1, dim2);
	for (int i = 0; i < dim1; i++) {
		getline(f, line);
		istringstream iss(line);
		for (int j = 0; j < dim2; j++) {
			iss >> t;
			mat(i,j) = (T) atof(t.data());
		}
	}

	return mat;
}

template <class T>
void Matrix<T>::writeCSV(const char* path, char sep, char comma) const {
	using namespace std;
	using namespace useful;

	ofstream f(path);
	if (!f) err("error: writeCSV: f.open()");
	f << scientific << setprecision(14);

	stringstream ss;
	ss << scientific << setprecision(14);
	for (int i = 0; i < dim1_; i++) {
		for (int j = 0; j < dim2_; j++) {
			ss.str("");
			ss << getEntry(i,j);
			string s = ss.str();
			for (unsigned k = 0; k < s.size(); k++)
				s[k] = s[k] == '.' ? comma : s[k];
			if (j != 0)
				f << sep;
			f << s;
		}
		f << '\n';
	}
}

template<class T>
Matrix<T> Matrix<T>::readCSV(const char* path, char sep, char comma) {
	using namespace std;
	using namespace useful;

	ifstream f(path);
	if (!f) err("error: readCSV: f.open()");

	int dim1 = 0, dim2 = 1;
	string line, t;

	// Spaltenanzahl
	getline(f, line);
	for (unsigned i = 0; i < line.size(); i++)
		if (line[i] == sep) dim2++;

	// Zeilenanzahl
	while (!f.eof()) {
		dim1++;
		getline(f, line);
	}

	// Daten lesen
	f.clear();
	f.seekg(0);
	Matrix<T> mat(dim1, dim2);
	for (int i = 0; i < dim1; i++) {
		getline(f, line);
		stringstream ss;
		int j = 0;
		for (unsigned k = 0; k < line.size(); k++) {
			if (line[k] == sep) {
				mat(i, j++) = (T) atof(ss.str().data());
				ss.str("");
			} else {
				ss << (line[k] == comma ? '.' : line[k]);
			}
		}
	}

	return mat;
}

//template <class T>
//Matrix<T> Matrix<T>::initATA(int dim2){
//	Matrix<T> mat = Matrix <T> (dim2, dim2);	//dim2_, dim2_
//	mat.fill(0);
//	return mat;
//}
//
//template <class T>
//void Matrix<T>::updateATA(const T* newLine){
//	for (int i = 0; i < dim1_; i++){
//		for (int j = 0; j < dim2_; j++){
//			//Dyadisches Produkt aufaddieren
//			addEntry(i, j, newLine[i] * newLine[j] );
//		}
//	}
//}
//
//template <class T>
//Matrix<T> Matrix<T>::initATb(int dim2){
//	Matrix<T> erg = Matrix<T>(dim2, 1);
//	erg.fill(0);
//	return erg;
//}
//
//template <class T>
//void Matrix<T>::updateATb(const T* newLine, const T* b, int index){
//	for (int i = 0; i < dim1_; i++){
//		addEntry(i, 0, newLine[i] * b[index] );
//	}
//}

template <class T>
void Matrix<T>::print(std::ostream &strm, const char* prefix, int max_dim1, int max_dim2) const {
	using namespace std;
	strm << prefix << (prefix[0] == '\0' ? "" : ": ") << dim1_ << " x " << dim2_ << " :" << '\n';
	int d1 = dim1_ < max_dim1 ? dim1_ : max_dim1;
	int d2 = dim2_ < max_dim2 ? dim2_ : max_dim2;
	for (int i = 0; i < d1; i++) {
		strm << "  [";
		for (int j = 0; j < d2; j++) {
			if (j != 0) strm << ", ";
			strm << getEntry(i, j);
		}
		if (d2 < dim2_) strm << ", ...";
		strm << "]\n";
	}
	if (d1 < dim1_) strm << "  [...]\n";
	strm << flush;
}

template <class T>
void Matrix<T>::print(const char* prefix, int max_dim1, int max_dim2, std::ostream &strm) const {
	print(strm, prefix, max_dim1, max_dim2);
}

// ==============================================================
// === ITERATOR =================================================
// ==============================================================

template <class T>
Matrix<T>::matrix_iterator::matrix_iterator(Matrix<T>& mat, bool line_by_line, bool end)
	: line_by_line(line_by_line) {
	m = &mat;
	if (end) {
		if (line_by_line) {
			i = mat.dim1_;
			j = 0;
		} else {
			i = 0;
			j = mat.dim2_;
		}
	} else {
		i = 0;
		j = 0;
	}
}

template <class T>
Matrix<T>::matrix_iterator::matrix_iterator() {
	m = NULL;
	i = 0;
	j = 0;
	line_by_line = true;
}

template <class T>
Matrix<T>::matrix_iterator::matrix_iterator(const matrix_iterator& it) {
	m = it.m;
	i = it.i;
	j = it.j;
	line_by_line = it.line_by_line;
}

template <class T>
typename Matrix<T>::matrix_iterator& Matrix<T>::matrix_iterator::operator=(const matrix_iterator& it) {
	m = it.m;
	i = it.i;
	j = it.j;
	line_by_line = it.line_by_line;
	return *this;
}

template <class T>
typename Matrix<T>::matrix_iterator& Matrix<T>::matrix_iterator::operator++() {
	if (line_by_line) {
		j++;
		if (j >= m->dim2_) {
			j = 0;
			i++;
		}
	} else {
		i++;
		if (i >= m->dim1_) {
			i = 0;
			j++;
		}
	}
	return *this;
}

template <class T>
typename Matrix<T>::matrix_iterator& Matrix<T>::matrix_iterator::operator--() {
	if (line_by_line) {
		j--;
		if (j < 0) {
			j = m->dim2_ - 1;
			i--;
		}
	} else {
		i--;
		if (i < 0) {
			i = m->dim1_ - 1;
			j--;
		}
	}
	return *this;
}

template <class T>
typename Matrix<T>::matrix_iterator Matrix<T>::matrix_iterator::operator++(int) {
	matrix_iterator it(*this);
	if (line_by_line) {
		j++;
		if (j >= m->dim2_) {
			j = 0;
			i++;
		}
	} else {
		i++;
		if (i >= m->dim1_) {
			i = 0;
			j++;
		}
	}
	return it;
}

template <class T>
typename Matrix<T>::matrix_iterator Matrix<T>::matrix_iterator::operator--(int) {
	matrix_iterator it(*this);
	if (line_by_line) {
		j--;
		if (j < 0) {
			j = m->dim2_ - 1;
			i--;
		}
	} else {
		i--;
		if (i < 0) {
			i = m->dim1_ - 1;
			j--;
		}
	}
	return it;
}

template <class T>
typename Matrix<T>::matrix_iterator& Matrix<T>::matrix_iterator::operator+=(diff_type x) {
	if (line_by_line) {
		ptrdiff_t d = m->dim2_;
		ptrdiff_t t = i * d + j + x;
		i = t / d;
		j = t % d;
	} else {
		ptrdiff_t d = m->dim1_;
		ptrdiff_t t = i + j * d + x;
		i = t % d;
		j = t / d;
	}
	return *this;
}

template <class T>
typename Matrix<T>::matrix_iterator& Matrix<T>::matrix_iterator::operator-=(diff_type x) {
	if (line_by_line) {
		ptrdiff_t d = m->dim2_;
		ptrdiff_t t = i * d + j - x;
		i = t / d;
		j = t % d;
	} else {
		ptrdiff_t d = m->dim1_;
		ptrdiff_t t = i + j * d - x;
		i = t % d;
		j = t / d;
	}
	return *this;
}

template <class T>
typename Matrix<T>::matrix_iterator Matrix<T>::matrix_iterator::operator+(diff_type x) const {
	matrix_iterator it;
	it.m = m;
	it.line_by_line = line_by_line;
	if (line_by_line) {
		ptrdiff_t d = m->dim2_;
		ptrdiff_t t = i * d + j + x;
		it.i = t / d;
		it.j = t % d;
	} else {
		ptrdiff_t d = m->dim1_;
		ptrdiff_t t = i + j * d + x;
		it.i = t % d;
		it.j = t / d;
	}
	return it;
}

template <class T>
typename Matrix<T>::matrix_iterator Matrix<T>::matrix_iterator::operator-(diff_type x) const {
	matrix_iterator it;
	it.m = m;
	it.line_by_line = line_by_line;
	if (line_by_line) {
		ptrdiff_t d = m->dim2_;
		ptrdiff_t t = i * d + j - x;
		it.i = t / d;
		it.j = t % d;
	} else {
		ptrdiff_t d = m->dim1_;
		ptrdiff_t t = i + j * d - x;
		it.i = t % d;
		it.j = t / d;
	}
	return it;
}

template <class T>
typename Matrix<T>::matrix_iterator::diff_type Matrix<T>::matrix_iterator::operator-(const matrix_iterator& it) const {
	if (line_by_line)
		return i * (ptrdiff_t) m->dim2_ + j - it.i * (ptrdiff_t) it.m->dim2_ - it.j;
	else
		return i + j * (ptrdiff_t) m->dim1_ - it.i - it.j * (ptrdiff_t) it.m->dim1_ ;
}

template <class T>
typename Matrix<T>::matrix_iterator::reference Matrix<T>::matrix_iterator::operator*() const {
	return (*m)(i,j);
}

template <class T>
typename Matrix<T>::matrix_iterator::pointer Matrix<T>::matrix_iterator::operator->() const {
	return &((*m)(i,j));
}

template <class T>
typename Matrix<T>::matrix_iterator::reference Matrix<T>::matrix_iterator::operator[](diff_type x) const {
	return *((*this)+x);
}


template <class T>
bool Matrix<T>::matrix_iterator::operator==(const matrix_iterator& it) const {
	return i == it.i && j == it.j && m == it.m && line_by_line == it.line_by_line;
}
template <class T>
bool Matrix<T>::matrix_iterator::operator!=(const matrix_iterator& it) const {
	return i != it.i || j != it.j || m != it.m || line_by_line != it.line_by_line;
}
template <class T>
bool Matrix<T>::matrix_iterator::operator<(const matrix_iterator& it) const {
	return (*this - it) < 0;
}
template <class T>
bool Matrix<T>::matrix_iterator::operator>(const matrix_iterator& it) const {
	return (*this - it) > 0;
}
template <class T>
bool Matrix<T>::matrix_iterator::operator<=(const matrix_iterator& it) const {
	return (*this - it) <= 0;
}
template <class T>
bool Matrix<T>::matrix_iterator::operator>=(const matrix_iterator& it) const {
	return (*this - it) >= 0;
}

template <class T>
typename Matrix<T>::matrix_iterator Matrix<T>::begin() {
	return matrix_iterator(*this, matrix_iterator::LINE_BY_LINE, matrix_iterator::BEGIN);
}

template <class T>
typename Matrix<T>::matrix_iterator Matrix<T>::end() {
	return matrix_iterator(*this, matrix_iterator::LINE_BY_LINE, matrix_iterator::END);
}

template <class T>
typename Matrix<T>::matrix_iterator Matrix<T>::begin_cbc() {
	return matrix_iterator(*this, matrix_iterator::COLUMN_BY_COLUMN, matrix_iterator::BEGIN);
}

template <class T>
typename Matrix<T>::matrix_iterator Matrix<T>::end_cbc() {
	return matrix_iterator(*this, matrix_iterator::COLUMN_BY_COLUMN, matrix_iterator::END);
}

// ==============================================================
// === Template-Methoden ========================================
// ==============================================================

template <class T>
template <class F>
void Matrix<T>::transform(F func) {
	for (int i = 0; i < dim1_; i++)
		for (int j = 0; j < dim2_; j++)
			updateEntry(i, j, func(getEntry(i, j)));
}

template <class T>
template <class F>
Matrix<T> Matrix<T>::map(F func) const {
	Matrix<T> erg(dim1_, dim2_);
	for (int i = 0; i < dim1_; i++)
		for (int j = 0; j < dim2_; j++)
			erg(i, j) = func(getEntry(i, j));
	return erg;
}

template <class T>
template <class F>
T Matrix<T>::fold(F func, T erg) const {
	for (int i = 0; i < dim1_; i++)
		for (int j = 0; j < dim2_; j++)
			erg = func(erg, getEntry(i,j));
	return erg;
}

template <class T>
template <class F>
void Matrix<T>::transform(const Matrix<T>& m, F func) {
	if (m.dim1_ != dim1_ || m.dim2_ != dim2_)
		throw dim_exception("compute: dim");
	for (int i = 0; i < dim1_; i++)
		for (int j = 0; j < dim2_; j++)
			updateEntry(i, j, func(getEntry(i,j), m.getEntry(i,j)));
}

template <class T>
template <class F>
Matrix<T> Matrix<T>::map(const Matrix<T>& m, F func) const {
	if (m.dim1_ != dim1_ || m.dim2_ != dim2_)
		throw dim_exception("compute: dim");
	Matrix<T> erg(dim1_, dim2_);
	for (int i = 0; i < dim1_; i++)
		for (int j = 0; j < dim2_; j++)
			erg(i, j) = func(getEntry(i,j), m.getEntry(i,j));
	return erg;
}

// ==============================================================
// === Operatoren ===============================================
// ==============================================================

// === Ausgabe ===
template <class T>
std::ostream& operator<<(std::ostream &strm, const Matrix<T> &m) {
	strm << "" << m.getDim1() << " x " << m.getDim2() << " :\n";
	for (int i = 0; i < m.getDim1(); i++) {
		strm << "  [";
		for (int j = 0; j < m.getDim2(); j++) {
			if (j != 0) strm << ", ";
			strm << m.getEntry(i, j);
		}
		strm << "]\n";
	}
	strm << std::flush;
	return strm;
}

// === Vergleich ===
template <class T>
bool operator==(const Matrix<T> &a, const Matrix<T> &b) {
	if (a.getDim1() != b.getDim1() || a.getDim2() != b.getDim2()) return false;
	for (int i = 0; i < a.getDim1(); i++)
		for (int j = 0; j < a.getDim2(); j++) {
			if (a.getEntry(i,j) != b.getEntry(i,j)) return false;
		}
	return true;
}

template <class T>
bool operator!=(const Matrix<T> &a, const Matrix<T> &b) {
	return !(a == b);
}

// === Matrix-Matrix-Operationen ===
template <class T>
Matrix<T> operator+(const Matrix<T> &a, const Matrix<T> &b) {
	return a.addMatrix(b, (a.getDim1() > b.getDim1() ? a.getDim1() : b.getDim1()), (a.getDim2() > b.getDim2() ? a.getDim2() : b.getDim2()), 0, 0, 0, 0);
}

template <class T>
Matrix<T> operator-(const Matrix<T> &a, const Matrix<T> &b) {
	return a.subMatrix(b, a.getDim1() > b.getDim1() ? a.getDim1() : b.getDim1(), a.getDim2() > b.getDim2() ? a.getDim2() : b.getDim2(), 0, 0, 0, 0);
}

template <class T>
Matrix<T> operator*(const Matrix<T> &a, const Matrix<T> &b) {
	return a.multiplyMix(&b);
}

template <class T>
Matrix<T> & operator+=(Matrix <T> &a, const Matrix<T> &b) {
	a.addMatrix(b);
	return a;
}

template <class T>
Matrix<T> & operator-=(Matrix <T> &a, const Matrix<T> &b) {
	a.subMatrix(b);
	return a;
}

template <class T>
Matrix<T> & operator*=(Matrix<T> &a, const Matrix<T> &b) {
	return a = a.multiplyMix(&b);
}

template <class T>
const Matrix<T> operator~(const Matrix<T> &a) {
	return a.getTranspose();
}

template <class T>
Matrix<T> operator!(const Matrix<T> &a) {
	return a.pseudo_inv();
}

// === Matrix-Vector-Operationen ===
template <class T>
Matrix<T> operator+(const Matrix<T> &a, const std::vector<T> &v) {
	if (a.getDim1() == 1) return a + Matrix<T>::fromVectorRow(v);
	if (a.getDim2() == 1) return a + Matrix<T>::fromVectorColumn(v);
    throw typename Matrix<T>::dim_exception("operator+: matrix is not a vector");
}

template <class T>
Matrix<T> operator-(const Matrix<T> &a, const std::vector<T> &v) {
	if (a.getDim1() == 1) return a - Matrix<T>::fromVectorRow(v);
	if (a.getDim2() == 1) return a - Matrix<T>::fromVectorColumn(v);
    throw typename Matrix<T>::dim_exception("operator-: matrix is not a vector");
}

template <class T>
Matrix<T> operator*(const Matrix<T> &a, const std::vector<T> &v) {
	Matrix <T> b;
	if (a.getDim2() == 1) b = Matrix<T>::fromVectorRow(v);
    else if ((unsigned) a.getDim2() != v.size()) throw typename Matrix<T>::dim_exception("operator*: dim");
	else b = Matrix<T>::fromVectorColumn(v);
	return a.multiplyMix(&b);
}


// === Matrix-Element-Operationen ===
template <class T>
Matrix<T> operator+(const Matrix<T> &mat, T a) {
	Matrix<T> erg = Matrix<T>(mat.getDim1(), mat.getDim2());
	for (int i = 0; i < erg.getDim1(); i++){
		for (int j = 0; j < erg.getDim2(); j++){
			erg(i, j) = mat(i, j) + a;
		}
	}
	return erg;
}

template <class T>
Matrix<T> operator-(const Matrix<T> &mat, T a) {
	Matrix<T> erg = Matrix<T>(mat.getDim1(), mat.getDim2());
	for (int i = 0; i < erg.getDim1(); i++){
		for (int j = 0; j < erg.getDim2(); j++){
			erg(i, j) = mat(i, j) - a;
		}
	}
	return erg;
}

template <class T>
Matrix<T> operator*(const Matrix<T> &mat, T a) {
	Matrix<T> erg = Matrix<T>(mat.getDim1(), mat.getDim2());
	for (int i = 0; i < erg.getDim1(); i++){
		for (int j = 0; j < erg.getDim2(); j++){
			erg(i, j) = mat(i, j) * a;
		}
	}
	return erg;
}

template <class T>
Matrix<T> operator/(const Matrix<T> &mat, T a) {
	Matrix<T> erg = Matrix<T>(mat.getDim1(), mat.getDim2());
	for (int i = 0; i < erg.getDim1(); i++){
		for (int j = 0; j < erg.getDim2(); j++){
			erg(i, j) = mat(i, j) / a;
		}
	}
	return erg;
}

template <class T>
Matrix<T> & operator+=(Matrix<T> &mat, T a) {
	mat.addtoAll(a);
	return mat;
}

template <class T>
Matrix<T> & operator-=(Matrix<T> &mat, T a) {
	mat.addtoAll(-a);
	return mat;
}

template <class T>
Matrix<T> & operator*=(Matrix<T> &mat, T a) {
	mat.multiplytoAll(a);
	return mat;
}

template <class T>
Matrix<T> & operator/=(Matrix<T> &mat, T a) {
	for (int i = 0; i < mat.getDim1(); i++){
		for (int j = 0; j < mat.getDim2(); j++){
			mat(i, j) /= a;
		}
	}
	return mat;
}

//template <class T>
//Matrix<T> operator<<(const Matrix<T>& mat, T (*func)(T)) {
//	return mat.map(func);
//}
//
//template <class T, class F>
//Matrix<T> operator<<(const Matrix<T>& mat, F func) {
//	return mat.map(func);
//}

template <class T>
Matrix<T> & operator<<(Matrix<T>& mat, T (*func)(T)) {
	mat.transform(func);
	return mat;
}

template <class T, class F>
Matrix<T> & operator<<(Matrix<T>& mat, F func) {
	mat.transform(func);
	return mat;
}




