/** @file BCH.h
 *  @brief Random bit sequence generator using the BCH scheme
 *  @author Jens Quedenfeld
 */

#ifndef BCH_H_
#define BCH_H_

#include <vector>

/** @brief Random bit generator
 *
 *	@details
 *  	This class creates random bit sequences with a specified independence.
 *  	The elements of the sequence can be access in an arbitrary order.
 *  	This class implements the BCH scheme. For further explanations, see
 *  	<em> Pseudo-Random Number Generation for Sketch-Based Estimations </em>
 * 		by FLORIN RUSU and ALIN DOBRA. */
class BCH {
public:
	/** @brief type definition for unsigned long long */
	typedef unsigned long long ULL;

	/** @brief Standard constructor (with a specified independence)
	 *  @param deg
	 *  	degree of independence. Default value is 4.
	 *  @details
	 *  	The created random bit generator will create a sequences with a degree of indepedence of \a deg.
	 *  @see BCH(std::vector<ULL>, int) */
	BCH(int deg = 4);

	/** @brief Constructor with a specified seed
	 *  @param seed
	 *  	Seed vector.
	 *  @param s0
	 *  	Extra seed bit. If the value \a EVEN_DEGREE is passed, then no extra bit is used.
	 *  @details
	 *  	The created random bit sequence has an independence of <em> (2k + 1) </em>, where \a k is the number of elements stored in \a seed.
	 *  	If \a s0 is equal to EVEN_DEGREE, then the independence is <em> 2k </em>.
	 *  @see BCH(int), EVEN_DEGREE */
	BCH(std::vector<ULL> seed, int s0 = EVEN_DEGREE);

	/** @brief Destructor
	 *  @details This destructor contains no explicit code. */
	virtual ~BCH();

	/** @brief Returns the degree of this generator
	 *  @returns the degree of independence. */
	int getDegree() const;

	/** @brief Returns the used seed.
	 *  @returns a copy of the used seed vector */
	std::vector<ULL> getSeed() const;

	/** @brief Returns the extra bit.
	 *  @returns the value of the extra bit or the value EVEN_DEGREE if no extra bit is used.
	 *  @details The extra bit increases the degree of independence by one. */
	int get_s0() const;

	/** @brief Sequence access (0 or 1)
	 *  @param idx
	 *  	index of the accessed element
	 *  @returns
	 *  	the \a idx-th element of the random bit sequence. The returned value is either equal zero or one.
	 *  @details
	 *  	This function is the fastest way to access the sequence's elements
	 *  @see {@link operator[](ULL)const operator[]} */
    inline
	int getValue_01(ULL idx) const;

	/** @brief Sequence access (-1 or +1)
	 *  @param idx
	 *  	index of the accessed element
	 *  @returns
	 *  	the \a idx-th element of the random bit sequence. The returned value is either equal minus or plus one.
	 *  @details
	 *  	This function is a bit slower than getValue_01. It returns the value <em> 2 * getValue_01(idx) - 1 </em>
	 *  @see getValue_01 */
    inline
	int operator[](ULL idx) const;

	/** @brief Compares this random bit generator to another
	 *  @param bch
	 *  	Another BCH generator.
	 *  @returns true if and only if the degree of independence, the value of the extra bit and the seed vector of \a bch and this generator are equal.
	 */
	bool operator==(BCH& bch) const;

	/** @brief Constant denoting that the extra bit is not used.
	 *  @see BCH(std::vector<ULL>, int), get_s0 */
	const static int EVEN_DEGREE = -1;


private:
	/** @brief Extra bit */
	int s0_;

	/** @brief Degree of independence */
	int deg_;

	/** @brief Seed vector */
	std::vector<ULL> seed_;
};

int BCH::getValue_01(ULL idx) const {
    const int size = seed_.size();
    const ULL domain = (1LL << 32) - 1;

    ULL help = idx;
    int erg = s0_;
    const ULL *seed = &seed_[0];
    for (int i = 0; i < size; i++){
        ULL x = seed[i] & help;
#ifdef __GNUC__
        // Fast version for GCC/clang.
        erg = erg ^ __builtin_parityll(x);
#else
//        // Generic implementation taken from "http://en.wikipedia.org/wiki/Hamming_weight"
//        x -= (x >> 1) & 0x5555555555555555;                             //put count of each 2 bits into those 2 bits
//        x = (x & 0x3333333333333333) + ((x >> 2) & 0x3333333333333333); //put count of each 4 bits into those 4 bits
//        x = (x + (x >> 4)) & 0x0f0f0f0f0f0f0f0f;                        //put count of each 8 bits into those 8 bits
//        x = (x * 0x0101010101010101)>>56;                            //returns left 8 bits of x + (x<<8) + (x<<16) + ...
//        erg = erg ^ x;

        while (t > 0){
            erg = erg ^ 1;
            t = t & (t-1);
        }
#endif
        help = (((help * idx) & domain) * idx) & domain;
    }
    return erg;
}

int BCH::operator[](ULL idx) const {
    return getValue_01(idx) * 2 - 1;
}

#endif /* BCH_H_ */
