/** @file BCH_Null.h
 *  @brief Simple embeddings using Rademacher matrices with zeros
 *  @author Jens Quedenfeld
 */

#ifndef BCH_NULL_H_
#define BCH_NULL_H_

#include "mat/matrix.h"
#include "sketch/BCH.h"


/** @brief Simple embeddings using Rademacher matrices with zeros
 *
 *  @details This class can embed matrices to reduce its first dimension.
 *  	For this purpose a random Rademacher matrix (containing +1 and 0 and -1 entries) is multiplied with the input matrix.
 *  	The difference to BCH_Sketch are the zero entry of the Rademacher matrix, thus it is maybe faster.
 *  @see BCH_Sketch, BCH_Small, SRHT, CW, KN */
template <class T>
class BCH_Null {
public:
	/** @brief 	The first template parameter (T) */
	typedef T value_type;

	/** @brief Constructor with a specified target dimension
	 *  @param dim_reduced
	 *  	The new first dimension of the input data.
	 *  @param dim_data
	 *  	The second dimension of the input data.
	 *  @param deg
	 *  	Degree of independence for the BCH generator.
	 *  @details The input data is an (n x dim_data) matrix and it will be reduced to a (dim_reduced x dim_data) matrix. */
	BCH_Null(int dim_reduced, int dim_data, int deg = 12);

	/** @brief Constructor with a specified epsilon
	 *  @param eps
	 *  	The quality of the embedding.
	 *  @param dim_data
	 *  	The second dimension of the input data.
	 *  @param deg
	 *  	Degree of independence for the BCH generator.
	 *  @details The input data is an (n x dim_data) matrix and it will be reduced to a (k x dim_data) matrix.
	 *  	The variable \a k is calculated by the function calcDim_Reduced. The embedding will be an (1 + \a eps) approximation of the input matrix.
	 *  @see BCH_Null(int, int), calcDim_Reduced */
	BCH_Null(double eps, int dim_data, int deg = 12);

	/** @brief Calculates the necessary number of rows for the embedding
	 *  @details If you have a linear equation system and you want to reduce the number of equations, you can
	 *  	use this function to calculate the number of equations of the embedded matrix.
	 *  @param eps
	 *  	The quality of the embedding.
	 *  @param dim_data
	 *  	The second dimension of the input data.
	 *  @returns the necessary number of rows of the embedded matrix for an (1 + \a eps) approximation. */
	static int calcDim_Reduced(double eps, int dim_data);

	/** @brief Destructor
	 *  @details This destructor contains no explicit code. */
	virtual ~BCH_Null();

	/** @brief Resets the embedded matrix to a zero matrix
	 * 	@details If you want to embed a new input matrix with the same Rademacher matrix
	 * 		you can use this method to clear the embedded matrix. */
	void reset();

	/** @brief Embeds one or several rows.
	 *  @param mat
	 *  	A matrix containing one or more rows of the input matrix.
	 *  	The second dimension of this matrix must be equal to the dimension \a dim_data passed, when BCH_Null was constructed.
	 *  @details This method embeds the next lines of an input matrix.
	 *  @see addLine, getSketch_normalized */
	void addLine(const Matrix<T>& mat);

	/** @brief Embeds a whole matrix
	 *  @param mat
	 *  	The matrix that will be embedded.
	 *  @details Calling this function has the same effect as calling {@link addLine addLine}.
	 *  @see addLine, getSketch_normalized */
	void updateAll(const Matrix<T>& mat);

	/** @brief Returns a reference to the not normalized embedded matrix.
	 *  @returns a reference to the not normalized embedded matrix.
	 *  @see getSketch_normalized */
	const Matrix<T>& getSketch() const;

	/** @brief Returns the normalized embedded matrix.
	 *  @returns the normalized embedded matrix.
	 *  @details Use this method to get the embedded matrix.
	 *  @see getSketch */
	Matrix<T> getSketch_normalized() const;

private:
	/** @brief Calculates the next random bit
	 *  @returns the next entry of the Rademacher matrix (-1 or 0 or +1) */
	T getNext();

	/** @brief BCH generator used for the Rademacher matrix */
	BCH bch_;

	/** @brief Embedded matrix */
	Matrix<T> sketch_;

	/** @brief Position counter for the BCH generator */
	long long pos_;
};

// ===========================
// ===== IMPLEMENTIERUNG =====
// ===========================

template <class T>
BCH_Null<T>::BCH_Null(int dim_reduced, int dim_data, int deg) :
	bch_(deg),
	sketch_(dim_reduced, dim_data) {
	sketch_.fill(0);
	pos_ = 0;
}

template <class T>
BCH_Null<T>::BCH_Null(double eps, int dim_data, int deg) :
	BCH_Null(calcDim_Reduced(eps, dim_data), dim_data, deg) {

}

template <class T>
int BCH_Null<T>::calcDim_Reduced(double eps, int dim_data) {
    return ceil(dim_data * log(dim_data) / (eps * eps));
}

template <class T>
BCH_Null<T>::~BCH_Null() {

}

template <class T>
void BCH_Null<T>::reset() {
	sketch_.fill(0);
	pos_ = 0;
}

template <class T>
void BCH_Null<T>::addLine(const Matrix<T>& mat) {
	if (mat.getDim2() != sketch_.getDim2())
		throw typename Matrix<T>::dim_exception("BCH_Null: addLine: Dimension nicht passend");

	// for each line
	for (int i = 0; i < mat.getDim1(); i++) {
		// for each line in skizze
		for (int k = 0; k < sketch_.getDim1(); k++) {
			T s = getNext();
			if (s != 0) {
				// for each element in skizze
				for (int j = 0; j < sketch_.getDim2(); j++) {
					sketch_(k, j) += s * mat(i, j);
				}
			}
		}
	}
}

template <class T>
void updateAll(const Matrix<T>& mat) {
	addLine(mat);
}

template <class T>
const Matrix<T>& BCH_Null<T>::getSketch() const {
	return (const Matrix<T>&) sketch_;
}

template <class T>
Matrix<T> BCH_Null<T>::getSketch_normalized() const {
	double t = sqrt(3.0 / sketch_.getDim1());
	Matrix<T> erg(sketch_.getDim1(), sketch_.getDim2());
	for (int i = 0; i < erg.getDim1(); i++)
		for (int j = 0; j < erg.getDim2(); j++)
			erg(i,j) = sketch_(i,j) * t;
	return erg;
}

template <class T>
T BCH_Null<T>::getNext() {
	while (true) {
		if (bch_.getValue_01(pos_++) == 0)
			return 0;
		if (bch_.getValue_01(pos_++) == 1)
			return (T) (2 * bch_.getValue_01(pos_++) - 1);
	}
}



#endif /* BCH_NULL_H_ */
