/** @file BCH_Small.h
 *  @brief Simple embeddings using Rademacher matrices with better reduction
 *  @author Jens Quedenfeld
 */

#ifndef BCH_SMALL_H_
#define BCH_SMALL_H_

#include "mat/matrix.h"

/** @brief Simple embeddings using Rademacher matrices with better reduction
 *
 *  @details This class can embed matrices to reduce its first dimension.
 *  	For this purpose a random Rademacher matrix (containing +1 and -1 entries) is multiplied with the input matrix.#
 *  	This class uses higher independence than BCH_Sketch and therefore the first dimension is reduced to a lower value.
 *  @see BCH_Sketch, BCH_Null, SRHT, CW, KN */
template <class T>
class BCH_Small {
public:
	/** @brief 	The first template parameter (T) */
	typedef T value_type;

	/** @brief Constructor with a specified target dimension
	 *  @param dim_reduced
	 *  	The new first dimension of the input data.
	 *  @param dim_data
	 *  	The second dimension of the input data.
	 *  @details The input data is an (n x dim_data) matrix and it will be reduced to a (dim_reduced x dim_data) matrix.
	 *  @see BCH_Small(double, int) */
	BCH_Small(int dim_reduced, int dim_data);

	/** @brief Constructor with a specified epsilon
	 *  @param eps
	 *  	The quality of the embedding.
	 *  @param dim_data
	 *  	The second dimension of the input data.
	 *  @details The input data is an (n x dim_data) matrix and it will be reduced to a (k x dim_data) matrix.
	 *  	The variable \a k is calculated by the function calcDim_Reduced. The embedding will be an (1 + \a eps) approximation of the input matrix.
	 *  @see BCH_Small(int, int), calcDim_Reduced */
	BCH_Small(double eps, int dim_data);

	/** @brief Destructor
	 *  @details This destructor contains no explicit code. */
	virtual ~BCH_Small();

	/** @brief Calculates the necessary number of rows for the embedding
	 *  @details If you have a linear equation system and you want to reduce the number of equations, you can
	 *  	use this function to calculate the number of equations of the embedded matrix.
	 *  @param eps
	 *  	The quality of the embedding.
	 *  @param dim_data
	 *  	The second dimension of the input data.
	 *  @returns the necessary number of rows of the embedded matrix for an (1 + \a eps) approximation. */
	static int calcDim_Reduced(double eps,int dim_data);

	/** @brief Direct access to the Rademacher matrix.
	 *  @param x
	 *  	Index of the first dimension.
	 *  @param y
	 *  	Index of the second dimension.
	 *  @returns the entry (x,y) of the Rademacher matrix.
	 *  @details The Rademacher matrix is not stored explicitly. The entry are calculated with a BCH generator.
	 *  @see getEmbeddingMatrix, updateEntry */
	T operator()(int x, int y) const;

	/** @brief Embeds a single entry.
	 *  @param aij
	 *  	Value at the position (i,j)
	 *  @param i
	 *  	First index
	 *  @param j
	 *  	Second index
	 *  @details This method embeds a single entry of an input matrix.
	 *  @see updateRows, updateColumns, updateMatrix, updateAll, {@link operator()(int, int)const operator()} */
	void updateEntry(T aij, int i, int j);

	/** @brief Embeds one or several rows.
	 *  @param rows
	 *  	A matrix containing one or more rows of the input data.
	 *  @param startRow
	 *  	Row number of the first row of the passed matrix in the input matrix.
	 *  @details This method embeds one or several rows of an input matrix.
	 *  	To produce correct results you have to specify the row offset
	 *  	such that <em> rows(i, j) = input(startRow + i, j) </em>.
	 *  	This method is faster than updateEntry and updateColumns:
	 *  @attention The Rademacher matrix is calculated and stored explicitly for this operation.
	 *  	Therefore you should not embed many rows with one call of this method, because storing
	 *  	the Rademacher matrix explicitly costs much memory. Use updateMatrix, if you want to embed a whole matrix.
	 *  @see updateEntry, updateColumns, updateMatrix, updateAll */
	void updateRows(const Matrix<T>& rows, int startRow = 0);

	/** @brief Embeds one or several columns.
	 *  @param columns
	 *  	A matrix containing one or more columns of the input data.
	 *  @param startColumn
	 *  	Column number of the first column of the passed matrix in the input matrix.
	 *  @details This method embeds one or several columns of an input matrix.
	 *  	To produce correct results you have to specify the column offset
	 *  	such that <em> columns(i, j) = input(i, startColumn + j) </em>.
	 *  @see updateEntry, updateColumns, updateRows, updateMatrix  */
	void updateColumns(const Matrix<T>& columns, int startColumn = 0);


	/** @brief Embeds a whole matrix
	 *  @param mat
	 *  	The matrix that will be embedded.
	 *  @param off1
	 *  	Offset of the first dimension
	 *  @param off2
	 *  	Offset of the second dimension
	 *  @details This method embeds a matrix. If the matrix is not equal to
	 *  	the input matrix, then you have to specify the offsets, such that
	 *  	<em> mat(i, j) = input(i + off1, j + off2) </em>
	 *  @see updateEntry, updateRows, updateColumns, updateAll */
	void updateMatrix(const Matrix<T>& mat, int off1 = 0, int off2 = 0);

		/** @brief Embeds a whole matrix
	 *  @param mat
	 *  	The matrix that will be embedded.
	 *  @param buffer
	 *  	Number of columns of the Rademacher matrix stored explicitly
	 *  @details This method embeds a whole input matrix.
	 *  	The function works very fast, because it stores \a buffer columns of the Rademacher matrix explicitly.
	 *  	The matrix multiplications of the buffered Rademacher matrices with \a buffer rows of the input matrix
	 *  	is calculated by the Strassen algorithm. Therefore this method is very fast, however it
	 *  	requires more memory than updateEntry ord updateColumns.
	 *  @see updateEntry, updateRows, updateColumns, updateMatrix  */
	void updateAll(const Matrix<T>& mat, int buffer = 1000);

	/** @brief Resets the embedded matrix to a zero matrix
	 * 	@details If you want to embed a new input matrix with the same Rademacher matrix
	 * 		you can use this method to clear the embedded matrix. */
	void reset();

	/** @brief Returns a reference to the not normalized embedded matrix.
	 *  @returns a reference to the not normalized embedded matrix.
	 *  @see getSketch_normalized */
	const Matrix<T>& getSketch() const;

	/** @brief Returns the normalized embedded matrix.
	 *  @returns the normalized embedded matrix.
	 *  @details Use this method to get the embedded matrix.
	 *  @see getSketch */
	Matrix<T> getSketch_normalized() const;

	/** @brief Returns an explicit stored copy of the Rademacher matrix
	 *  @param dim_original
	 *  	The number of rows of the input matrix. This number is equal to the number of columns of the returned matrix.
	 *  @returns the Rademacher matrix.
	 *  @details The Rademacher matrix is usually very big and therefore not stored explicitly.
	 *  	Normally, this method is not used.
	 *  @see {@link operator()(int, int)const operator()} */
	Matrix<T> getEmbeddingMatrix(int dim_original) const;

private:
	/** @brief Reduced dimension
	 *  @returns the reduced dimension */
	int getDim_reduced() const;

	/** @brief Data dimension
	 *  @returns the second dimension of the input matrix respectively the embedded matrix */
	int getDim_data() const;

	/** @brief Seed initialization
	 *  @details This method initializes the BCH generators */
	void initSeeds();

	/** @brief BCH generators used for the Rademacher matrix */
	std::vector<BCH> bch_;

	/** @brief Embedded matrix */
	Matrix<T> sketch_;
};

template <class T>
BCH_Small<T>::BCH_Small(int dim_reduced, int dim_data) :
	bch_(dim_data+1), // +1 (lieber so als möglicherweise seg-fault...
	sketch_(dim_reduced, dim_data) {
	reset();
	initSeeds();
}

template <class T>
BCH_Small<T>::BCH_Small(double eps, int dim_data) :
	BCH_Small(calcDim_Reduced(eps, dim_data), dim_data) {

}

template <class T>
BCH_Small<T>::~BCH_Small() {

}

template <class T>
int BCH_Small<T>::calcDim_Reduced(double eps, int dim_data) {
	return dim_data * (int) ceil(1 / eps);
}

template <class T>
T BCH_Small<T>::operator()(int x, int y) const {
	typedef unsigned long long ULL;
	const int epsinv = sketch_.getDim1() / sketch_.getDim2();
	int outer_idx = x / epsinv;
	int inner_idx = x % epsinv;

	return (T) bch_[outer_idx][y * (ULL) getDim_reduced() + inner_idx];
}

template <class T>
void BCH_Small<T>::updateEntry(T aij, int i, int j) {
	for (int x = 0; x < sketch_.getDim1(); x++) {
		sketch_.addEntry(x, j, (*this)(x,i) * aij);
	}
}

template <class T>
void BCH_Small<T>::updateRows(const Matrix<T>& mat, int startRow) {
	updateMatrix(mat, startRow, 0);
}

template <class T>
void BCH_Small<T>::updateColumns(const Matrix<T>& mat, int startColumn) {
	for (int i = 0; i < mat.getDim1(); i++)
		for (int j = 0; j < mat.getDim2(); j++)
			updateEntry(mat(i,j), i, j + startColumn);
}

template <class T>
void BCH_Small<T>::updateMatrix(const Matrix<T>& mat, int off1, int off2) {
	int dim1 = mat.getDim1();
	int dim2 = mat.getDim2();
	int dim_reduced = getDim_reduced();
	Matrix<double> partS(dim_reduced, dim1);
	for (int i = 0; i < dim_reduced; i++)
		for (int j = 0; j < dim1; j++)
			partS.updateEntry(i, j, (*this)(i, off1 + j));
	Matrix<double> erg = partS * mat;
	for (int i = 0; i < dim_reduced; i++)
		for (int j = 0; j < dim2; j++)
			sketch_(i, j + off2) += erg(i, j);
}

template <class T>
void BCH_Small<T>::updateAll(const Matrix<T>& mat, int buffer) {
	int current_row = 0;
	int rows = mat.getDim1();
	while (current_row < rows) {
		updateMatrix(mat.select(current_row, 0, std::min(buffer, rows - current_row), mat.getDim2()), current_row, 0);
		current_row += buffer;
	}

}

template <class T>
void BCH_Small<T>::reset() {
	sketch_.fill(0);
}

template <class T>
const Matrix<T>& BCH_Small<T>::getSketch() const {
	return sketch_;
}

template <class T>
Matrix<T> BCH_Small<T>::getSketch_normalized() const {
	return sketch_ / sqrt((T) getDim_reduced());
}

template <class T>
Matrix<T> BCH_Small<T>::getEmbeddingMatrix(int dim_original) const {
	Matrix<T> mat(getDim_reduced(), dim_original);
	for (int i = 0; i < mat.getDim1(); i++) {
		for (int j = 0; j < mat.getDim2(); j++) {
			mat(i,j) = (*this)(i,j);
		}
	}
	return mat;
}

template <class T>
int BCH_Small<T>::getDim_reduced() const {
	return sketch_.getDim1();
}

template <class T>
int BCH_Small<T>::getDim_data() const {
	return sketch_.getDim1();
}

template <class T>
void BCH_Small<T>::initSeeds() {
	std::mt19937_64 random;
	random.seed(time(0));
	std::vector<unsigned long long> seed(2);
	for (unsigned i = 0; i < bch_.size(); i++) {
		seed[0] = random();
		seed[1] = random();
		bch_[i] = BCH(seed);
	}
}



#endif /* BCH_SMALL_H_ */
