/** @file CW.h
 *  @brief Very fast embedding method by Clarkson and Woodruff
 *  @author Jens Quedenfeld
 */

#ifndef CW_H_
#define CW_H_

#include "mat/matrix.h"
#include "sketch/BCH.h"

/** @brief Very fast embeddings
 *  @details This class implements the embedding from the publication
 *     <em> "Low Rank Approximation and Regression Input Sparsity Time" </em>
 *     by David P. Woodruff und Clarkson. The embedding is very fast, however,
 *     the necessary target dimension is very big.
 *  @see BCH_Sketch, BCH_Small, BCH_Null, SRHT, KN */
template <class T = double>
class CW {
public:
	/** @brief 	The first template parameter (T) */
	typedef T value_type;

	/** @brief Constructor with a specified target dimension
	 *  @param dim_reduced
	 *  	The new first dimension of the input data.
	 *  @param dim_data
	 *  	The second dimension of the input data.
	 *  @details The input data is an (n x dim_data) matrix and it will be reduced to a (dim_reduced x dim_data) matrix.
	 *  @see CW(double, int) */
	CW(int dim_reduced, int dim_data);

	/** @brief Constructor with a specified epsilon
	 *  @param eps
	 *  	The quality of the embedding.
	 *  @param dim_data
	 *  	The second dimension of the input data.
	 *  @details The input data is an (n x dim_data) matrix and it will be reduced to a (k x dim_data) matrix.
	 *  	The variable \a k is calculated from the passed parameters. The embedding will be an (1 + \a eps) approximation of the input matrix.
	 *  @see CW(int, int) */
	CW(double eps, int dim_data);

	/** @brief Destructor
	 *  @details This destructor contains no explicit code. */
	virtual ~CW();

	/** @brief Calculates the necessary number of rows for the embedding
	 *  @details If you have a linear equation system and you want to reduce the number of equations, you can
	 *  	use this function to calculate the number of equations of the embedded matrix.
	 *  @param eps
	 *  	The quality of the embedding.
	 *  @param dim_data
	 *  	The second dimension of the input data.
	 *  @returns the necessary number of rows of the embedded matrix for an (1 + \a eps) approximation. */
	static int calcDim_Reduced(double eps, int dim_data);

	/** @brief Resets the embedded matrix to a zero matrix
	 * 	@details If you want to embed a new input matrix with the same Rademacher matrix
	 * 		you can use this method to clear the embedded matrix. */
	void reset();

	/** @brief Embeds a single entry.
	 *  @param aij
	 *  	Value at the position (i,j)
	 *  @param i
	 *  	First index
	 *  @param j
	 *  	Second index
	 *  @details This method embeds a single entry of an input matrix.
	 *  @see updateRows, updateColumns, updateMatrix, updateAll */
	void updateEntry(T aij, int i, int j);

	/** @brief Embeds one or several rows.
	 *  @param rows
	 *  	A matrix containing one or more rows of the input data.
	 *  @param startRow
	 *  	Row number of the first row of the passed matrix in the input matrix.
	 *  @details This method embeds one or several rows of an input matrix.
	 *  	To produce correct results you have to specify the row offset
	 *  	such that <em> rows(i, j) = input(startRow + i, j) </em>.
	 *  @see updateEntry, updateColumns, updateMatrix, updateAll */
	void updateRows(const Matrix<T>& rows, int startRow = 0);

	/** @brief Embeds one or several columns.
	 *  @param columns
	 *  	A matrix containing one or more columns of the input data.
	 *  @param startColumn
	 *  	Column number of the first column of the passed matrix in the input matrix.
	 *  @details This method embeds one or several columns of an input matrix.
	 *  	To produce correct results you have to specify the column offset
	 *  	such that <em> columns(i, j) = input(i, startColumn + j) </em>.
	 *  @see updateEntry, updateColumns, updateRows, updateMatrix  */
	void updateColumns(const Matrix<T>& columns, int startColumn = 0);

	/** @brief Embeds a whole matrix
	 *  @param mat
	 *  	The matrix that will be embedded.
	 *  @param off1
	 *  	Offset of the first dimension
	 *  @param off2
	 *  	Offset of the second dimension
	 *  @details This method embeds a matrix. If the matrix is not equal to
	 *  	the input matrix, then you have to specify the offsets, such that
	 *  	<em> mat(i, j) = input(i + off1, j + off2) </em>
	 *  @see updateEntry, updateRows, updateColumns, updateAll */
	void updateMatrix(const Matrix<T>& mat, int off1 = 0, int off2 = 0);

	/** @brief Embeds a whole matrix
	 *  @param mat
	 *  	The matrix that will be embedded.
	 *  @details Calling this method has the same effect as calling <em> updateMatrix(mat, 0, 0) </em>
	 *  @see updateEntry, updateRows, updateColumns, updateMatrix */
	void updateAll(const Matrix<T>& mat);

	/** @brief Returns a reference to the not normalized embedded matrix.
	 *  @returns a reference to the not normalized embedded matrix.
	 *  @see getSketch_normalized */
	const Matrix<T>& getSketch() const;

	/** @brief Returns the normalized embedded matrix.
	 *  @returns the normalized embedded matrix.
	 *  @details Use this method to get the embedded matrix.
	 *  @see getSketch */
	Matrix<T> getSketch_normalized() const;

private:
	/** @brief Embedded matrix */
	Matrix<T> sketch_;

	/** @brief Number of bits created by the hash function */
	int m_;

	/** @brief Seed value for the hash function
	 *  @details This seed is used for a linear congruential generator.
	 *  @see b_ */
	long long a_;

	/** @brief Seed value for the hash function
	 *  @details This seed is used for a linear congruential generator.
	 *  @see a_ */
	long long b_;

	/** @brief BCH generator
	 *  @details Random sequence to decide if an embedded value is added or subtracted to/from the embedded matrix. */
	BCH sign_;
};


// ===========================
// ===== IMPLEMENTIERUNG =====
// ===========================

// (unsigned) (a*x+b) >> (w-M)

template <class T>
CW<T>::CW(int dim_reduced, int dim_data) :
	sign_(4) {
	a_ = useful::random_LL();
	b_ = useful::random_LL();
	// Bestimmung der Anzahl der nötigen bits
	m_ = 0;
	dim_reduced--;
	while (dim_reduced > 0) {
		dim_reduced >>= 1;
		m_++;
	}
	dim_reduced = 1 << m_;
	sketch_ = Matrix<T>::zero(dim_reduced, dim_data);
}

template <class T>
CW<T>::CW(double eps, int dim_data) :
	CW(calcDim_Reduced(eps, dim_data), dim_data) {
}

template <class T>
int CW<T>::calcDim_Reduced(double eps, int dim_data) {
    return ceil(dim_data * (dim_data + 1) / (eps * eps));
}

template <class T>
CW<T>::~CW() {

}

template <class T>
void CW<T>::reset() {
	sketch_.fill(0);
}

template <class T>
void CW<T>::updateEntry(T aij, int i, int j) {
	unsigned long long hash = (a_*i + b_);
	int h_i = hash >> (64 - m_);
	sketch_(h_i, j) += aij * sign_[i];
}

template <class T>
void CW<T>::updateRows(const Matrix<T>& matA, int startRow) {
	updateMatrix(matA, startRow, 0);
}

template <class T>
void CW<T>::updateColumns(const Matrix<T>& matA, int startColumn) {
	updateMatrix(matA, 0, startColumn);
}

template <class T>
void CW<T>::updateMatrix(const Matrix<T>& matA, int off1, int off2) {
    for (int i = 0; i < matA.getDim1(); i++) {
        unsigned long long hash = (a_*(i+off1) + b_);
        int h_i = hash >> (64 - m_);
        int sign_i = sign_[i + off1];
        for (int j = 0; j < matA.getDim2(); j++) {
//            updateEntry(matA(i,j), off1 + i, off2 + j);
            sketch_(h_i, j + off2) += matA(i,j) * sign_i;
        }
    }
}

template <class T>
void CW<T>::updateAll(const Matrix<T>& matA) {
	updateMatrix(matA, 0, 0);
}

template <class T>
const Matrix<T>& CW<T>::getSketch() const {
	return sketch_;
}

template <class T>
Matrix<T> CW<T>::getSketch_normalized() const {
	return Matrix<T>(sketch_);
//	double t = 1;
//	Matrix<T> erg(skizze_.getDim1(), skizze_.getDim2());
//	for (int i = 0; i < erg.getDim1(); i++)
//		for (int j = 0; j < erg.getDim2(); j++)
//			erg(i,j) = skizze_(i,j) * t;
//	return erg;
}

#endif /* CW_H_ */
