/** @file KN.h
 *  @brief Fast and sparse embedding by Kane and Nelson
 *  @author Jens Quedenfeld
 */

/*
 * KN.h
 *
 *  Created on: 08.04.2013
 *      Author: Admin
 */

#ifndef KN_H_
#define KN_H_

#include "mat/matrix.h"
#include "sketch/BCH.h"

/** @brief Fast and sparse embeddings by Kane and Nelson
 *
 *  @details This class can embed matrices to reduce its first dimension.
 *  	For this purpose each entry of a column vector is projected of several areas.
 *  	Each area consists of many entries, but the entry of the vector that is embedded
 *  	is only projected to one of these entries within an area.
 *  	So this method can be seen as an expansion of the {@link CW CW} embedding.
 *		For more information, read "Sparser Johnson-Lindenstrauss Transforms"
 *		by <em> Daniel M. Kane and Jelani Nelson </em> */
template <class T>
class KN {
public:
	/** @brief 	The first template parameter (T) */
	typedef T value_type;

	/** @brief Constant denoting that the degree of independence is set automatically
	 *  @details This constant can be used in the constructor
	 *  	{@link KN(double, int, int, int) KN(double, int, int, int)}. */
	const static int AUTO_DEG;

	/** @brief Constructor with a specified target dimension
	 *  @param dim_reduced
	 *  	Target dimension: The number of rows of the embedded matrix.
	 *  @param dim_data
	 *  	The number of columns of the original respectively embedded data matrix.
	 *  @param sparsity
	 *  	The number of areas an entry is projected.
	 *  	\a dim_reduced must be divisible by this value.
	 *  @param m
	 *  	The number of random bits created for each projection.
	 *  	This value is equal to <em> lb (dim_reduced / s) </em>.
	 *  @param deg_sigma
	 *  	Degree of independence for the sigma function. The use of \a AUTO_DEG is not supported.
	 *  @param deg_hash
	 *  	Degree of independence for the hash function. The use of \a AUTO_DEG is not supported.
	 *  @pre The following equation must be fulfilled: <em> 2^m * sparsity = dim_reduced </em>
	 *  @details The input data is an (n x dim_data) matrix and it will be reduced to a (dim_reduced x dim_data) matrix.
	 *  @see KN(double, int, int, int) */
	KN(int dim_reduced, int dim_data, int sparsity, int m, int deg_sigma = 4, int deg_hash = 4);

	/** @brief Constructor with a specified epsilon
	 *  @param epsilon
	 *  	The quality of the embedding.
	 *  @param dim_data
	 *  	The second dimension of the input data.
	 *  @param deg_sigma
	 *  	Degree of independence for the sigma function. If the constant
	 *  	\a AUTO_DEG is used, the necessary degree of independence is calculated.
	 *  @param deg_hash
	 *  	Degree of independence for the hash function. If the constant
	 *  	\a AUTO_DEG is used, the necessary degree of independence is calculated.
	 *  @details The input data is an (n x dim_data) matrix and it will be reduced to a (k x dim_data) matrix.
	 *  	The variable \a k is calculated from \a epsilon and \a dim_data. The embedding will be an (1 + \a epsilon) approximation of the input matrix.
	 *  @see KN(int, int, int, int, int, int), AUTO_DEG */
	KN(double epsilon, int dim_data, int deg_sigma = AUTO_DEG, int deg_hash = AUTO_DEG);

	/** @brief Destructor
	 *  @details This destructor contains no explicit code. */
	virtual ~KN();

	/** @brief Embeds a single entry.
	 *  @param aij
	 *  	Value at the position (i,j)
	 *  @param i
	 *  	First index
	 *  @param j
	 *  	Second index
	 *  @details This method embeds a single entry of an input matrix.
	 *  @see updateRows, updateColumns, updateMatrix, updateAll */
	void updateEntry(T aij, int i, int j);

	/** @brief Embeds one or several rows.
	 *  @param rows
	 *  	A matrix containing one or more rows of the input data.
	 *  @param startRow
	 *  	Row number of the first row of the passed matrix in the input matrix.
	 *  @details This method embeds one or several rows of an input matrix.
	 *  	To produce correct results you have to specify the row offset
	 *  	such that <em> rows(i, j) = input(startRow + i, j) </em>.
	 *  @see updateEntry, updateColumns, updateMatrix, updateAll */
	void updateRows(const Matrix<T>& rows, int startRow = 0);

	/** @brief Embeds one or several columns.
	 *  @param columns
	 *  	A matrix containing one or more columns of the input data.
	 *  @param startColumn
	 *  	Column number of the first column of the passed matrix in the input matrix.
	 *  @details This method embeds one or several columns of an input matrix.
	 *  	To produce correct results you have to specify the column offset
	 *  	such that <em> columns(i, j) = input(i, startColumn + j) </em>.
	 *  @see updateEntry, updateRows, updateMatrix, updateAll  */
	void updateColumns(const Matrix<T>& columns, int startColumn = 0);

	/** @brief Embeds a whole matrix
	 *  @param mat
	 *  	The matrix that will be embedded.
	 *  @param off1
	 *  	Offset of the first dimension
	 *  @param off2
	 *  	Offset of the second dimension
	 *  @details This method embeds a matrix. If the matrix is not equal to
	 *  	the input matrix, then you have to specify the offsets, such that
	 *  	<em> mat(i, j) = input(i + off1, j + off2) </em>
	 *  @see updateEntry, updateRows, updateColumns, updateAll */
	void updateMatrix(const Matrix<T>& mat, int off1 = 0, int off2 = 0);

	/** @brief Embeds a whole matrix
	 *  @param mat
	 *  	The matrix that will be embedded.
	 *  @details This method embeds a whole input matrix.
	 *  @see updateEntry, updateRows, updateColumns, updateMatrix  */
	void updateAll(const Matrix<T>& mat);

	/** @brief Resets the embedded matrix to a zero matrix
	 * 	@details If you want to embed a new input matrix with the same Rademacher matrix
	 * 		you can use this method to clear the embedded matrix. */
	void reset();

	/** @brief Returns a reference to the not normalized embedded matrix.
	 *  @returns a reference to the not normalized embedded matrix.
	 *  @see getSketch_normalized */
	const Matrix<T>& getSketch() const;

	/** @brief Returns the normalized embedded matrix.
	 *  @returns the normalized embedded matrix.
	 *  @details Use this method to get the embedded matrix.
	 *  @see getSketch */
	Matrix<T> getSketch_normalized() const;

private:
	/** @brief Reduced dimension
	 *  @returns the reduced dimension */
	int getDim_reduced() const;

	/** @brief Calculates a value of the hash function
	 *  @param i
	 *  	Row index. The i-th entry of a vector
	 *  	(respectively the i-th row of the data matrix)
	 *  	is currently embedded.
	 *  @param r
	 *  	Area index. The entry is projected to the r-th area. (There are altogether \a sparsity_ areas)
	 *  @returns the index within the current area.
	 *  @details The returned value is larger or equal zero and smaller than \a k_div_s_
	 *  @see sparsity_, k_div_s_ */
	int hash(int i, int r) const;

	/** @brief A single element is projected to this number of areas */
	int sparsity_; // sparsity (Anzahl der "Grüppchen")

	/** @brief The size of each area
	 *  @details This value is equal to the reduced dimension divided by \a sparsity_ */
	int k_div_s_; // k/s

	/** @brief The number of bits created for each projection to a single area
	 *  @details This value is equal to the logarithm of the basis 2 of \a k_div_s_
	 *  @see k_div_s_ */
	int m_; // 2^m == k_div_s

	/** @brief BCH generator used for the sigma function */
	BCH sigma_;

	/** @brief BCH generator used for the hash function */
	BCH hash_;

	/** @brief Embedded matrix */
	Matrix<T> sketch_;
};

template <class T>
const int KN<T>::AUTO_DEG = -1;

template <class T>
KN<T>::KN(int dim_reduced, int dim_data, int sparsity, int m, int deg_sigma, int deg_hash) :
	sparsity_(sparsity),
	k_div_s_(dim_reduced / sparsity),
//	d_mul_s_(dim_data * sparsity),
	m_(m),
	sigma_(deg_sigma),
	hash_(deg_hash) {
	if ((1 << m_) * sparsity_ != dim_reduced)
		throw std::invalid_argument("KN: (k/s) * s != k");
	sketch_ = Matrix<T>::zero(dim_reduced, dim_data, true);
}

template <class T>
KN<T>::KN(double epsilon, int dim_data, int deg_sigma, int deg_hash) :
	sigma_(deg_sigma != AUTO_DEG ? deg_sigma : 4),
	hash_(deg_hash != AUTO_DEG ? deg_hash : 4) {
	sparsity_ = ceil(2 / epsilon);
	int dim_reduced = ceil(dim_data * log(dim_data) / (epsilon * epsilon));
	k_div_s_ = dim_reduced / sparsity_;

	// Vergrößere k_div_s auf eine Zweierpotenz
	m_ = 0;
	k_div_s_--;
	while (k_div_s_ > 0) {
		k_div_s_ >>= 1;
		m_++;
	}
	k_div_s_ = 1 << m_;

	dim_reduced = k_div_s_ * sparsity_;
//	d_mul_s_ = dim_data * sparsity_;
	sketch_ = Matrix<T>::zero(dim_reduced, dim_data, true);


//	std::cout << sigma_.getDegree() << "," << hash_.getDegree() << "," << sparsity_ << "," << dim_reduced  << "," << k_div_s_ << "," << d_mul_s_ << std::endl;
}

template <class T>
KN<T>::~KN() {

}

template <class T>
void KN<T>::updateEntry(T aip, int i, int p) {
	for (int r = 0; r < sparsity_; r++) {
		int idx = r * k_div_s_ + hash(i,r);
		sketch_.addEntry(idx, p, aip * sigma_[i * sparsity_ + r]);
	}
}

template <class T>
void KN<T>::updateRows(const Matrix<T>& mat, int startRow) {
	updateMatrix(mat, startRow, 0);
}

template <class T>
void KN<T>::updateColumns(const Matrix<T>& mat, int startColumn) {
	updateMatrix(mat, 0, startColumn);
}

template <class T>
void KN<T>::updateMatrix(const Matrix<T>& mat, int off1, int off2) {
	for (int i = 0; i < mat.getDim1(); i++) {
		int i_offset = i + off1;
		for (int r = 0; r < sparsity_; r++) {
			int idx = r * k_div_s_ + hash(i_offset ,r);
			int sigma = sigma_[i_offset * sparsity_ + r];
			for (int p = 0; p < mat.getDim2(); p++)
				sketch_.addEntry(idx, p + off2, mat(i, p) * sigma);
		}
	}
}

template <class T>
void KN<T>::updateAll(const Matrix<T>& mat) {
	updateMatrix(mat, 0, 0);
}

template <class T>
void KN<T>::reset() {
	sketch_.fill(0);
}

template <class T>
const Matrix<T>& KN<T>::getSketch() const {
	return sketch_;
}

template <class T>
Matrix<T> KN<T>::getSketch_normalized() const {
	return sketch_ / sqrt((T) sparsity_);
}

template <class T>
int KN<T>::getDim_reduced() const {
	return sketch_.getDim1();
}

template <class T>
int KN<T>::hash(int i, int r) const {
	typedef unsigned long long ULL;
	ULL idx = (i * (ULL) sparsity_ + r) * m_;
	int erg = hash_.getValue_01(idx);
	for (int t = 1; t < m_; t++) {
		erg <<= 1;
		erg += hash_.getValue_01(idx + t);
	}
	return erg;
}

#endif /* KN_H_ */
