/** @file SRHT.h
 *  @brief Fast embedding using the Fast Johnson-Lindenstrauss transform
 *  @author Jens Quedenfeld
 */

#ifndef SRHT_NEW_H_
#define SRHT_NEW_H_

#include "mat/matrix.h"
#include "util/Useful.h"
#include <map>
#include <cstdint>
#if __cplusplus >= 201103L
#include <unordered_map>
#include <unordered_set>
#endif

/** @brief Fast embeddings using the Fast Johnson-Lindenstrauss transform
 *
 *  @details This class can embed matrices to reduce its first dimension.
 *  	The following equation is calculated to embed a (n x d)-matrix \a A: <em> P*H*D*A </em> \n
 *  	\a P is a (k x n)-matrix with exactly one 1 in each row \n
 *  	\a H is a (n x n)-Hadamard-matrix \n
 *  	\a D is a (n x n) diagonal matrix with random +1 and -1 entries on its main diagonal. \n
 *  	For more information, see \n
 *  	"IMPROVED ANALYSIS OF THE SUBSAMPLED RANDOMIZED HADAMARD TRANSFORM" by Joel A. Tropp \n
 *  	"Approximate Nearest Neighbors and the Fast Johnson-Lindenstrauss Transform" by Nir Ailon and Bernard Chazelle \n\n
 *  	There is the related class {@link SRHT_direct SRHT_direct} which also embeds matrices using the same method. There are some differences between
 *  	the two classes: This class has an interface similar to the other embedding classes like BCH_Sketch, CW and so on. \a SRHT_direct has a completly different
 *  	interface, however, it supports the inverse SRHT calculation. See {@link SRHT_direct::calc_inv calc_inv} for more information.
 *  @see SRHT_direct, BCH_Sketch, BCH_Null, BCH_Small, CW, KN */
template <class T = double>
class SRHT {
public:
	/** @brief The first template parameter (T) */
	typedef T value_type;

	/** @brief Constructor with a specified target dimension
	 *  @param dim_reduced
	 *  	The new first dimension of the input data.
	 *  @param dim_data
	 *  	The second dimension of the input data.
	 *  @details The input data is an (n x dim_data) matrix and it will be reduced to a (dim_reduced x dim_data) matrix.
	 *  @see SRHT(double, int) */
	SRHT(int dim_reduced, int dim_data);

	/** @brief Constructor with a specified epsilon
	 *  @param eps
	 *  	The quality of the embedding.
	 *  @param dim_data
	 *  	The second dimension of the input data.
	 *  @details The input data is an (n x dim_data) matrix and it will be reduced to a (k x dim_data) matrix.
	 *  	The variable \a k is calculated by the function calcDim_Reduced. The embedding will be an (1 + \a eps) approximation of the input matrix.
	 *  @see SRHT(int, int), calcDim_Reduced */
	SRHT(double eps, int dim_data);

	/** @brief Destructor
	 *  @details This destructor contains no explicit code. */
	virtual ~SRHT();

	/** @brief Embeds one or several columns.
	 *  @param columns
	 *  	A matrix containing one or more columns of the input data.
	 *  @param startColumn
	 *  	Column number of the first column of the passed matrix in the input matrix.
	 *  @details This method embeds one or several columns of an input matrix.
	 *  	To produce correct results you have to specify the column offset
	 *  	such that <em> columns(i, j) = input(i, startColumn + j) </em>.
	 *  @see updateAll  */
	void updateColumns(const Matrix<T>& columns, int startColumn = 0);

	/** @brief Embeds a whole matrix
	 *  @param mat
	 *  	The matrix that will be embedded.
	 *  @param buffer
	 *  	Number of columns of the data matrix that is computed simultaneously.
	 *  @details This method embeds a whole input matrix.
	 *  @see updateEntry, updateRows, updateColumns, updateMatrix  */
	void updateAll(const Matrix<T>& mat, int buffer = 1000);

	/** @brief Returns a reference to the not normalized embedded matrix.
	 *  @returns a reference to the not normalized embedded matrix.
	 *  @see getSketch_normalized */
	const Matrix<T> getSketch() const;

	/** @brief Returns the normalized embedded matrix.
	 *  @returns the normalized embedded matrix.
	 *  @details Use this method to get the embedded matrix.
	 *  @see getSketch */
	Matrix<T> getSketch_normalized() const;

	/** @brief Calculates the necessary number of rows for the embedding
	 *  @details If you have a linear equation system and you want to reduce the number of equations, you can
	 *  	use this function to calculate the number of equations of the embedded matrix.
	 *  @param eps
	 *  	The quality of the embedding.
	 *  @param dim_data
	 *  	The second dimension of the input data.
	 *  @returns the necessary number of rows of the embedded matrix for an (1 + \a eps) approximation. */
	static int calcDim_Reduced(double eps, int dim_data);

	/** @brief Resets the embedded matrix to a zero matrix
	 * 	@details If you want to embed a new input matrix with the same Rademacher matrix
	 * 		you can use this method to clear the embedded matrix. */
	void reset();

	/** @brief rekursive function to calculate the multiplication with a Hadamard matrix
	 *  @details This function need not to be used by users of this library.
	 *  	It is an internal function used by this class and {@link SRHT_direct SRHT_direct}.
	 *  	This function calculates the product <em> P * H * A </em>.
	 *  @param p
	 *  	Array with indices that should be selected (after Hadamard multiplication). It is the implicit form of the matrix \a P. See {@link p_ p_} for more information.
	 *  @param k
	 *  	Length of \a p respectively the number of rows of the current submatrix of \a P.
	 *  @param q
	 *  	Size of the matrices (columns of \a P).
	 *  @param offset
	 *  	Row offset in the original input matrix. It is needed to divide the array \a p into two parts.
	 *  @param matA
	 *  	The current submatrix of \a A.
	 *  @param result
	 *  	Into this matrix the result is written. After the recursion this matrix contains the product <em> P * H * A </em>
	 *  @param result_offset
	 *  	Row offset in the result matrix \a result. */
	static void calc_SRHT_rek(const int* p, int k, int q, int offset, const Matrix<T>& matA, Matrix<T>& result, int result_offset);

//private:
	/** @brief Initialization of \a p_, q_ and dim_original_
	 *  @param dim_original
	 *  	The number of rows of the input matrix.*/
	void init_p(int dim_original);

    void init_p_oldImpl(int dim_original);

private:

	/** @brief Number of rows (first dimension) of the input data */
	int dim_original_;

	/** @brief Lowest power of two that is equal or bigger than dim_original */
	int q_;

	/** @brief Implicit stored matrix \a P
	 *  @details The vector contains \a dim_reduced elements.
	 *  	The matrix \a P has a 1 at <em> (i, p_[i]) </em> for all i in {0, 1, ..., dim_reduced-1} */
	std::vector<int> p_;

	/** @brief BCH generator for the entries of the diagonal matrix \a D */
	BCH d_;

	/** @brief Embedded matrix */
	Matrix<T> sketch_;
};


// ===========================
// ===== IMPLEMENTIERUNG =====
// ===========================

template <class T>
SRHT<T>::SRHT(int dim_reduced, int dim_data) :
	dim_original_(0),
	q_(0),
	p_(dim_reduced),
	d_(4),
	sketch_(dim_reduced, dim_data) {
	reset();
}


template <class T>
SRHT<T>::SRHT(double eps, int dim_data) :
	SRHT(calcDim_Reduced(eps, dim_data), dim_data) {

}

template <class T>
SRHT<T>::~SRHT() {

}

template <class T>
void SRHT<T>::updateColumns(const Matrix<T>& mat, int startColumn) {
	if (!dim_original_) init_p(mat.getDim1());
	int d = mat.getDim2();

	// RandomMatrix
	Matrix<T> matDA(dim_original_, d);
	for (int i = 0; i < dim_original_; i++) {
		T fac = d_[i];
		for (int j = 0; j < d; j++)
			matDA.updateEntry(i, j, mat.getEntry(i,j) * fac);
	}

	// Divide and Conquer
	Matrix<T> result = Matrix<T>::zero(sketch_.getDim1(), d);
	SRHT<T>::calc_SRHT_rek(&p_[0], sketch_.getDim1(), q_, 0, matDA, result, 0);

	for (int i = 0; i < result.getDim1(); i++)
		for (int j = 0; j < result.getDim2(); j++)
			sketch_(i, j + startColumn) += result(i,j);
}

template <class T>
void SRHT<T>::updateAll(const Matrix<T>& mat, int buffer) {
	int currentColumn = 0;
	while (currentColumn < mat.getDim2()) {
		updateColumns(mat.select(0, currentColumn, mat.getDim1(), std::min(buffer, mat.getDim2() - currentColumn)), currentColumn);
		currentColumn += buffer;
	}
}

template <class T>
const Matrix<T> SRHT<T>::getSketch() const {
	return sketch_;
}

template <class T>
Matrix<T> SRHT<T>::getSketch_normalized() const {
	return sketch_ / (T) sqrt((T) sketch_.getDim1());
}

template <class T>
int SRHT<T>::calcDim_Reduced(double eps, int dim_data) {
    return ceil(dim_data * log(dim_data) / (eps * eps));
}

template <class T>
void SRHT<T>::reset() {
	sketch_.fill(0);
	dim_original_ = 0;
	q_ = 0;
}

template <class T>
void SRHT<T>::init_p(int dim_original) {
	dim_original_ = dim_original;
	q_ = 1;
    unsigned log_q = 0;
    while (q_ < dim_original) {
		q_ <<= 1;
        log_q++;
    }
	int dim_reduced = sketch_.getDim1();

	if (dim_reduced > dim_original)
		throw std::invalid_argument("SRHT::init_p(): dim_reduced > dim_original");

    std::mt19937_64 rnd(useful::random_LL());
    uint64_t a = rnd();
    uint64_t b = rnd();

    // prepare, if the dim_reduced is big
    bool bigDimReduced = dim_reduced > q_ / 2;
    std::vector<int> pInverted; // only used, if bigDimReduced==true
    int numSamples = bigDimReduced ? q_ - dim_reduced : dim_reduced; // choose at most dim_original/2 values to ensure a linear runtime
    if (bigDimReduced) {
        pInverted.resize(numSamples + 1);
        pInverted[numSamples] = std::numeric_limits<int>::max();
        pInverted.swap(p_);
    }

    // sampling
    std::unordered_set<uint64_t> values;
    int i = 0;
    uint64_t count = 1;
    while (i < numSamples) {
        uint64_t hash = (a * count++ + b) >> (64 - log_q);
        if (values.find(hash) == values.end()) {
            // the hash value was not used yet
//            std::cout << "i:" << hash << std::endl;
            values.insert(hash);
            p_[i++] = hash;
        }
    }
    sort(p_.begin(), p_.end());

    // post processing, if dim_reduced is big
    if (bigDimReduced) {
        pInverted.swap(p_); // now pInverted contains the samples

        auto pWriter = p_.begin();
        auto pInvReader = pInverted.begin();
//        std::cout << "p_.size() = " << p_.size() << std::endl;
//        std::cout << "pInverted.size() = " << pInverted.size() << std::endl;
        for (int i = 0; i < dim_original; i++) {
//            std::cout << "i=" << i << std::endl;
            if (*pInvReader == i) {
//                std::cout << "++" << std::endl;
                ++pInvReader; // next value in pInverted, if i was sampled.
            } else {
//                std::cout << "wr: " << (p_.end() - pWriter) << std::endl;
                *pWriter++ = i; // write i, if i was not in pInverted
            }
        }
    }
}

template <class T>
void SRHT<T>::init_p_oldImpl(int dim_original) {
    dim_original_ = dim_original;
    q_ = 1;
    while (q_ < dim_original) {
        q_ <<= 1;
    }
    int dim_reduced = sketch_.getDim1();

    if (dim_reduced > dim_original)
        throw std::invalid_argument("SRHT::init_p(): dim_reduced > dim_original");

#if __cplusplus >= 201103L
    std::unordered_map<int, int> conv;
#else
    std::map<int, int> conv;
#endif
	for (int i = 0; i < dim_reduced; i++) {
		int choose_max = q_ - i;
		unsigned long long rnd;
		const unsigned long long rnd_max = -1ULL;
		do {
			rnd = useful::random_LL();
		} while (rnd > rnd_max - rnd_max % choose_max);
		int idx = rnd % choose_max;
		int conv_idx = idx;
		while (conv.find(conv_idx) != conv.end()) {
			conv_idx = conv[conv_idx];
		}
		conv[idx] = choose_max - 1;
		p_[i] = conv_idx;
	}
    sort(p_.begin(), p_.end());


}

//namespace {
/* Parameter
 *     p : Matrix P: in Zeile i steht an Positon p[i] eine 1, sonst nur Nullen
 *     k : Zeilen von P (d.h. Länge des p-Vektors)
 *     q : Größe der Matrizen (z.B. Spalten von P)
 *     offset : offset in der Matrix A
 *     matA : Matrix A
 */
template <class T>
void SRHT<T>::calc_SRHT_rek(const int* p, int k, int q, int offset, const Matrix<T>& matA, Matrix<T>& erg, int erg_offset) {
	if (k == 0) {
		// Rekursionsabbruch
		return;
	}

	if (k == 1) {
		// Rekursionsabbruch
		int counter = 0;
		for (int j = 0; j < matA.getDim2(); j++) {
			erg(erg_offset, j) += matA(0, j); // es gilt immer hd(x,0) == 1
		}
		for (int i = 1; i < matA.getDim1(); i++) {
			int btmp = i;
			int atmp = p[0];
			while ((btmp & 1) == 0) {
				counter ^= atmp & 1;
				atmp >>= 1;
				btmp >>= 1;
			}
			counter ^= atmp & 1;
			T hd = counter ? -1 : 1;
			for (int j = 0; j < matA.getDim2(); j++) {
				erg(erg_offset, j) += hd * matA(i, j);
			}
		}
		return;
	}

	// Bin�re Suche
	int left = 0; // inklusive
	int right = k-1; // exklusive
	const int find = offset + q/2;
	int k1 = k;
	while (true) {
		int mid = (right + left) / 2;
		if (left == right) {
			if (p[mid] >= find)
				k1 = mid;
			break;
		} else if (p[mid] == find) {
			k1 = mid;
			break;
		} else if (p[mid] < find) {
			left = mid + 1;
		} else {
			right = mid;
		}
	}

	// Rekursion
	int d = matA.getDim2();
	Matrix<T> matA_para;
	matA_para = matA.select(0, 0, q/2, d).addMatrix(matA, q/2, d, 0, 0, q/2, 0);
	calc_SRHT_rek(p, k1, q/2, offset, matA_para, erg, erg_offset);
	matA_para = matA.select(0, 0, q/2, d).subMatrix(matA, q/2, d, 0, 0, q/2, 0);
	calc_SRHT_rek(p + k1, k - k1, q/2, offset + q/2, matA_para, erg, erg_offset + k1);
}//}




#endif /* SRHT_NEW_H_ */
