/** @file SRHT_direct.h
 *  @brief Fast embedding and inverse embedding using the Fast Johnson-Lindenstrauss transform
 *  @author Jens Quedenfeld
 */

#ifndef SRHT_H_
#define SRHT_H_

#include "mat/matrix.h"
#include "sketch/SRHT.h"
#include <map>
#if __cplusplus >= 201103L
#include <unordered_map>
#endif

/** @brief Fast embeddings using the Fast Johnson-Lindenstrauss transform
 *
 *  @details This class can embed matrices to reduce its first dimension.
 *  	The following equation is calculated to embed a (n x d)-matrix \a A: <em> P*H*D*A </em> \n
 *  	\a P is a (k x n)-matrix with exactly one 1 in each row \n
 *  	\a H is a (n x n)-Hadamard-matrix \n
 *  	\a D is a (n x n) diagonal matrix with random +1 and -1 entries on its main diagonal. \n
 *  	For more information, see \n
 *  	"IMPROVED ANALYSIS OF THE SUBSAMPLED RANDOMIZED HADAMARD TRANSFORM" by Joel A. Tropp \n
 *  	"Approximate Nearest Neighbors and the Fast Johnson-Lindenstrauss Transform" by Nir Ailon and Bernard Chazelle
 *  @see SRHT, BCH_Sketch, BCH_Null, BCH_Small, CW, KN */
template <class T = double>
class SRHT_direct {
public:
	/** @brief The first template parameter (T) */
	typedef T value_type;

	/** @brief Standard constructor
	 *  @param dim_original
	 * 		The first dimension of the input data.
	 *  @param dim_reduced
	 * 		The first dimension is reduced to this value.
	 * 	@details The input data is an (dim_original x d) matrix and it will
	 * 		be reduced to a (dim_reduced x d) matrix */
	SRHT_direct(int dim_original, int dim_reduced);

	/** @brief Destructor
	 *  @details This destructor contains no explicit code. */
	~SRHT_direct();

	/** @brief Embeds one or several columns of the input data.
	 *  @param matA
	 *  	One or several columns of the input data.
	 *  @returns
	 *  	the embedded matrix \a matA.
	 *  @details If you want to embed a big matrix,
	 *  	split it into column vectors (or matrices with less columns)
	 *  	and call this method for each vector (or matrix). The second dimension
	 *  	of the input is equal to the second dimension of the output, so you
	 *  	can connect the results to one big matrix. \n
	 *  	This function calculates <em> P*H*D*matA </em>, where \n
	 *  	\a P is a (k x n)-matrix with exactly one 1 in each row \n
	 *  	\a H is a (n x n)-Hadamard-matrix \n
	 *  	\a D is a (n x n) diagonal matrix with random +1 and -1 entries on its main diagonal. \n
	 * @see calc_inv, SRHT::calc_SRHT_rek */
	Matrix<T> calc(const Matrix<T>& matA) const;

	/** @brief Inverted SRHT
	 *	@details This method calculates the inverted SRHT:
	 *		A input matrix A that is embedded by the function \a calc
	 *		can be transformed back by this method. However, the result of
	 *		<em> calc_inv(calc(A)) </em>
	 *		is not equal to A, but it has similar properties like A.
	 *		In fact, the result of this operation is the pseudoinverse of <em> (P*H*D) </em> multiplied with \a y.
	 *		This is equal to <em> D * H * P^T * y </em>, where <em> P^T </em> is the transpose matrix of \a P
	 *  @param y
	 *  	An embedded matrix.
	 *  @attention
	 *  	The first dimension of y must be a power of two to create good results.
	 *  @see calc_inv_alt, calc */
	Matrix<T> calc_inv(const Matrix<T>& y) const;

	/** @brief Alternativ method to calculate the inverted SRHT
	 *  @details See {@link calc_inv calc_inv} for information about this method (which calculates the same like \a calc_inv, but with another algorithm).
	 *  	This function is faster than \a calc_inv, if <em> O(dim_reduced) < O(log dim_original) </em>.
	 *  @see calc_inv */
	Matrix<T> calc_inv_alt(const Matrix<T>& y) const;

private:
	/** @brief Friend for testing the inverse SRHT */
	friend void srht_inv_test();

	/** @brief Number of rows (first dimension) of the input data */
	int dim_original;

	/** @brief Number of rows (first dimension) of the embedded matrix */
	int dim_reduced;

	/** @brief Lowest power of two that is equal or bigger than dim_original */
	int q;

	/** @brief BCH generator for the entries of the diagonal matrix \a D */
	BCH d_;

	/** @brief Implicit stored matrix \a P
	 *  @details The vector contains \a dim_reduced elements.
	 *  	The matrix \a P has a 1 at <em> (i, p_[i]) </em> for all i in {0, 1, ..., dim_reduced-1} */
	std::vector<int> p_;

	/** @brief Initialization of \a p_ */
	void init_p();
};

// ===========================
// ===== IMPLEMENTIERUNG =====
// ===========================

template <class T>
SRHT_direct<T>::SRHT_direct(int dim_original, int dim_reduced)
	: dim_original(dim_original),
	  dim_reduced(dim_reduced),
	  d_(4), // TODO: SRHT: 4-fach ausreichend?
	  p_(dim_reduced) {
	q = 1;
	while (q < dim_original)
		q <<= 1;
	init_p();

}

template <class T>
SRHT_direct<T>::~SRHT_direct() {

}

template <class T>
Matrix<T> SRHT_direct<T>::calc(const Matrix<T>& matA) const {
	int d = matA.getDim2();

	// RandomMatrix
	Matrix<T> matDA(dim_original, d);
	for (int i = 0; i < dim_original; i++) {
		T fac = d_[i];
		for (int j = 0; j < d; j++)
			matDA.updateEntry(i, j, matA.getEntry(i,j) * fac);
	}

	// Divide and Conquer
	Matrix<T> srht(dim_reduced, d);
	srht.fill(0);
	SRHT<T>::calc_SRHT_rek(&p_[0], dim_reduced, q, 0, matDA, srht, 0);

	srht /= (T) sqrt((T) dim_reduced);

	return srht;
}

namespace {

template <class T>
void calc_SRHT_rek_inv(int q, const Matrix<T>& input, Matrix<T>& erg, int erg_offset) {
	if (q < 1)
		return;
	if (q == 1) {
		// Rekursionsabbruch
		int counter = 0;
		for (int j = 0; j < input.getDim2(); j++) {
			erg(erg_offset, j) += input(0, j); // es gilt immer hd(x,0) == 1
		}
		for (int i = 1; i < input.getDim1(); i++) {
			int btmp = i;
			int atmp = erg_offset;
			while ((btmp & 1) == 0) {
				counter ^= atmp & 1;
				atmp >>= 1;
				btmp >>= 1;
			}
			counter ^= atmp & 1;
			T hd = counter ? -1 : 1;
			for (int j = 0; j < input.getDim2(); j++) {
				erg(erg_offset, j) += hd * input(i, j);
			}
		}
		return;
	}
	int d = input.getDim2();
	Matrix<T> input_para;
	input_para = input.select(0, 0, q/2, d).addMatrix(input, q/2, d, 0, 0, q/2, 0);
	calc_SRHT_rek_inv(q/2, input_para, erg, erg_offset);
	input_para = input.select(0, 0, q/2, d).subMatrix(input, q/2, d, 0, 0, q/2, 0);
	calc_SRHT_rek_inv(q/2, input_para, erg, erg_offset + q/2);
}}

template <class T>
Matrix<T> SRHT_direct<T>::calc_inv(const Matrix<T>& y) const {
	int d = y.getDim2();

	// P^T*y
	Matrix<T> pty(q, d);
	pty.fill(0);
	for (int k = 0; k < dim_reduced; k++)
		for (int j = 0; j < d; j++)
			pty(p_[k], j) = y(k, j);

	// H * pty
	Matrix<T> hpty(dim_original, d);
	hpty.fill(0);
	calc_SRHT_rek_inv(q, pty, hpty, 0);

	// RandomMatrix
	for (int i = 0; i < hpty.getDim1(); i++) {
		T fac = d_[i];
		for (int j = 0; j < d; j++)
			hpty(i,j) *= fac;
	}

	hpty /= (T) (dim_original / sqrt(dim_reduced));

	return hpty;
}

template <class T>
Matrix<T> SRHT_direct<T>::calc_inv_alt(const Matrix<T>& y) const {
	int d = y.getDim2();
	Matrix<T> srht_inv(dim_original, d);
	srht_inv.fill(0);

	// Z = H*P^T*y
	// zij = sum(hik*(pty)kj)
	// zij = sum(k€p[], hik*ykj)
	for (int k = 0; k < y.getDim1(); k++) {
		int counter = 0;
		for (int j = 0; j < srht_inv.getDim2(); j++) {
			srht_inv(0, j) += y(k, j); // es gilt immer hd(x,0) == 1
		}
		for (int i = 1; i < srht_inv.getDim1(); i++) {
			int btmp = i;
			int atmp = p_[k];
			while ((btmp & 1) == 0) {
				counter ^= atmp & 1;
				atmp >>= 1;
				btmp >>= 1;
			}
			counter ^= atmp & 1;
			T hd = counter ? -1 : 1;
			for (int j = 0; j < srht_inv.getDim2(); j++) {
				srht_inv(i, j) += hd * y(k, j);
			}
		}
	}


	// RandomMatrix
	for (int i = 0; i < srht_inv.getDim1(); i++) {
		T fac = d_[i];
		for (int j = 0; j < d; j++)
			srht_inv(i,j) *= fac;
	}

	srht_inv /= (T) (dim_original / sqrt(dim_reduced));

	return srht_inv;
}


template <class T>
void SRHT_direct<T>::init_p() {
	if (dim_reduced > dim_original)
		throw std::invalid_argument("SRHT::init_p(): dim_reduced > dim_original");

#if __cplusplus >= 201103L
	std::unordered_map<int, int> conv;
#else
	std::map<int, int> conv;
#endif
	for (int i = 0; i < dim_reduced; i++) {
		int choose_max = q - i;
		unsigned long long rnd;
		const unsigned long long rnd_max = -1ULL;
		do {
			rnd = useful::random_LL();
		} while (rnd > rnd_max - rnd_max % choose_max);
		int idx = rnd % choose_max;
		int conv_idx = idx;
		while (conv.find(conv_idx) != conv.end()) {
			conv_idx = conv[conv_idx];
		}
		conv[idx] = choose_max - 1;
		p_[i] = conv_idx;
	}
	sort(p_.begin(), p_.end());
}




#endif /* SRHT_H_ */
