/** @file Useful.h
 * 	@brief Useful functions
 *  @author Jens Quedenfeld
 */

#ifndef USEFUL_H_
#define USEFUL_H_

namespace useful {}

template<class T>
class Matrix;

#include <algorithm>
#include <climits>
#include <fstream>
#include <string>
#include <vector>

/** @brief This namespace contains several useful functions. */
namespace useful {
	/** @name Matrix-Multiplication Time-Measuring
	 *
	 *  After each matrix-multiplication with the strassen-algorihm
	 *  the following variables contain the runtime of the different steps. */
	/** @{ */
	/** @brief Time used for the naive matrix multiplication for small matrices */
	extern long long timeMultiply_naive;
	/** @brief Time used for matrix addition */
	extern long long timeMultiply_add;
	/** @brief Time used to connect the partial results to one big matrix*/
	extern long long timeMultiply_last;
	/** @brief Time used for other operations*/
	extern long long timeMultiply_other;
	/** @} */

	/** @name Time-handling functions */
	/** @{ */

		/** @brief Returns the time difference and sets t0 to the current time.
		 *  @details
		 *  	First the time difference between now and t0 is saved, then
		 *  	t0 is set to the current time and at the end the saved
		 *  	difference is returned.
		 *  @param t0
		 *  	the start time of the operation which time is measured.
		 *  @returns the time difference
		 *  @see printTime, time2str
		 */
		long long getTime(long long& t0);

		/** @brief Prints the time difference and sets t0 to the current time.
		 *
		 *  @details Prints the prefix followed by the time difference between now and t0.
		 *  	The reference t0 is set to the current time via the clock()-function.
		 *
		 *  @param prefix
		 * 		a prefix string that will be printed first
		 *  @param t0
		 * 		the start time of the operation which time is measured and printed.
		 * 	@see getTime, time2str */
		void printTime(const char* prefix, long long &t0);

		/** @brief converts a time difference into a string.
		 *  @param time
		 *  		the time difference. The value CLOCKS_PER_SEC equates one second.
		 *  @return an useful string containing milliseconds, seconds, minutes, hours and/or days of the given time difference.
		 *  @see getTime, printTime */
		std::string time2str(long long time);
	/** @} */

	/** @brief this constant contains the biggest value that can be returned by random_LL() */
    const unsigned long long RANDOM_LL_MAX = std::mt19937_64::max();

	/** @name Random functions */
	/** @{ */
		/** @brief creates a random number.
		 *
		 *  if c++11 is enabled, a random generator of the c++11 standard is used,
		 *  else the rand()-function is called and its value is returned. */
        unsigned long long rand_();

		/** @brief sets the seed used by rand_().
		 *  @param seed
		 *  	the new seed for the random generator
		 *  srand(seed) is called in any way, also when c++11 is not enabled*/
		void srand_(unsigned seed);

		/** @brief creates a 64 bit long random number
		 *
		 *  the rand_()-function is called several times to create this number */
		unsigned long long random_LL();

		/** @brief creates a random number
		 *
		 *  @return a random number x, so that min_inclusive <= x < max_exclusive
		 *  @param min_inclusive
		 *  	the smallest value that can be returned
		 *  @param max_exclusive
		 *  	the smallest value that is bigger than min_inclusive and that cannot be returned */
		long long random_LL(long long min_inclusive, long long max_exclusive);

		/** @brief creates a random number between 0 and 1
		 *
		 *  @return a uniformly distributes random value between zero and one */
		double rnd();

		/** @brief creates a random number
		 *
		 *  @return a uniformly distributes random value between min and max */
		double rnd(double min, double max);

		/** @brief creates normal distributed random numbers
		 *
		 *  @param sigma
		 *  	the standard deviation of the created random numbers
		 *  @return a normal distributed random number with a standard deviation of sigma */
		double rnd_boxMuller(double sigma);
	/** @} */

	/** @name Random matrix creation */
	/** @{ */
		/** @brief Creates a random double matrix
		 *
		 *  @param d1
		 *  	first dimensionn of the matrix
		 *  @param d2
		 *  	second dimension of the matrix
		 *  @param min
		 *  	lowest possible value of an entry
		 *  @param max
		 *  	highest possible value of an entry
		 *  @return
		 *  	a (\a d1 x \a d2)-matrix with values of type double between \a min and \a max */
		Matrix<double> randomDoubleMatrix(int d1 = 5, int d2 = 5, double min = -1, double max = 1);

		/** @brief Creates a random int matrix
		 *
		 *  @param d1
		 *  	first dimensionn of the matrix
		 *  @param d2
		 *  	second dimension of the matrix
		 *  @param min
		 *  	smallest possible value of an entry
		 *  @param max
		 *  	greatest possible value of an entry
		 *  @return
		 *  	a (\a d1 x \a d2)-matrix with values of type int between \a min (inclusive) and \a max (inclusive) */
		Matrix<int> randomIntMatrix(int d1 = 1000, int d2 = 1000, int min = -100, int max = 100);

	/** @} */


	/** @brief Shows debug outputs, if true.
	 *
	 *  If this variable is true, additional debug messages (mainly time indications) are shown.
	 *  You can change the value of this variable during the runtime */
	extern bool debug_;

	/** @brief converts the input to the string "false" or "true"
	 *  @retval "false" if b is false.
	 *  @retval "true" if b is true. */
	const char* b2s(bool b);

	/** @brief Prints the given string and throws an exception
	 *  @param str
	 *  	the string that is printed to the standard error stream (cerr) */
	void err(const char* str);

	/** @brief splits the given string at the given character
	 *
	 *  @param c
	 *  	the character where the input string is splitted
	 *  @param str
	 *  	the string that is splitted
	 *  @return
	 *  	a vector with substrings of the input string
	 *
	 *  Example: split("ab,cd,,e,", ',') = {"ab","cd","","e",""} */
	std::vector<std::string> split(std::string str, char c);

	/** @brief checks, if the given file exists
	 *  @param path
	 *  	path to the file that will be checked
	 *  @retval false, if the file does not exist
	 *  @retval true, if the file exists */
	bool fileExists(std::string path);

	/** @brief Returns the parity of the number of 1 within the given bitsting
	 *
	 * 	@param input
	 * 		bitstring that will be analysed
	 * 	@param start
	 * 		Start value of the parity.
	 * 		This parameter must be zero or one, else the behaviour is undefined.
	 * 		The default value for this parameter is zero.
	 * 	@retval 0 if \a start plus the number of ones in \a input is even.
	 * 	@retval 1 if \a start plus the number of ones in \a input is odd. */
	int sequentialXor(unsigned long long input, int start = 0);

	/** @brief auxiliary function for the assert2-Macro
	 *
     *  Prints the given information to the standard error stream and throws an exception. */
	void assert_failed(const char* msg, const char* expr, const char* file, int line);

	/** @brief This structure analyses an array and stores the minimum, maximum, median and mean. */
	template <class T>
	struct MinMeanMax {
		/** @brief The minimum of the passed array */
		T min;

		/** @brief The arithmetic mean of the passed array */
		T mean;

		/** @brief The median of the passed array */
		T median;

		/** @brief The maximum of the passed array */
		T max;

		/** @brief Standard Constructor
		 *  @param begin
		 *  	Iterator pointing to the first element
		 *  @param end
		 *  	Iterator pointing to the element after the last element
		 *  @details The array from \a begin to \a end is analysed. */
		template <class Iterator>
		MinMeanMax(Iterator begin, Iterator end);

		/** @brief Default Constructor
		 *  @details No attribute is initialized by this constructor */
		MinMeanMax();
	};
}

/** @brief Prints the content of a std::vector.
 *
 *  The values are separated with the string ", " and the output is surrounded by square brackets. */
template <class T>
std::ostream& operator<<(std::ostream &strm, const std::vector<T> &v) {
	strm << "[";
	if (v.size() >= 1) strm << v[0];
	for (unsigned int i = 1; i < v.size(); i++) {
		strm << ", " << v[i];
	}
	strm << "]\n";
	return strm;
}

/** @def assert2(expr, msg)
 *  @brief macro for assertions
 *
 *  this macro works like standard macro "assert", however, it has an additonal parameter to print some extra information when the assertion fails.
 *  if NDEBUG is defined, this macro is ignored, else the given expression is checked.
 *  @param expr
 * 		the expression that will be checked.
 * 		if it is false, the program is aborted and some information like the file name, the line number and the evaluated msg parameter is printed
 *  @param msg
 *  	this parameter allows an easy advanced output (when the assertion fails). The content of the parameter is put into an stream using operator<<.
 *  	Therefore you can write for example "assert2(a == b, a << " != " << b);". If e.g. a = 3 and b = 4, the output contains the string "3 != 4". */
#ifdef NDEBUG
#define assert2(expr, msg) ((void)0);
#else
#include <sstream>
#define assert2(expr, msg) if(!(expr)) {std::stringstream ss; ss << msg; useful::assert_failed(ss.str().c_str(), #expr, __FILE__, __LINE__);}
#endif

/** @def SIZEOF(x)
 *  @brief Returns the size of an array
 *  @param x
 * 	 	The array whose size will be returned
 *  @returns
 *  	the size of the array
 *  @attention
 *  	This macro does not work for pointers that points to the first element of the array.
 *  @par Examples
 *  	If you define an array like this: <em> double arr[] = {1.2, 7.4, 10.3}; </em>\n
 *  	Then this will save its size: <em> int size = SIZEOF(arr) </em>
 */
#define SIZEOF(x) (sizeof((x))/sizeof((x)[0]))

// ===========================
// ===== IMPLEMENTIERUNG =====
// ===========================

template <class T>
template <class Iterator>
useful::MinMeanMax<T>::MinMeanMax(Iterator begin, Iterator end){
	min = std::numeric_limits<T>::max();
	max = std::numeric_limits<T>::min();
	mean = 0;

	int count = 0;
	Iterator it = begin;
	for (; it != end; ++it) {
		count++;
		T value = *it;
		mean += value;
		min = std::min(min, value);
		max = std::max(max, value);
	}

	mean /= count;

	std::sort(begin, end);
	median = begin[count/2];
	if (count % 2 == 0) {
		median += begin[count/2-1];
		median /= 2;
	}
}

template <class T>
useful::MinMeanMax<T>::MinMeanMax() {

}


#endif /* USEFUL_H_ */
